@@ -17,6 +17,9 @@
 #include <sys/time.h>
 
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -24,6 +27,7 @@
 #include "db.h"
 #include "spells.h"
 #include "screen.h"
+#include "constants.h"
 
 /* extern variables */
 extern struct room_data *world;
@@ -48,20 +52,8 @@ extern char *immlist;
 extern char *policies;
 extern char *handbook;
 extern char *buglist;
-extern char *dirs[];
-extern char *where[];
-extern char *color_liquid[];
-extern char *fullness[];
-extern char *connected_types[];
-extern char *class_abbrevs[];
-extern char *race_abbrevs[];
-extern char *room_bits[];
-extern char *spells[];
-extern char *sector_types[];
-extern char *affected_bits[];
-extern char *affected_bits2[];
-extern char *affected_bits3[];
-extern char *apply_types[];
+extern char *idealist;
+extern char *typolist;
 
 long find_class_bitvector(char arg);
 
@@ -281,6 +273,9 @@ void list_one_char(struct char_data * i, struct char_data * ch)
   if (AFF3_FLAGGED(i, AFF3_PASSDOOR))
 	strcat(buf, "(Translucent) ");
 
+  if (MOB_FLAGGED(i, MOB_ETHEREAL))
+	strcat(buf, "<ETHEREAL> ");
+
   if (IS_NPC(i) && i->player.long_descr && GET_POS(i) == GET_DEFAULT_POS(i)) {
     strcat(buf, i->player.long_descr);
     showflag = FALSE;
@@ -381,9 +376,9 @@ void list_char_to_char(struct char_data * list, struct char_data * ch)
 
   for (i = list; i; i = i->next_in_room)
     if (ch != i) {
-      if (CAN_SEE(ch, i))
+      if (CAN_SEE(ch, i) && (!MOB_FLAGGED(i, MOB_ETHEREAL) || GET_LEVEL(ch) > LVL_IMMORT))
 	list_one_char(i, ch);
-      else if (IS_DARK(ch->in_room) && !CAN_SEE_IN_DARK(ch))
+      else if (IS_DARK(ch->in_room) && !CAN_SEE_IN_DARK(ch) && !MOB_FLAGGED(i, MOB_ETHEREAL))
 	send_to_char("You see a pair of glowing red eyes looking your way.\r\n", ch);
     }
 }
@@ -482,7 +477,7 @@ ACMD(do_exits)
       } else {
 	sprintf(buf2, "  %-5s - ", dirs[door]);
 	if (IS_DARK(EXIT(ch, door)->to_room) && !CAN_SEE_IN_DARK(ch))
-	  strcat(buf2, "Too dark to tell\r\n");
+	  strcat(buf2, "Too dark to tell");
 	else {
 	  if (IS_SET(EXIT(ch, door)->exit_info, EX_HIDDEN)) {
   	    strcat(buf2, "(hidden) ");
@@ -529,6 +524,8 @@ void look_at_room(struct char_data * ch, int ignore_brief)
     sprintf(buf2, "[%5d] %s [%s: %s]", world[ch->in_room].number,
 	    world[ch->in_room].name, sector_types[world[ch->in_room].sector_type], buf);
     send_to_char(buf2, ch);
+    if (world[ch->in_room].tele != NULL)
+	send_to_char(" Teleport", ch);
   } else
     send_to_char(world[ch->in_room].name, ch);
 
@@ -602,9 +599,10 @@ void look_in_direction(struct char_data * ch, int dir)
         send_to_char(buf, ch);
       }
       if (EXITN(temp_room, dir)->to_room == NOWHERE) {
-	sprintf(buf, "Incorrect room number found in #%d -> #%d", world[temp_room].number, EXITN(temp_room, dir)->to_room_vnum);
+/*	sprintf(buf, "Incorrect room number found in #%d -> #%d", world[temp_room].number, EXITN(temp_room, dir)->to_room_vnum);
 	mudlog(buf, BRF, LVL_IMMORT, TRUE);
 	send_to_char("There's something wrong with this room, please report to an IMP.", ch);
+*/
 	return;
       }
       temp_room = EXITN(temp_room, dir)->to_room;
@@ -837,9 +835,9 @@ ACMD(do_gold)
   if (GET_GOLD(ch) == 0)
     send_to_char("You're broke!\r\n", ch);
   else if (GET_GOLD(ch) == 1)
-    send_to_char("You have one miserable little gold coin.\r\n", ch);
+    send_to_char("You have one miserable little coin.\r\n", ch);
   else {
-    sprintf(buf, "You have %d gold coins.\r\n", GET_GOLD(ch));
+    sprintf(buf, "You have %d coins.\r\n", GET_GOLD(ch));
     send_to_char(buf, ch);
   }
 }
@@ -856,8 +854,8 @@ ACMD(do_score)
   int i;
   char colorbuf[30];
 
- sprintf(buf, "Your name is  %s %s (level %d).\r\n",
-	  GET_NAME(ch), GET_TITLE(ch), GET_LEVEL(ch));
+ sprintf(buf, "Your name is  %s %s (level %d, Incarnation %d).\r\n",
+	  GET_NAME(ch), GET_TITLE(ch), GET_LEVEL(ch), GET_REINCARN(ch));
 
   sprintf(buf, "%sYou are a %d year old %s.", buf, 
 	GET_AGE(ch), pc_race_types[(int)GET_RACE(ch)]);
@@ -901,9 +899,14 @@ ACMD(do_score)
         GET_WIS(ch), GET_CHA(ch));
 
   sprintf(buf,
-        "%sYour armor class is %d/10, and your alignment is %d.\r\n",
-	 buf,  GET_AC(ch) + (10 *  dex_app[GET_DEX(ch)].defensive),
-         GET_ALIGNMENT(ch));
+        "%sYour armor class is %.1f, and your alignment is %s",
+	 buf, (float) GET_AC(ch) + (10 *  dex_app[GET_DEX(ch)].defensive) / 10,
+	 align_types[get_alignment_type(GET_ALIGNMENT(ch))]);
+
+  if (GET_LEVEL(ch) >= 20)
+	sprintf(buf,"%s (%d)\r\n", buf, GET_ALIGNMENT(ch));
+  else
+	strcat(buf, ".\r\n");
 
   if (GET_LEVEL(ch) >= 16) 
      sprintf(buf,
@@ -913,11 +916,11 @@ ACMD(do_score)
           GET_SAVE(ch, 0), GET_SAVE(ch, 1), GET_SAVE(ch, 2), GET_SAVE(ch, 3),
 	  GET_SAVE(ch, 4));
 
-  sprintf(buf, "%s\r\nYou have scored %d exp, and have %d gold coins.\r\n",
+  sprintf(buf, "%s\r\nYou have scored %d exp, and have %d coins.\r\n",
 	  buf, GET_EXP(ch), GET_GOLD(ch));
 
   if (!IS_NPC(ch)) {
-    if (GET_LEVEL(ch) < LVL_IMMORT)
+    if (GET_LEVEL(ch) < (LVL_IMMORT - 1))
       sprintf(buf, "%sYou need %d exp to reach your next level.\r\n\r\n", buf,
 	(titles[(int) GET_CLASS_NUM_FULL(ch)][GET_LEVEL(ch) + 1].exp) - GET_EXP(ch));
 
@@ -1055,8 +1058,6 @@ ACMD(do_time)
   char *suf;
   int weekday, day;
   extern struct time_info_data time_info;
-  extern const char *weekdays[];
-  extern const char *month_name[];
 
   sprintf(buf, "It is %d o'clock %s, on ",
 	  ((time_info.hours % 12 == 0) ? 12 : ((time_info.hours) % 12)),
@@ -1278,14 +1279,14 @@ ACMD(do_who)
     } else {
       num_can_see++;
       sprintf(buf1, "%s %s", GET_NAME(tch), GET_TITLE(tch));
-      sprintf(buf, "%s[%s %2d %s] %-30.30s",
+      sprintf(buf, "%s[%s %2d %s] %-38.38s",
 	      (GET_LEVEL(tch) >= LVL_IMMORT ? CCYEL(ch, C_SPR) : ""),
 	      RACE_ABBR(tch), GET_LEVEL(tch), CLASS_ABBR(tch), buf1);
 
       strcat(buf, "[");
-      if (PRF_FLAGGED(tch, PRF_BOUNTYHUNT))
+      if (PRF2_FLAGGED(tch, PRF2_BOUNTYHUNT))
 	strcat(buf, "B");
-      else if (PRF_FLAGGED(tch, PRF_ASSASSIN))
+      else if (PRF2_FLAGGED(tch, PRF2_ASSASSIN))
 	strcat(buf, "A");
       else
 	strcat(buf, "-");
@@ -1305,11 +1306,11 @@ ACMD(do_who)
 		strcat(buf, ".Unreg");
       }
       if (PRF2_FLAGGED(tch, PRF2_ARENA_RED))
-	strcat(buf, ".Team-R");
+	strcat(buf, ".Red");
       else if (PRF2_FLAGGED(tch, PRF2_ARENA_BLUE))
-	strcat(buf, ".Team-B");
+	strcat(buf, ".Blu");
       else
-	strcat(buf, ".NoTeam");
+	strcat(buf, ".---");
 
       strcat(buf, "](");
       if (PLR_FLAGGED(tch, PLR_THIEF))
@@ -1326,6 +1327,8 @@ ACMD(do_who)
       }
       strcat(buf, ")");
 
+      if (PRF2_FLAGGED(tch, PRF2_AFK))
+	strcat(buf, " (AFK)");
       if (PLR_FLAGGED(tch, PLR_MAILING))
 	strcat(buf, " (mailing)");
       else if (PLR_FLAGGED(tch, PLR_WRITING))
@@ -1421,9 +1424,9 @@ ACMD(do_users)
     }
   }				/* end while (parser) */
   strcpy(line,
-	 "Num Class   Name         State          Idl Login@   Site\r\n");
+	 "Num Class     Name         State          Idl Login@   Site\r\n");
   strcat(line,
-	 "--- ------- ------------ -------------- --- -------- ------------------------\r\n");
+	 "--- --------  ------------ -------------- --- -------- ------------------------\r\n");
   send_to_char(line, ch);
 
   one_argument(argument, arg);
@@ -1454,13 +1457,13 @@ ACMD(do_users)
 	continue;
 
       if (d->original)
-	sprintf(classname, "[%2d %s]", GET_LEVEL(d->original),
+	sprintf(classname, "[%2d  %s]", GET_LEVEL(d->original),
 		CLASS_ABBR(d->original));
       else
-	sprintf(classname, "[%2d %s]", GET_LEVEL(d->character),
+	sprintf(classname, "[%2d  %s]", GET_LEVEL(d->character),
 		CLASS_ABBR(d->character));
     } else
-      strcpy(classname, "   -   ");
+      strcpy(classname, "     -     ");
 
     timeptr = asctime(localtime(&d->login_time));
     timeptr += 11;
@@ -1475,7 +1478,7 @@ ACMD(do_users)
       sprintf(idletime, "%3d", d->character->char_specials.timer *
 	      SECS_PER_MUD_HOUR / SECS_PER_REAL_MIN);
     else
-      strcpy(idletime, "");
+      strcpy(idletime, "N/A");
 
     format = "%3d %-7s %-12s %-14s %-3s %-8s ";
 
@@ -1537,6 +1540,12 @@ ACMD(do_gen_ps)
   case SCMD_BUGLIST:
     page_string(ch->desc, buglist, 0);
     break;
+  case SCMD_IDEALIST:
+    page_string(ch->desc, idealist, 0);
+    break;
+  case SCMD_TYPOLIST:
+    page_string(ch->desc, typolist, 0);
+    break;
   case SCMD_POLICIES:
     page_string(ch->desc, policies, 0);
     break;
@@ -1821,24 +1830,29 @@ ACMD(do_toggle)
 	  "  Grats Channel: %-3s    "
 	  "   Repeat Comm.: %-3s\r\n"
 
-          "   Gold Display: %-3s    "
+          "   Coin Display: %-3s    "
 	  " Combat Channel: %-3s    "
           "      AutoTitle: %-3s\r\n"
 
           "     XP Display: %-3s    "
-	  "     Brief Mode: %-3s    "
+	  "   Chat Channel: %-3s    "
           "        AutoSac: %-3s\r\n"
 
           " Damage Display: %-3s    "
-	  "   Compact Mode: %-3s    "
+	  "  Arena Channel: %-3s    "
           "  Auto Show Dir: %-3s\r\n"
 
 	  "           Deaf: %-3s    "
 	  "         NoTell: %-3s    "
 	  " Auto Show Exit: %-3s\r\n"
 
+	  "     Brief Mode: %-3s    "
+	  "   Compact Mode: %-3s\r\n"
+
 	  "     Wimp Level: %-3s    "
-	  "    Color Level: %s",
+	  "    Color Level: %s\r\n"
+
+	  " Radio Channels:",
 
 	  ONOFF(PRF_FLAGGED(ch, PRF_DISPHP)),
 	  ONOFF(!PRF_FLAGGED(ch, PRF_NOGOSS)),
@@ -1857,21 +1871,26 @@ ACMD(do_toggle)
 	  ONOFF(!PLR_FLAGGED(ch, PLR_NOSETTITLE)),
 
 	  ONOFF(PRF_FLAGGED(ch, PRF_DISPXP)),
-	  ONOFF(PRF_FLAGGED(ch, PRF_BRIEF)),
+	  ONOFF(!PRF_FLAGGED(ch, PRF_NOCHAT)),
 	  ONOFF(PRF_FLAGGED(ch, PRF_AUTOSAC)),
 
 	  ONOFF(PRF_FLAGGED(ch, PRF_DISPDAM)),
-	  ONOFF(PRF_FLAGGED(ch, PRF_COMPACT)),
+	  ONOFF(!PRF_FLAGGED(ch, PRF_NOTEAM)),
 	  ONOFF(PRF_FLAGGED(ch, PRF_AUTODIR)),
 
 	  YESNO(PRF_FLAGGED(ch, PRF_DEAF)),
 	  ONOFF(PRF_FLAGGED(ch, PRF_NOTELL)),
 	  ONOFF(PRF_FLAGGED(ch, PRF_AUTOEXIT)),
 
+	  ONOFF(PRF_FLAGGED(ch, PRF_BRIEF)),
+	  ONOFF(PRF_FLAGGED(ch, PRF_COMPACT)),
+
 	  buf2,
 	  ctypes[COLOR_LEV(ch)]);
 
   send_to_char(buf, ch);
+  sprintbit(GET_CHANNEL(ch), channel_bits, buf);
+  send_to_char(buf, ch);
 }
 
 struct sort_struct {
@@ -1972,3 +1991,99 @@ ACMD(do_commands)
   strcat(buf, "\r\n");
   send_to_char(buf, ch);
 }
+
+/* functions and macros for 'scan' command */
+void list_scanned_chars(struct char_data * list, struct char_data * ch,
+ int distance, int door)
+{
+  const char *how_far[] = {
+    "close by",
+    "near by",
+    "not too far"
+    "a ways off",
+    "far off to the"
+  };
+
+  struct char_data *i;
+  int count = 0;
+  *buf = '\0';
+
+/* this loop is a quick, easy way to help make a grammatical sentence
+   (i.e., "You see x, x, y, and z." with commas, "and", etc.) */
+
+  for (i = list; i; i = i->next_in_room)
+
+/* put any other conditions for scanning someone in this if statement -
+   i.e., if (CAN_SEE(ch, i) && condition2 && condition3) or whatever */
+
+    if (CAN_SEE(ch, i))
+     count++;
+
+  if (!count)
+    return;
+
+  for (i = list; i; i = i->next_in_room) {
+
+/* make sure to add changes to the if statement above to this one also, using
+   or's to join them.. i.e., 
+   if (!CAN_SEE(ch, i) || !condition2 || !condition3) */
+
+    if (!CAN_SEE(ch, i))
+      continue; 
+    if (!*buf)
+      sprintf(buf, "You see %s", GET_NAME(i));
+    else 
+      sprintf(buf, "%s%s", buf, GET_NAME(i));
+    if (--count > 1)
+      strcat(buf, ", ");
+    else if (count == 1)
+      strcat(buf, " and ");
+    else {
+      sprintf(buf2, " %s %s.\r\n", how_far[distance], dirs[door]);
+      strcat(buf, buf2);      
+    }
+    
+  }
+  send_to_char(buf, ch);
+}
+
+ACMD(do_scan)
+{
+  /* >scan
+     You quickly scan the area.
+     You see John, a large horse and Frank close by north.
+     You see a small rabbit a ways off south.
+     You see a huge dragon and a griffon far off to the west.
+
+  */
+  int door;
+  
+  *buf = '\0';
+  
+  if (IS_AFFECTED(ch, AFF_BLIND)) {
+    send_to_char("You can't see a damned thing, you're blind!\r\n", ch);
+    return;
+  }
+  /* may want to add more restrictions here, too */
+  send_to_char("You quickly scan the area.\r\n", ch);
+  for (door = 0; door < NUM_OF_DIRS - 2; door++) /* don't scan up/down */
+    if (EXIT(ch, door) && EXIT(ch, door)->to_room != NOWHERE &&
+	!IS_SET(EXIT(ch, door)->exit_info, EX_CLOSED)) {
+      if (world[EXIT(ch, door)->to_room].people) {
+	list_scanned_chars(world[EXIT(ch, door)->to_room].people, ch, 0, door);
+      } else if (_2ND_EXIT(ch, door) && _2ND_EXIT(ch, door)->to_room != 
+		 NOWHERE && !IS_SET(_2ND_EXIT(ch, door)->exit_info, EX_CLOSED)) {
+   /* check the second room away */
+	if (world[_2ND_EXIT(ch, door)->to_room].people) {
+	  list_scanned_chars(world[_2ND_EXIT(ch, door)->to_room].people, ch, 1, door);
+	} else if (_3RD_EXIT(ch, door) && _3RD_EXIT(ch, door)->to_room !=
+		   NOWHERE && !IS_SET(_3RD_EXIT(ch, door)->exit_info, EX_CLOSED)) {
+	  /* check the third room */
+	  if (world[_3RD_EXIT(ch, door)->to_room].people) {
+	    list_scanned_chars(world[_3RD_EXIT(ch, door)->to_room].people, ch, 2, door);
+	  }
+
+	}
+      }
+    }                
+}
