@@ -12,6 +12,9 @@
 #include <string.h>
 
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -27,6 +30,7 @@ extern struct descriptor_data *descriptor_list;
 extern struct index_data *obj_index;
 extern int rev_dir[];
 extern char *dirs[];
+extern char *dirs2[];
 extern int movement_loss[];
 
 /* external functs */
@@ -185,6 +189,8 @@ int do_simple_move(struct char_data *ch, int dir, int need_specials_check)
   need_movement = (movement_loss[SECT(ch->in_room)] +
 		   movement_loss[SECT(EXIT(ch, dir)->to_room)]) >> 1;
 
+  if (AFF2_FLAGGED(ch, AFF2_HASTE))
+	need_movement = need_movement >> 1;
   if (GET_MOVE(ch) < need_movement && !IS_NPC(ch)) {
     if (need_specials_check && ch->master)
       send_to_char("You are too exhausted to follow.\r\n", ch);
@@ -225,16 +231,24 @@ int do_simple_move(struct char_data *ch, int dir, int need_specials_check)
   }
 
   if (!IS_AFFECTED(ch, AFF_SNEAK)) {
-    sprintf(buf2, "$n leaves %s.", dirs[dir]);
+    if (AFF2_FLAGGED(ch, AFF2_HASTE))
+	    sprintf(buf2, "$n leaves %s quickly.", dirs[dir]);
+    else
+	    sprintf(buf2, "$n leaves %s.", dirs[dir]);
     act(buf2, TRUE, ch, 0, 0, TO_ROOM);
   }
+
   was_in = ch->in_room;
   char_from_room(ch);
   char_to_room(ch, world[was_in].dir_option[dir]->to_room);
 
-  if (!IS_AFFECTED(ch, AFF_SNEAK))
-    act("$n has arrived.", TRUE, ch, 0, 0, TO_ROOM);
-
+  if (!IS_AFFECTED(ch, AFF_SNEAK)) {
+    if (AFF2_FLAGGED(ch, AFF2_HASTE))
+	    sprintf(buf2, "$n arrives quickly from %s.", dirs2[rev_dir[dir]]);
+    else
+	    sprintf(buf2, "$n arrives from %s.", dirs2[rev_dir[dir]]);
+    act(buf2, TRUE, ch, 0, 0, TO_ROOM);
+  }
   if (ch->desc != NULL)
     look_at_room(ch, 0);
 
@@ -391,7 +405,7 @@ const int flags_door[] =
 
 void do_doorcmd(struct char_data *ch, struct obj_data *obj, int door, int scmd)
 {
-  int other_room;
+  int other_room = 0;
   struct room_direction_data *back = 0;
 
   sprintf(buf, "$n %ss ", cmd_door[scmd]);
@@ -484,7 +498,7 @@ int ok_pick(struct char_data *ch, int keynum, int pickproof, int scmd)
 
 ACMD(do_gen_door)
 {
-  int door, keynum;
+  int door = 0, keynum;
   char type[MAX_INPUT_LENGTH], dir[MAX_INPUT_LENGTH];
   struct obj_data *obj = NULL;
   struct char_data *victim = NULL;
@@ -764,16 +778,6 @@ ACMD(do_follow)
     return;
   }
 
-  if (GET_LEVEL(leader) > ( 5 + GET_LEVEL(ch) )) {
-    act("You would feel too much like a lacky if you where to follow $M.", FALSE, ch, 0, leader, TO_CHAR);
-    return;
-  }
-
-  if (GET_LEVEL(leader) < ( GET_LEVEL(ch) - 5)) {
-    act("You would feel foolish following $M.", FALSE, ch, 0, leader, TO_CHAR);
-    return;
-  }
-
   if (IS_AFFECTED(ch, AFF_CHARM) && (ch->master)) {
     act("But you only feel like following $N!", FALSE, ch, 0, ch->master, TO_CHAR);
   } else {			/* Not Charmed follow person */
