@@ -19,6 +19,9 @@
 #include "handler.h"
 #include "db.h"
 #include "spells.h"
+#include "objs.h"
+#include "class.h"
+#include "rooms.h"
 
 /* extern variables */
 extern struct str_app_type str_app[];
@@ -336,13 +339,13 @@ void perform_drop_gold(struct char_data * ch, int amount,
       WAIT_STATE(ch, PULSE_VIOLENCE);	/* to prevent coin-bombing */
       obj = create_money(amount);
       if (mode == SCMD_DONATE) {
-	send_to_char("You throw some gold into the air where it disappears in a puff of smoke!\r\n", ch);
-	act("$n throws some gold into the air where it disappears in a puff of smoke!",
+	send_to_char("You throw some coin into the air where it disappears in a puff of smoke!\r\n", ch);
+	act("$n throws some coins into the air where it disappears in a puff of smoke!",
 	    FALSE, ch, 0, 0, TO_ROOM);
 	obj_to_room(obj, RDR);
 	act("$p suddenly appears in a puff of orange smoke!", 0, 0, obj, 0, TO_ROOM);
       } else {
-	send_to_char("You drop some gold.\r\n", ch);
+	send_to_char("You drop some coins.\r\n", ch);
 	sprintf(buf, "$n drops %s.", money_desc(amount));
 	act(buf, FALSE, ch, 0, 0, TO_ROOM);
 	obj_to_room(obj, ch->in_room);
@@ -351,7 +354,7 @@ void perform_drop_gold(struct char_data * ch, int amount,
       sprintf(buf, "$n drops %s which disappears in a puff of smoke!",
 	      money_desc(amount));
       act(buf, FALSE, ch, 0, 0, TO_ROOM);
-      send_to_char("You drop some gold which disappears in a puff of smoke!\r\n", ch);
+      send_to_char("You drop some coins which disappears in a puff of smoke!\r\n", ch);
     }
     GET_GOLD(ch) -= amount;
   }
@@ -573,7 +576,7 @@ void perform_give_gold(struct char_data * ch, struct char_data * vict,
   }
   send_to_char(OK, ch);
   mprog_bribe_trigger(vict, ch, amount);
-  sprintf(buf, "$n gives you %d gold coins.", amount);
+  sprintf(buf, "$n gives you %d coins.", amount);
   MOBTrigger = FALSE;
   act(buf, FALSE, ch, 0, vict, TO_VICT);
   sprintf(buf, "$n gives %s to $N.", money_desc(amount));
@@ -725,13 +728,13 @@ ACMD(do_drink)
     send_to_char("You have to be holding that to drink from it.\r\n", ch);
     return;
   }
-  if ((GET_COND(ch, DRUNK) > 10) && (GET_COND(ch, THIRST) > 0)) {
+  if ((GET_COND(ch, DRUNK) > 30) && (GET_COND(ch, THIRST) > 0)) {
     /* The pig is drunk */
     send_to_char("You can't seem to get close enough to your mouth.\r\n", ch);
     act("$n tries to drink but misses $s mouth!", TRUE, ch, 0, 0, TO_ROOM);
     return;
   }
-  if ((GET_COND(ch, FULL) > 20) && (GET_COND(ch, THIRST) > 0)) {
+  if ((GET_COND(ch, FULL) > 60) && (GET_COND(ch, THIRST) > 0)) {
     send_to_char("Your stomach can't contain anymore!\r\n", ch);
     return;
   }
@@ -878,8 +881,8 @@ ACMD(do_pour)
 {
   char arg1[MAX_INPUT_LENGTH];
   char arg2[MAX_INPUT_LENGTH];
-  struct obj_data *from_obj;
-  struct obj_data *to_obj;
+  struct obj_data *from_obj = NULL;
+  struct obj_data *to_obj = NULL;
   int amount;
 
   two_arguments(argument, arg1, arg2);
@@ -1049,7 +1052,7 @@ void wear_message(struct char_data * ch, struct obj_data * obj, int where)
     {"$n wears $p on $s arms.",
     "You wear $p on your arms."},
 
-    {"$n straps $p around $s arm as a shield.",
+    {"$n wears $p in $s arm as a shield.",
     "You start to use $p as a shield."},
 
     {"$n wears $p about $s body.",
@@ -1067,6 +1070,9 @@ void wear_message(struct char_data * ch, struct obj_data * obj, int where)
     {"$n wields $p.",
     "You wield $p."},
 
+    {"$n wields $p in $s off-hand.",
+     "You wield $p in your off-hand."},
+
     {"$n grabs $p.",
     "You grab $p."},
 
@@ -1075,8 +1081,8 @@ void wear_message(struct char_data * ch, struct obj_data * obj, int where)
 
     {"$n puts $p on $s face.",
      "You put $p on your face."}
-  };
 
+  };
   act(wear_messages[where][0], TRUE, ch, obj, 0, TO_ROOM);
   act(wear_messages[where][1], FALSE, ch, obj, 0, TO_CHAR);
 }
@@ -1091,7 +1097,8 @@ void perform_wear(struct char_data * ch, struct obj_data * obj, int where)
     ITEM_WEAR_NECK, ITEM_WEAR_BODY, ITEM_WEAR_HEAD, ITEM_WEAR_LEGS,
     ITEM_WEAR_FEET, ITEM_WEAR_HANDS, ITEM_WEAR_ARMS, ITEM_WEAR_SHIELD,
     ITEM_WEAR_ABOUT, ITEM_WEAR_WAIST, ITEM_WEAR_WRIST, ITEM_WEAR_WRIST,
-    ITEM_WEAR_WIELD, ITEM_WEAR_TAKE, ITEM_WEAR_BACK, ITEM_WEAR_FACE};
+    ITEM_WEAR_WIELD, ITEM_WEAR_WIELD, ITEM_WEAR_TAKE, ITEM_WEAR_BACK,
+    ITEM_WEAR_FACE};
 
   char *already_wearing[] = {
     "You're already using a light.\r\n",
@@ -1105,12 +1112,13 @@ void perform_wear(struct char_data * ch, struct obj_data * obj, int where)
     "You're already wearing something on your feet.\r\n",
     "You're already wearing something on your hands.\r\n",
     "You're already wearing something on your arms.\r\n",
-    "You're already using a shield.\r\n",
+    "You're already using something with your shield hand.\r\n",
     "You're already wearing something about your body.\r\n",
     "You already have something around your waist.\r\n",
     "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT.\r\n",
     "You're already wearing something around both of your wrists.\r\n",
     "You're already wielding a weapon.\r\n",
+    "You're already wielding a weapon in your off-hand.\r\n",
     "You're already holding something.\r\n",
     "You're already wearing something on your back.\r\n",
     "You already have something on your face.\r\n"
@@ -1122,16 +1130,33 @@ void perform_wear(struct char_data * ch, struct obj_data * obj, int where)
     return;
   }
   /* for neck, finger, and wrist, try pos 2 if pos 1 is already full */
-  if ((where == WEAR_FINGER_R) || (where == WEAR_NECK_1) || (where == WEAR_WRIST_R))
+  if ((where == WEAR_FINGER_R) || (where == WEAR_NECK_1) || (where == WEAR_WRIST_R) || (where == WEAR_WIELD))
     if (GET_EQ(ch, where))
       where++;
 
+  if (where == WEAR_DUALWIELD) {
+	if (IS_OBJ_STAT2(obj, ITEM2_2HANDED)) {
+		act("You cannot dual wield $p because it's a two-handed weapon.", FALSE, ch, obj, 0, TO_CHAR);
+		return;
+        } else if (GET_SKILL(ch, SKILL_DUAL_WIELD) <= 50) {
+		act("You cannot dual wield $p yet.", FALSE, ch, obj, 0, TO_CHAR);
+		return;
+        } else if ((obj2 = GET_EQ(ch, WEAR_SHIELD))) {
+		act("You're already using $p.", FALSE, ch, obj2, 0, TO_CHAR);
+		return;
+	}
+  }
+		
   if (GET_EQ(ch, where)) {
     send_to_char(already_wearing[where], ch);
     return;
   }
   if (where == WEAR_SHIELD && (obj2 = GET_EQ(ch, WEAR_WIELD)) && IS_OBJ_STAT2(obj2, ITEM2_2HANDED)) {
-    send_to_char("You're using a two handed weapon!\r\n", ch);
+    act("You are already using $p, which is a two handed weapon!", FALSE, ch, obj2, 0, TO_CHAR);
+    return;
+  }
+  if (where == WEAR_SHIELD && (obj2 = GET_EQ(ch, WEAR_DUALWIELD))) {
+    act("You're already using $p... remove it first.", FALSE, ch, obj2, 0, TO_CHAR);
     return;
   }
   if (IS_OBJ_STAT2(obj, ITEM2_ENGRAVED) && (strcmp(obj->owner_name, GET_NAME(ch)) != 0)) {
@@ -1167,6 +1192,7 @@ int find_eq_pos(struct char_data * ch, struct obj_data * obj, char *arg)
     "!RESERVED!",
     "!RESERVED!",
     "!RESERVED!",
+    "!RESERVED!",
     "back",
     "face",
     "\n"
@@ -1285,8 +1311,6 @@ ACMD(do_wield)
   }
 }
 
-
-
 ACMD(do_grab)
 {
   struct obj_data *obj;
