@@ -13,6 +13,9 @@
 #include <stdlib.h>
 
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"	
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -214,6 +217,7 @@ ACMD(do_slay)
 ACMD(do_backstab)
 {
   struct char_data *vict;
+  struct obj_data *obj;
   int percent, prob;
 
   one_argument(argument, buf);
@@ -226,11 +230,11 @@ ACMD(do_backstab)
     send_to_char("How can you sneak up on yourself?\r\n", ch);
     return;
   }
-  if (!GET_EQ(ch, WEAR_WIELD)) {
+  if (!(obj = GET_EQ(ch, WEAR_WIELD))) {
     send_to_char("You need to wield a weapon to make it a success.\r\n", ch);
     return;
   }
-  if (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_PIERCE - TYPE_HIT) {
+  if (GET_OBJ_VAL(obj, 3) != TYPE_PIERCE - TYPE_HIT) {
     send_to_char("Only piercing weapons can be used for backstabbing.\r\n", ch);
     return;
   }
@@ -260,7 +264,7 @@ ACMD(do_backstab)
   prob = GET_SKILL(ch, SKILL_BACKSTAB);
 
   if (AWAKE(vict) && (percent > prob))
-    damage(ch, vict, 0, SKILL_BACKSTAB);
+    damage(ch, vict, 0, SKILL_BACKSTAB, obj);
   else
     hit(ch, vict, SKILL_BACKSTAB);
   WAIT_STATE(ch, PULSE_VIOLENCE * 3);
@@ -362,6 +366,7 @@ ACMD(do_bash)
 {
   struct char_data *vict;
   int percent, prob;
+  struct obj_data *obj;
 
   one_argument(argument, arg);
 
@@ -381,8 +386,8 @@ ACMD(do_bash)
     send_to_char("Aren't we funny today...\r\n", ch);
     return;
   }
-  if (!GET_EQ(ch, WEAR_WIELD)) {
-    send_to_char("You need to wield a weapon to make it a success.\r\n", ch);
+  if (!(obj = GET_EQ(ch, WEAR_SHIELD))) {
+    send_to_char("You need to wear a shield to make it a success.\r\n", ch);
     return;
   }
   if (ROOM_FLAGGED(ch->in_room, ROOM_PEACEFUL)) {
@@ -401,10 +406,10 @@ ACMD(do_bash)
     percent = 101;
 
   if (percent > prob) {
-    damage(ch, vict, 0, SKILL_BASH);
+    damage(ch, vict, 0, SKILL_BASH, obj);
     GET_POS(ch) = POS_SITTING;
   } else {
-    damage(ch, vict, 1, SKILL_BASH);
+    damage(ch, vict, 1, SKILL_BASH, obj);
     GET_POS(vict) = POS_SITTING;
     WAIT_STATE(vict, PULSE_VIOLENCE);
   }
@@ -471,6 +476,7 @@ ACMD(do_kick)
 {
   struct char_data *vict;
   int percent, prob;
+  struct obj_data *obj;
 
   if (!(IS_WARRIOR(ch) || IS_BARBARIAN(ch) || IS_SAMURAI(ch))) {
     send_to_char("You'd better leave all the martial arts to fighters.\r\n", ch);
@@ -480,6 +486,11 @@ ACMD(do_kick)
     send_to_char("This room has a nice peaceful feeling.\r\n", ch);
     return;
   }
+  if (!(obj = GET_EQ(ch, WEAR_FEET))) {
+    send_to_char("You need some kind of footwear to make it a success.\r\n", ch);
+    return;
+  }
+
   one_argument(argument, arg);
 
   if (!(vict = get_char_room_vis(ch, arg))) {
@@ -504,9 +515,9 @@ ACMD(do_kick)
   prob = GET_SKILL(ch, SKILL_KICK);
 
   if (percent > prob) {
-    damage(ch, vict, 0, SKILL_KICK);
+    damage(ch, vict, 0, SKILL_KICK, obj);
   } else
-    damage(ch, vict, GET_LEVEL(ch) >> 1, SKILL_KICK);
+    damage(ch, vict, GET_LEVEL(ch) >> 1, SKILL_KICK, obj);
   WAIT_STATE(ch, PULSE_VIOLENCE * 3);
 }
 
@@ -548,17 +559,23 @@ ACMD(do_fire)
 	return;
   }
   if (subcmd == SCMD_THROW) {
-	if (!obj) {
-		if (!(obj = GET_EQ(ch, WEAR_HOLD))) {
+	obj = GET_EQ(ch, WEAR_DUALWIELD);
+	if (!obj && !GET_EQ(ch, WEAR_WIELD)) {
+		if (!GET_EQ(ch, WEAR_HOLD)) {
 			send_to_char(fire_msg[0], ch);
 			return;
 		}
-		obj = unequip_char(ch, WEAR_HOLD);
-	} else {
+		else
+		  obj = unequip_char(ch, WEAR_HOLD);
+	} else if (!obj) {
 		obj = unequip_char(ch, WEAR_WIELD);
+	} else {
+		obj = unequip_char(ch, WEAR_DUALWIELD);
 	}
-
-	distance = 1;
+	if (GET_OBJ_TYPE(obj) == ITEM_MISSILE)
+		distance = 2;
+	else 
+		distance = 1;
   } else { 	/* fire, not throw */
 	if (!obj || GET_OBJ_TYPE(obj) != ITEM_FIREWEAPON) {
 		send_to_char(fire_msg[1], ch);
@@ -595,16 +612,16 @@ void fire_in_direction(struct char_data *ch, struct obj_data *obj, int dir, int
 	 !IS_SET(EXITN(temp_room, dir)->exit_info, EX_CLOSED)) {
 	temp_room = EXITN(temp_room, dir)->to_room;
 	if (ROOM_FLAGGED(temp_room, ROOM_MURKY))
-	   contin = FALSE;
-	else if (!ROOM_FLAGGED(temp_room, ROOM_PEACEFUL)) 
+	   distance--;
+	else
 	   contin = fire_at_char(ch, world[temp_room].people, obj, dir);
   }
-
   if (IS_OBJ_STAT2(obj, ITEM2_RETURNING)) {
 	  act(fire_msg[10], FALSE, ch, obj, 0, TO_CHAR);
 	  act(fire_msg[9], FALSE, ch, obj, 0, TO_ROOM);
   	  if ((vict = world[temp_room].people)) {
 	    act(fire_msg[9], FALSE, ch, obj, vict, TO_VICT);
+	    act(fire_msg[9], FALSE, ch, obj, vict, TO_NOTVICT);
 	  }
 	  obj_to_char(obj, ch);
   } else {
