@@ -17,6 +17,9 @@
 #include <sys/stat.h>
 
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -25,16 +28,14 @@
 #include "spells.h"
 #include "screen.h"
 #include "house.h"
+#include "constants.h"
 
 /* extern variables */
-extern struct str_app_type str_app[];
 extern struct room_data *world;
 extern struct descriptor_data *descriptor_list;
 extern struct room_data *world;
-extern struct dex_skill_type dex_app_skill[];
 extern struct spell_info_type spell_info[];
 extern struct index_data *mob_index;
-extern char *class_abbrevs[];
 extern struct title_type titles[NUM_CLASSES][LVL_IMPL + 1];
 
 void write_aliases(struct char_data *ch);
@@ -61,8 +62,8 @@ ACMD(do_quit)
   else if (GET_POS(ch) < POS_STUNNED) {
     send_to_char("You die before your time...\r\n", ch);
     die(ch, NULL); }
-  else if (IS_SET(PRF_FLAGS(ch), PRF_NOQUIT))
-    send_to_char("You have Quit-Intercept on.  Save and Rent or type 'noquit' then quit again.\r\n", ch);
+  else if (IS_SET(PRF2_FLAGS(ch), PRF2_NOQUIT))
+    send_to_char("You have Quit-Intercept on.  Toggle with 'noquit'  You'll lose your equip if you quit.\r\n", ch);
   else {
     if (!GET_INVIS_LEV(ch))
       act("$n has left the game.", TRUE, ch, 0, 0, TO_ROOM);
@@ -136,6 +137,8 @@ ACMD(do_sneak)
   af.modifier = 0;
   af.location = APPLY_NONE;
   af.bitvector = AFF_SNEAK;
+  af.bitvector2 = 0;
+  af.bitvector3 = 0;
   affect_to_char(ch, &af);
 }
 
@@ -261,18 +264,18 @@ ACMD(do_steal)
       ohoh = TRUE;
       act("Oops..", FALSE, ch, 0, 0, TO_CHAR);
       act("You discover that $n has $s hands in your wallet.", FALSE, ch, 0, vict, TO_VICT);
-      act("$n tries to steal gold from $N.", TRUE, ch, 0, vict, TO_NOTVICT);
+      act("$n tries to steal coins from $N.", TRUE, ch, 0, vict, TO_NOTVICT);
     } else {
-      /* Steal some gold coins */
+      /* Steal some coins */
       gold = (int) ((GET_GOLD(vict) * number(1, 10)) / 100);
       gold = MIN(1782, gold);
       if (gold > 0) {
 	GET_GOLD(ch) += gold;
 	GET_GOLD(vict) -= gold;
-	sprintf(buf, "Bingo!  You got %d gold coins.\r\n", gold);
+	sprintf(buf, "Bingo!  You got %d coins.\r\n", gold);
 	send_to_char(buf, ch);
       } else {
-	send_to_char("You couldn't get any gold...\r\n", ch);
+	send_to_char("You couldn't get any coins...\r\n", ch);
       }
     }
   }
@@ -351,6 +354,16 @@ int perform_group(struct char_data *ch, struct char_data *vict)
   if (IS_AFFECTED(vict, AFF_GROUP) || !CAN_SEE(ch, vict))
     return 0;
 
+  if (GET_LEVEL(vict) > ( 5 + GET_LEVEL(ch) )) {
+    act("You would feel too much like a lacky if you were in a group with $M.", FALSE, ch, 0, vict, TO_CHAR);
+    return 0;
+  }
+
+  if (!IS_MOB(vict) && GET_LEVEL(vict) < ( GET_LEVEL(ch) - 5)) {
+    act("You would feel foolish leading $M.", FALSE, ch, 0, vict, TO_CHAR);
+    return 0;
+  }
+
   SET_BIT(AFF_FLAGS(vict), AFF_GROUP);
   if (ch != vict)
     act("$N is now a member of your group.", FALSE, ch, 0, vict, TO_CHAR);
@@ -445,8 +458,6 @@ ACMD(do_group)
   }
 }
 
-
-
 ACMD(do_ungroup)
 {
   struct follow_type *f, *next_fol;
@@ -527,16 +538,20 @@ ACMD(do_report)
   }
   if (ch->player_specials->saved.killed_by_player) {
 	temp = (float) (ch->player_specials->saved.killed_player / ch->player_specials->saved.killed_by_player);  
-	sprintf(buf, "%s reports:  Player-Kill/Killed Ratio: %4.2f:1\r\n", GET_NAME(ch), temp);
+	sprintf(buf, "%s reports:  Incarnation %d.  Player-Kill/Killed Ratio: %4.2f:1", GET_NAME(ch),
+		GET_REINCARN(ch), temp);
 	act(buf, FALSE, ch, 0, 0, TO_ROOM);
-	sprintf(buf, "Your player-kill/killed ratio is: %4.2f:1\r\n", temp);
+	sprintf(buf, "You have reincarnated %d times.  Your player-kill/killed ratio is: %4.2f:1",
+		GET_REINCARN(ch), temp);
 	act(buf, FALSE, ch, 0, 0, TO_CHAR);
   }
   else {
 	temp = ch->player_specials->saved.killed_player;
-	sprintf(buf, "%s reports:  Player-Kill/Killed Ratio: %d:0\r\n", GET_NAME(ch), (int) temp);
+	sprintf(buf, "%s reports:  Incarnation %d.  Player-Kill/Killed Ratio: %d:0", GET_NAME(ch), 
+		GET_REINCARN(ch), (int) temp);
 	act(buf, FALSE, ch, 0, 0, TO_ROOM);
-	sprintf(buf, "Your player-kill/killed ratio is: %d:0\r\n", (int) temp);
+	sprintf(buf, "You have reincarnated %d times.  Your player-kill/killed ratio is: %d:0", 
+		GET_REINCARN(ch), (int) temp);
 	act(buf, FALSE, ch, 0, 0, TO_CHAR);
   }
 }
@@ -561,7 +576,7 @@ ACMD(do_split)
       return;
     }
     if (amount > GET_GOLD(ch)) {
-      send_to_char("You don't seem to have that much gold to split.\r\n", ch);
+      send_to_char("You don't seem to have that much coins to split.\r\n", ch);
       return;
     }
     k = (ch->master ? ch->master : ch);
@@ -580,7 +595,7 @@ ACMD(do_split)
     if (num && IS_AFFECTED(ch, AFF_GROUP))
       share = amount / num;
     else {
-      send_to_char("With whom do you wish to share your gold?\r\n", ch);
+      send_to_char("With whom do you wish to share your coins?\r\n", ch);
       return;
     }
 
@@ -724,7 +739,7 @@ ACMD(do_display)
 {
   int i;
 
-  if (IS_NPC(ch)) {
+  if (ch->desc == NULL) {
     send_to_char("Mosters don't need displays.  Go away.\r\n", ch);
     return;
   }
@@ -854,8 +869,8 @@ ACMD(do_gen_tog)
     "You are now deaf to auctions.\r\n"},
     {"You can now hear shouts.\r\n",
     "You are now deaf to shouts.\r\n"},
-    {"You can now hear gossip/chat.\r\n",
-    "You are now deaf to gossip/chat.\r\n"},
+    {"You can now hear global gossip.\r\n",
+    "You are now deaf to global gossip.\r\n"},
     {"You can now hear the congratulation messages.\r\n",
     "You are now deaf to the congratulation messages.\r\n"},
     {"You can now hear the Wiz-channel.\r\n",
@@ -880,9 +895,16 @@ ACMD(do_gen_tog)
     "Group fire enabled (Will now AutoSac too).\r\n"},
     {"AutoTitle enabled.  Title will change on leveling\r\n",
     "AutoTitle disabled.  Title will not change on leveling\r\n"},
-    {"You can now hear the War/Arena encode/broadcast channel.\r\n",
-    "You are now deaf to the War/Arena encode/broadcast channel.\r\n"}
-
+    {"You can now hear encoded messages.\r\n",
+    "You are now deaf to encoded messages.\r\n"},
+    {"Yay!  You're back!  You don't know what you missed while you were gone.",
+     "Okay, love you bye bye!  Come back to the keyboard soon, 'cuz we'll miss ya!"},
+    {"You can now hear your race-specific chatter.",
+     "You are now deaf to race-specific chatter."},
+    {"You can now hear team broadcasts.",
+     "You are now deaf to team broadcasts."},
+    {"You can now hear inner-clan discussions.",
+     "You are now deaf inner-clan discussions."}
   };
 
 
@@ -939,7 +961,7 @@ ACMD(do_gen_tog)
     result = PRF_TOG_CHK(ch, PRF_AUTOEXIT);
     break;
   case SCMD_NOQUIT:
-    result = PRF_TOG_CHK(ch, PRF_NOQUIT);
+    result = PRF2_TOG_CHK(ch, PRF2_NOQUIT);
     break;
   case SCMD_AUTODIR:
     result = PRF_TOG_CHK(ch, PRF_AUTODIR);
@@ -953,6 +975,18 @@ ACMD(do_gen_tog)
   case SCMD_NOWAR:
     result = PRF_TOG_CHK(ch, PRF_NOWAR);
     break;
+  case SCMD_AFK:
+    result = PRF2_TOG_CHK(ch, PRF2_AFK);
+    break;
+  case SCMD_NOCHAT:
+    result = PRF_TOG_CHK(ch, PRF_NOCHAT);
+    break;
+  case SCMD_NOTEAM:
+    result = PRF_TOG_CHK(ch, PRF_NOTEAM);
+    break;
+  case SCMD_NOCLAN:
+    result = PRF_TOG_CHK(ch, PRF_NOCLAN);
+    break;
   default:
     log("SYSERR: Unknown subcmd in do_gen_toggle");
     return;
@@ -966,3 +1000,35 @@ ACMD(do_gen_tog)
 
   return;
 }
+
+
+ACMD(do_channel)
+{
+  char buf1[256];
+  int channel = 0;
+
+  one_argument(argument, buf);
+
+  if (!*buf) {
+    send_to_char("You are tuned to the following channels: ", ch);
+    sprintbit(GET_CHANNEL(ch), channel_bits, buf1);
+    send_to_char(buf1, ch);
+    return;
+  }
+
+  channel = atoi(buf);
+
+  if (channel == 0) {
+	GET_CHANNEL(ch) = 0;
+	send_to_char("Untoggling all channels.\r\n", ch);
+        return;
+  }
+
+  if (channel < 0 || channel > 32) {
+	send_to_char("Sorry, choose a channel between 1 to 32, or 0 to clear channels.\r\n", ch);
+	return;
+  }
+
+  SET_BIT(GET_CHANNEL(ch), 1 << (channel -1));
+  return;
+}
