@@ -16,6 +16,9 @@
 #include <sys/types.h>
 
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
+#include "rooms.h"
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -24,6 +27,7 @@
 #include "spells.h"
 #include "house.h"
 #include "screen.h"
+#include "constants.h"
 
 /*   external vars  */
 extern FILE *player_fl;
@@ -34,8 +38,6 @@ extern struct descriptor_data *descriptor_list;
 extern struct title_type titles[NUM_CLASSES][LVL_IMPL + 1];
 extern struct index_data *mob_index;
 extern struct index_data *obj_index;
-extern struct int_app_type int_app[36];
-extern struct wis_app_type wis_app[36];
 extern struct zone_data *zone_table;
 extern int top_of_zone_table;
 extern int restrict;
@@ -44,40 +46,10 @@ extern int top_of_mobt;
 extern int top_of_objt;
 extern int top_of_p_table;
 
-/* for objects */
-extern char *item_types[];
-extern char *wear_bits[];
-extern char *extra_bits[];
-extern char *extra_bits2[];
-extern char *drinks[];
-
-/* for rooms */
-extern char *dirs[];
-extern char *room_bits[];
-extern char *exit_bits[];
-extern char *sector_types[];
-
-/* for chars */
 extern char *spells[];
-extern char *equipment_types[];
-extern char *affected_bits[];
-extern char *affected_bits2[];
-extern char *affected_bits3[];
-extern char *apply_types[];
+extern char *pc_race_types[];
 extern char *pc_class_types[];
-extern char *npc_class_types[];
-extern char *action_bits[];
-extern char *action_bits2[];
-extern char *action_bits3[];
-extern char *player_bits[];
-extern char *player_bits2[];
-extern char *player_bits3[];
-extern char *preference_bits[];
-extern char *preference_bits2[];
-extern char *position_types[];
-extern char *connected_types[];
-
-
+extern struct attack_hit_type attack_hit_text[];
 
 ACMD(do_echo)
 {
@@ -163,8 +135,8 @@ sh_int find_target_room(struct char_data * ch, char *rawroomstr)
     return NOWHERE;
   }
 
-  /* a location has been found -- if you're < GRGOD, check restrictions. */
-  if (GET_LEVEL(ch) < LVL_GRGOD) {
+  /* a location has been found -- if you're < GOD, check restrictions. */
+  if (GET_LEVEL(ch) < LVL_GOD) {
     if (ROOM_FLAGGED(location, ROOM_GODROOM)) {
       send_to_char("You are not godly enough to use that room!\r\n", ch);
       return NOWHERE;
@@ -378,6 +350,44 @@ void do_stat_room(struct char_data * ch)
     strcat(buf, CCNRM(ch, C_NRM));
     send_to_char(strcat(buf, "\r\n"), ch);
   }
+  if (IS_SET(rm->room_flags, ROOM_BROADCAST)) {
+	if (rm->broad == NULL) {
+		sprintf(buf, "Room does not have a broadcast struct... ERROR");
+	} else {
+	  send_to_char("Broadcasts to:\r\n", ch);
+	  if (rm->broad->channel != 0) {
+		sprintf(buf, "   Channel : %s\r\n", channel_bits[rm->broad->channel - 1]);
+		send_to_char(buf, ch);
+          }
+          if (rm->broad->targ1 != 0 && real_room(rm->broad->targ1) != -1) {
+		sprintf(buf, "   Room    : %s (%d)\t\n",
+			world[real_room(rm->broad->targ1)].name,
+			rm->broad->targ1);
+		send_to_char(buf, ch);
+          }
+          if (rm->broad->targ2 != 0 && real_room(rm->broad->targ2) != -1) {
+		sprintf(buf, "   Room %s (%d)\t\n",
+			world[real_room(rm->broad->targ2)].name,
+			rm->broad->targ2);
+		send_to_char(buf, ch);
+          }
+        }
+  }
+  if (rm->tele != NULL) {
+	sprintf(buf, "Teleports every %d*10 seconds to %20s (Room %d)\r\n",
+		rm->tele->time, world[real_room(rm->tele->targ)].name, rm->tele->targ);
+	send_to_char(buf, ch);
+	send_to_char("Teleport Flags  :", ch);
+	sprintbit(rm->tele->mask, teleport_bits, buf);
+	send_to_char(buf, ch);
+	send_to_char("\r\n", ch);
+        if (IS_SET(rm->tele->mask, TELE_OBJ) || IS_SET(rm->tele->mask, TELE_NOOBJ)) {
+		j = read_object(rm->tele->obj, VIRTUAL, world[ch->in_room].zone);
+		sprintf(buf, "Teleport Object : %s\r\n", j->short_description);
+		send_to_char(buf, ch);
+		extract_obj(j);
+        }
+  }
   sprintf(buf, "Chars present:%s", CCYEL(ch, C_NRM));
   for (found = 0, k = rm->people; k; k = k->next_in_room) {
     if (!CAN_SEE(ch, k))
@@ -628,10 +638,6 @@ void do_stat_character(struct char_data * ch, struct char_data * k)
   struct obj_data *j;
   struct follow_type *fol;
   struct affected_type *aff;
-  extern struct attack_hit_type attack_hit_text[];
-  extern struct str_app_type str_app[];
-  extern struct dex_app_type dex_app[];
-  extern char *pc_race_types[];
 
   switch (GET_SEX(k)) {
   case SEX_NEUTRAL:    strcpy(buf, "NEUTRAL-SEX");   break;
@@ -655,21 +661,15 @@ void do_stat_character(struct char_data * ch, struct char_data * k)
   sprintf(buf, "L-Des: %s", (k->player.long_descr ? k->player.long_descr : "<None>\r\n"));
   send_to_char(buf, ch);
 
-  if (IS_NPC(k)) {
-    strcpy(buf, "Monster Class: ");
-    sprinttype(k->player.class, npc_class_types, buf2);
-  } else {
-    strcpy(buf2, "\r\n");
-    sprintf(buf, "Race: %s	", pc_race_types[(int)GET_RACE(k)]);
-    sprintf(buf, "%sClass: ", buf); 
-	if (GET_NUM_OF_CLASS(k) > 1)  {
-		for (i = 0; i < NUM_CLASSES; i++)
-			if (IS_SET(GET_CLASS(k), (1 << i)))
-				sprintf(buf, "%s%s ", buf, pc_class_types[i]);
-	}
-	else
-	    sprintf(buf, "%s%s ", buf, pc_class_types[GET_CLASS_NUM(k)]);
-  }
+  strcpy(buf2, "\r\n");
+  sprintf(buf, "Race: %s	", pc_race_types[(int)GET_RACE(k)]);
+  sprintf(buf, "%sClass: ", buf); 
+  if (GET_NUM_OF_CLASS(k) > 1)  {
+	for (i = 0; i < NUM_CLASSES; i++)
+		if (IS_SET(GET_CLASS(k), (1 << i)))
+			sprintf(buf, "%s%s ", buf, pc_class_types[i]);
+  } else
+        sprintf(buf, "%s%s ", buf, pc_class_types[GET_CLASS_NUM(k)]);
   strcat(buf, buf2);
 
   sprintf(buf2, "Lev: [%s%2d%s], XP: [%s%7d%s], Align: [%4d]\r\n",
@@ -763,7 +763,7 @@ if (!IS_NPC(k)) {
     sprintf(buf, "NPC flags : %s%s%s\r\n", CCCYN(ch, C_NRM), buf2, CCNRM(ch, C_NRM));
     sprintbit(MOB2_FLAGS(k), action_bits2, buf2);
     sprintf(buf, "%sNPC flags2: %s%s%s\r\n",buf, CCCYN(ch, C_NRM), buf2, CCNRM(ch, C_NRM));
-    sprintbit(MOB3_FLAGS(k), action_bits2, buf2);
+    sprintbit(MOB3_FLAGS(k), action_bits3, buf2);
     sprintf(buf, "%sNPC flags3: %s%s%s\r\n",buf, CCCYN(ch, C_NRM), buf2, CCNRM(ch, C_NRM));
     send_to_char(buf, ch);
   } else {
@@ -955,6 +955,7 @@ ACMD(do_stat)
 ACMD(do_shutdown)
 {
   extern int circle_shutdown, circle_reboot;
+  int temp = 0;
 
   if (subcmd != SCMD_SHUTDOWN) {
     send_to_char("If you want to shut something down, say so!\r\n", ch);
@@ -962,36 +963,80 @@ ACMD(do_shutdown)
   }
   one_argument(argument, arg);
 
-  if (!*arg) {
+  if (!*arg && GET_LEVEL(ch) == LVL_IMPL) {
     sprintf(buf, "(GC) Shutdown by %s.", GET_NAME(ch));
     log(buf);
-    send_to_all("Shutting down.\r\n");
-    circle_shutdown = 1;
+    sprintf(buf, "\007\007Shutdown sequence has been requested by %s\r\n", GET_NAME(ch));
+    send_to_all(buf);
+    circle_shutdown = (600 RL_SEC);
+  } else if (GET_LEVEL(ch) < LVL_IMPL || !str_cmp(arg, "time")) {
+	if (circle_shutdown == 0)
+		send_to_char(" LostLands is not shutting down.\r\n", ch);
+	else if (circle_shutdown == 1)
+		send_to_char(" LostLands is shutting down NOW!!!\r\n", ch);
+	else {
+		sprintf(buf, " LostLands is shutting down in %d seconds\r\n", circle_shutdown / PASSES_PER_SEC);
+		send_to_char(buf, ch);
+	}		
+  } else if (!str_cmp(arg, "off")) {
+	if (circle_shutdown >= 1) {
+	    sprintf(buf, "(GC) Reboot suspeneded by %s.", GET_NAME(ch));
+	    log(buf);
+	    sprintf(buf, "\007\007Shutdown CANCELLED by %s.\r\n", GET_NAME(ch));
+	    send_to_all(buf);
+	    circle_shutdown = 0;
+	} else
+	    send_to_char("But the mud isn't shutting down...\r\n", ch);
+  } else if (!str_cmp(arg, "quick-reboot")) {
+    sprintf(buf, "(GC) Quick-Reboot by %s.", GET_NAME(ch));
+    log(buf);
+    sprintf(buf, "\007\007Quick-reboot sequence has been requested by %s.\r\n", GET_NAME(ch));
+    send_to_all(buf);
+    touch("../.fastboot");
+    circle_shutdown = circle_reboot = (5 RL_SEC);
   } else if (!str_cmp(arg, "reboot")) {
     sprintf(buf, "(GC) Reboot by %s.", GET_NAME(ch));
     log(buf);
-    send_to_all("Rebooting.. come back in a minute or two.\r\n");
+    sprintf(buf, "\007\007Shutdown-reboot sequence has been requested by %s.\r\n", GET_NAME(ch));
+    send_to_all(buf);
     touch("../.fastboot");
-    circle_shutdown = circle_reboot = 1;
+    circle_shutdown = circle_reboot = (60 RL_SEC);
   } else if (!str_cmp(arg, "die")) {
     sprintf(buf, "(GC) Shutdown by %s.", GET_NAME(ch));
     log(buf);
-    send_to_all("Shutting down for maintenance.\r\n");
+    sprintf(buf, "\007\007PANIC\007\007 Shutdown sequence has been requested by %s.\r\n", GET_NAME(ch));
+    send_to_all(buf);
     touch("../.killscript");
-    circle_shutdown = 1;
+    circle_shutdown = (10 RL_SEC);
   } else if (!str_cmp(arg, "remake")) {
     sprintf(buf, "(GC) Shutdown by %s.", GET_NAME(ch));
     log(buf);
-    send_to_all("Shutting down to rebuild.\r\n");
+    sprintf(buf, "Code-maintanence sequence has been requested by %s.\r\n", GET_NAME(ch));
+    send_to_all(buf);
     touch("../.remake");
-    circle_shutdown = 1;
+    circle_shutdown = (300 RL_SEC);
   } else if (!str_cmp(arg, "pause")) {
     sprintf(buf, "(GC) Shutdown by %s.", GET_NAME(ch));
     log(buf);
-    send_to_all("Shutting down for maintenance.\r\n");
+    sprintf(buf, "Indefinite Shutdown requested by %s.\r\n", GET_NAME(ch));
+    send_to_all(buf);
     touch("../pause");
-    circle_shutdown = 1;
-  } else
+    circle_shutdown = (30 RL_SEC);
+  } else if (isdigit(*arg)) {
+    temp = atoi(arg);
+    if (temp > 3600) {
+	temp = 3600;
+	send_to_char("Timer set to 3600 seconds.\r\n", ch);
+    } else if (temp < 10) {
+	temp = 10;
+	send_to_char("Timer set to 10 seconds.\r\n", ch);
+    }
+    sprintf(buf, "(GC) Timed Shutdown by %s (%d seconds).", GET_NAME(ch), temp);
+    log(buf);
+    sprintf(buf, "\007\007Shutdown in %d seconds requested by %s.\r\n", temp, GET_NAME(ch));
+    send_to_all(buf);
+    circle_shutdown = (temp RL_SEC);
+  } else 
     send_to_char("Unknown shutdown option.\r\n", ch);
 }
 
@@ -1025,8 +1070,11 @@ ACMD(do_snoop)
     send_to_char("There's no link.. nothing to snoop.\r\n", ch);
   else if (victim == ch)
     stop_snooping(ch);
-  else if (victim->desc->snoop_by)
-    send_to_char("Busy already. \r\n", ch);
+  else if (victim->desc->snoop_by) {
+    send_to_char("Busy already. Snooped by: ", ch);
+    send_to_char(GET_NAME(victim->desc->snoop_by->character), ch);
+    send_to_char("\r\n", ch);
+  }
   else if (victim->desc->snooping == ch->desc)
     send_to_char("Don't be stupid.\r\n", ch);
   else {
@@ -1065,10 +1113,14 @@ ACMD(do_switch)
     send_to_char("No such character.\r\n", ch);
   else if (ch == victim)
     send_to_char("Hee hee... we are jolly funny today, eh?\r\n", ch);
+  else if (!IS_NPC(victim))
+    send_to_char("Sorry, you can only switch into mobs!\r\n", ch);
   else if (victim->desc)
     send_to_char("You can't do that, the body is already in use!\r\n", ch);
   else if ((GET_LEVEL(ch) < LVL_IMPL) && !IS_NPC(victim))
     send_to_char("You aren't holy enough to use a mortal's body.\r\n", ch);
+  else if (GET_LEVEL(ch) < GET_LEVEL(victim))
+    send_to_char("You aren't holy enough to use a mortal's body.\r\n", ch);
   else {
     send_to_char(OK, ch);
 
@@ -1077,6 +1129,8 @@ ACMD(do_switch)
 
     victim->desc = ch->desc;
     ch->desc = NULL;
+/*    victim->player_specials = ch->player_specials;
+    victim->player_specials->aliases = ch->player_specials->aliases; */
   }
 }
 
@@ -1122,7 +1176,7 @@ ACMD(do_load)
       send_to_char("There is no monster with that number.\r\n", ch);
       return;
     }
-    mob = read_mobile(r_num, REAL);
+    mob = read_mobile(r_num, REAL, world[ch->in_room].zone);
     char_to_room(mob, ch->in_room);
 
     act("$n makes a quaint, magical gesture with one hand.", TRUE, ch,
@@ -1134,7 +1188,7 @@ ACMD(do_load)
       send_to_char("There is no object with that number.\r\n", ch);
       return;
     }
-    obj = read_object(r_num, REAL);
+    obj = read_object(r_num, REAL, world[ch->in_room].zone);
     obj_to_room(obj, ch->in_room);
     act("$n makes a strange magical gesture.", TRUE, ch, 0, 0, TO_ROOM);
     act("$n has created $p!", FALSE, ch, obj, 0, TO_ROOM);
@@ -1166,7 +1220,7 @@ ACMD(do_vstat)
       send_to_char("There is no monster with that number.\r\n", ch);
       return;
     }
-    mob = read_mobile(r_num, REAL);
+    mob = read_mobile(r_num, REAL, world[ch->in_room].zone);
     char_to_room(mob, 0);
     do_stat_character(ch, mob);
     extract_char(mob);
@@ -1175,7 +1229,7 @@ ACMD(do_vstat)
       send_to_char("There is no object with that number.\r\n", ch);
       return;
     }
-    obj = read_object(r_num, REAL);
+    obj = read_object(r_num, REAL, world[ch->in_room].zone);
     do_stat_object(ch, obj);
     extract_obj(obj);
   } else
@@ -1230,7 +1284,6 @@ ACMD(do_purge)
       if (IS_NPC(vict))
 	extract_char(vict);
     }
-
     for (obj = world[ch->in_room].contents; obj; obj = next_o) {
       next_o = obj->next_content;
       extract_obj(obj);
@@ -1251,6 +1304,11 @@ ACMD(do_advance)
 
   two_arguments(argument, name, level);
 
+  if (GET_LEVEL(ch) < LVL_IMPL) {
+	send_to_char("You're not godly enough to do that!\r\n", ch);
+	return;
+  }
+
   if (*name) {
     if (!(victim = get_char_vis(ch, name))) {
       send_to_char("That player is not here.\r\n", ch);
@@ -1914,8 +1972,9 @@ void print_zone_to_buf(char *bufptr, int zone)
 }
 void print_zone_to_buf_mort(char *bufptr, int zone)
 {
-  sprintf(bufptr, "%s%3d %-30.30s\r\n",
-	  bufptr, zone_table[zone].number, zone_table[zone].name);
+  sprintf(bufptr, "%s  %-35.35s by: %-12.12s [lvs:%3d to%3d]\r\n",
+	  bufptr, zone_table[zone].name, zone_table[zone].creator,
+	  zone_table[zone].lvl_low, zone_table[zone].lvl_high);
 }
 
 ACMD(do_show)
@@ -1926,7 +1985,6 @@ ACMD(do_show)
   struct char_data *vict;
   struct obj_data *obj;
   char field[40], value[40], birth[80];
-  extern char *genders[];
   extern int buf_switches, buf_largecount, buf_overflows;
   void show_shops(struct char_data * ch, char *value);
 
@@ -1935,7 +1993,7 @@ ACMD(do_show)
     char level;
   } fields[] = {
     { "nothing",	0  },				/* 0 */
-    { "zones",		1  },			        /* 1 */
+    { "zones",		LVL_GOD },		        /* 1 */
     { "player",		LVL_GOD },
     { "rent",		LVL_GOD },
     { "stats",		LVL_IMMORT },
@@ -1943,6 +2001,10 @@ ACMD(do_show)
     { "death",		LVL_GOD },
     { "godrooms",	LVL_GOD },
     { "shops",		LVL_IMMORT },
+    { "teleport",       LVL_GOD },
+    { "areas",		1  },				/* 10 */
+    { "status",         1  },
+    { "pfile",          1  },
     { "\n", 0 }
   };
 
@@ -1974,28 +2036,20 @@ ACMD(do_show)
   switch (l) {
   case 1:			/* zone */
     /* tightened up by JE 4/6/93 */
-    if (self)
-	if (GET_LEVEL(ch) < LVL_IMMORT)
-	   print_zone_to_buf_mort(buf, world[ch->in_room].zone);
-	else
+    if (self) {
 	   print_zone_to_buf(buf, world[ch->in_room].zone);
-    else if (*value && is_number(value)) {
+    } else if (*value && is_number(value)) {
       for (j = atoi(value), i = 0; zone_table[i].number != j && i <= top_of_zone_table; i++);
       if (i <= top_of_zone_table)
-	if (GET_LEVEL(ch) < LVL_IMMORT)
-	   print_zone_to_buf_mort(buf, i);
-        else
 	   print_zone_to_buf(buf, i);
       else {
 	send_to_char("That is not a valid zone.\r\n", ch);
 	return;
       }
-    } else
+     } else {
       for (i = 0; i <= top_of_zone_table; i++)
-	if (GET_LEVEL(ch) < LVL_IMMORT)
-	  print_zone_to_buf_mort(buf, i);
-        else
 	  print_zone_to_buf(buf, i);
+    }
     send_to_char(buf, ch);
     break;
   case 2:			/* player */
@@ -2083,6 +2137,46 @@ ACMD(do_show)
   case 8:
     show_shops(ch, value);
     break;
+  case 9:	/* teleports */
+    strcpy(buf, "Teleport Rooms\r\n-----------\r\n");
+    for (i = 0, j = 0; i <= top_of_world; i++)
+      if (world[i].tele != NULL)
+	sprintf(buf, "%s%2d: [%5d] %-24.24s Targ: %5d\r\n", buf, ++j,
+		world[i].number, world[i].name, world[i].tele->targ);
+    send_to_char(buf, ch);
+    break;
+  case 10:
+    send_to_char("Available Areas for your Perusing Pleasure:\r\n", ch);
+    for (i = 0; i <= top_of_zone_table; i++)
+	if (zone_table[i].number >= 40 || GET_LEVEL(ch) > LVL_IMMORT)
+	  print_zone_to_buf_mort(buf, i);
+    send_to_char(buf, ch);
+    break;
+  case 11:				/* stats of a player */
+    if (!*value) {
+      send_to_char("A name would help.\r\n", ch);
+      return;
+    }
+    if (load_char(value, &vbuf) < 0) {
+      send_to_char("There is no such player.\r\n", ch);
+      return;
+    }
+    sprintf(buf, "%s is a level %2d %s %s.\r\n", vbuf.name, vbuf.level,
+      genders[(int) vbuf.sex], pc_race_types[(int) vbuf.race]);
+    sprintf(buf, "%s%s seems %s to you.\r\n",
+	    buf, vbuf.name,
+	    align_types[get_alignment_type(vbuf.char_specials_saved.alignment)]);
+/*    sprintf(buf, "%s%s has been killed %d times, and has killed %d times.\r\n",
+	    buf, vbuf.name,
+	    GET_REINCARN(&vbuf), vbuf.player_specials.saved.killed_by_player +
+		vbuf.player_specials.saved.killed_by_mob,
+		vbuf.player_specials.saved.killed_player);
+*/
+    send_to_char(buf, ch);
+    break;
+  case 12:
+    showplay();
+    break;
   default:
     send_to_char("Sorry, I don't understand that.\r\n", ch);
     break;
@@ -2107,13 +2201,13 @@ ACMD(do_show)
 ACMD(do_set)
 {
   int i, l;
-  struct char_data *vict;
-  struct char_data *cbuf;
+  struct char_data *vict = NULL;
+  struct char_data *cbuf = NULL;
   struct char_file_u tmp_store;
   char field[MAX_INPUT_LENGTH], name[MAX_INPUT_LENGTH], val_arg[MAX_INPUT_LENGTH];
   int on = 0, off = 0, value = 0;
   char is_file = 0, is_mob = 0, is_player = 0;
-  int player_i;
+  int player_i = 0;
   int parse_class(char arg, int race);
   int parse_race(char arg);
 
@@ -2142,7 +2236,7 @@ ACMD(do_set)
    { "con", 		LVL_GRGOD, 	BOTH, 	NUMBER },
    { "sex", 		LVL_GRGOD, 	BOTH, 	MISC },
    { "ac", 		LVL_GRGOD, 	BOTH, 	NUMBER },
-   { "gold",		LVL_GOD, 	BOTH, 	NUMBER },
+   { "coins",		LVL_GOD, 	BOTH, 	NUMBER },
    { "bank",		LVL_GOD, 	PC, 	NUMBER },  /* 20 */
    { "exp", 		LVL_GRGOD, 	BOTH, 	NUMBER },
    { "hitroll", 	LVL_GRGOD, 	BOTH, 	NUMBER },
@@ -2530,10 +2624,10 @@ ACMD(do_set)
     GET_RACE(vict) = i;
     break;
   case 49:
-    SET_OR_REMOVE(PRF_FLAGS(vict), PRF_BOUNTYHUNT);
+    SET_OR_REMOVE(PRF2_FLAGS(vict), PRF2_BOUNTYHUNT);
     break;
   case 50:
-    SET_OR_REMOVE(PRF_FLAGS(vict), PRF_ASSASSIN);
+    SET_OR_REMOVE(PRF2_FLAGS(vict), PRF2_ASSASSIN);
     break;
   case 51:
     SET_OR_REMOVE(PRF2_FLAGS(vict), PRF2_WAR_DRUHARI);
@@ -2605,3 +2699,294 @@ ACMD(do_syslog)
   sprintf(buf, "Your syslog is now %s.\r\n", logtypes[tp]);
   send_to_char(buf, ch);
 }
+
+#define OBJ_NEW(ch) ((ch)->player.obj_buffer)
+#define MOB_NEW(ch) ((ch)->player.mob_buf)
+#define ZONE_BUF(ch) ((ch)->player.zone_edit)
+#define GET_ROOM_SECTOR(room) (world[(room)].sector_type)
+
+ACMD(do_rstat)
+{
+  /*struct room_data *rm;*/
+  int number, r_num, room_temp;
+
+  one_argument(argument, buf);
+
+  if(*buf) {
+    if (!isdigit(*buf)) {
+      send_to_char("Usage: rstat [vnum]\r\n", ch);
+      return;
+    }
+
+    if ((number = atoi(buf)) < 0) {
+      send_to_char("A NEGATIVE number??\r\n", ch);
+      return;
+    }
+
+    if ((r_num = real_room(number)) < 0) {
+      send_to_char("There is no room with that number.\r\n", ch);
+      return;
+    }
+    room_temp = ch->in_room;
+    char_from_room(ch);
+    char_to_room(ch, r_num);
+    do_stat_room(ch);
+    char_from_room(ch);
+    char_to_room(ch, room_temp);
+  } else {
+    do_stat_room(ch);
+  }
+}
+
+
+
+ACMD(do_zstat)
+{
+  int i, zn = -1, pos = 0, none = TRUE;
+  FILE *zfile;
+  char line[256];
+  char arg3[40], arg4[80], arg5[40], arg6[40];
+
+  one_argument(argument, buf);
+
+  if(*buf) {
+    if(isdigit(*buf))
+      zn = atoi(buf);
+    else {
+      send_to_char("Usage: zstat [zone number]\r\n", ch);
+      return;
+    }
+  } else zn = world[ch->in_room].number / 100;
+  
+  if(zn < 0) return;
+
+  for(i = 0; i <= top_of_zone_table && zone_table[i].number != zn; i++);
+  if(i > top_of_zone_table) {
+    send_to_char("That zone doesn't exist.\r\n", ch);
+    return;
+  }
+
+  if((zone_table[i].reset_mode < 0) || (zone_table[i].reset_mode > 2))
+    zone_table[i].reset_mode = 2;
+
+  sprintf(buf, "%sZone #      : %s%3d       %sName      : %s%s\r\n", 
+    CCMAG(ch, C_SPR),
+    CCWHT(ch, C_SPR), zone_table[i].number, CCMAG(ch, C_SPR),
+    CCWHT(ch, C_SPR), zone_table[i].name);
+  sprintf(buf, "%s%sMin/Max LVL : %s%3d%s/%s%3d   %sCreator   : %s%s\r\n",
+    buf, CCMAG(ch, C_SPR), CCWHT(ch, C_SPR), zone_table[i].lvl_low,
+    CCMAG(ch, C_SPR), CCWHT(ch, C_SPR), zone_table[i].lvl_high,
+    CCMAG(ch, C_SPR), CCWHT(ch, C_SPR), zone_table[i].creator);
+  sprintf(buf, "%s%sBuilders    : %s", buf, CCMAG(ch, C_SPR), CCWHT(ch, C_SPR));
+
+  sprintf(buf1, "%s/%i.zon", ZON_PREFIX, zn);
+  if(!(zfile = fopen(buf1, "r"))) {
+    send_to_char(CCNRM(ch, C_SPR), ch);
+    send_to_char("\r\nError:  unable to open zone file\r\n", ch);
+    return;
+  }
+
+  do {
+    get_line2(zfile, line);
+    if(*line == '*') {
+      half_chop(line, arg3, arg4);
+      half_chop(arg4, arg5, arg6);
+      if(is_abbrev(arg5, "Builder")) {
+        none = FALSE;
+	switch (pos) {
+	  case 0:
+	    sprintf(buf, "%s%s", buf, arg6);
+            pos = 2;
+	    break;
+	  case 1:
+	    sprintf(buf, "%s\r\n              %s", buf, arg6);
+	    pos = 2;
+	    break;
+	  case 2:
+	    sprintf(buf, "%s, %s", buf, arg6);
+	    pos = 3;
+	    break;
+	  case 3:
+	    sprintf(buf, "%s, %s", buf, arg6);
+	    pos = 1;
+	    break;
+	}
+      }
+    }
+  } while((*line != '$') && !feof(zfile));
+  fclose(zfile);
+
+  if(none) sprintf(buf, "%sNONE%s\r\n", buf, CCNRM(ch, C_SPR));
+  else sprintf(buf, "%s%s\r\n", buf, CCNRM(ch, C_SPR));
+  send_to_char(buf, ch);
+}
+
+void remove(char * filename);
+void rename(char * filename, char * filename2);
+
+ACMD(do_zallow)
+{
+  int zone;
+  FILE *old_file, *new_file;
+  char *old_fname, *new_fname, line[256];
+  char arg1[40], arg2[40];
+
+  half_chop(argument, arg1, arg2);
+
+  if(!*arg1 || !*arg2 || !isdigit(*arg2)) {
+    send_to_char("Usage  : zallow <player> <zone>\r\n", ch);
+    return;
+  }
+  *arg1 = UPPER(*arg1);
+  zone = atoi(arg2);
+
+  sprintf(buf1, "%s/%i.zon", ZON_PREFIX, zone);
+  old_fname = buf1;
+  sprintf(buf2, "%s/%i.zon.temp", ZON_PREFIX, zone);
+  new_fname = buf2;
+
+  if(!(old_file = fopen(old_fname, "r"))) {
+    sprintf(buf, "Error: Could not open %s for read.\r\n", old_fname);
+    send_to_char(buf, ch);
+    return;
+  }
+  if(!(new_file = fopen(new_fname, "w"))) {
+    sprintf(buf, "Error: Could not open %s for write.\r\n", new_fname);
+    send_to_char(buf, ch);
+    fclose(old_file);
+    return;
+  }
+  get_line2(old_file, line);
+  fprintf(new_file, "%s\n", line);
+  get_line2(old_file, line);
+  fprintf(new_file, "%s\n", line);
+  get_line2(old_file, line);
+  fprintf(new_file, "%s\n", line);
+  get_line2(old_file, line);
+  fprintf(new_file, "%s\n", line);
+  get_line2(old_file, line);
+  fprintf(new_file, "* Builder  %s\n", arg1);
+  fprintf(new_file, "%s\n", line);
+  do {
+    get_line2(old_file, line);
+    fprintf(new_file, "%s\n", line);
+  } while(*line != 'S');
+  fclose(old_file);
+  fclose(new_file);
+  remove(old_fname);
+  rename(new_fname, old_fname);
+  send_to_char("Zone file edited.\r\n", ch);
+  sprintf(buf, "olc: %s gives %s zone #%i builder access.", GET_NAME(ch),
+    arg1, zone);
+  mudlog(buf, BRF, LVL_IMPL, TRUE);
+}
+
+
+ACMD(do_zdeny)
+{
+  int zone;
+  FILE *old_file, *new_file;
+  char *old_fname, *new_fname, line[256];
+  char arg1[40], arg2[40], arg3[40], arg4[40], arg5[40], arg6[40];
+
+  half_chop(argument, arg1, arg2);
+
+  if(!*arg1 || !*arg2 || !isdigit(*arg2)) {
+    send_to_char("Usage  : zdeny <player> <zone>\r\n", ch);
+    return;
+  }
+  *arg1 = UPPER(*arg1);
+  zone = atoi(arg2);
+
+  sprintf(buf1, "%s/%i.zon", ZON_PREFIX, zone);
+  old_fname = buf1;
+  sprintf(buf2, "%s/%i.zon.temp", ZON_PREFIX, zone);
+  new_fname = buf2;
+
+  if(!(old_file = fopen(old_fname, "r"))) {
+    sprintf(buf, "Error: Could not open %s for read.\r\n", old_fname);
+    send_to_char(buf, ch);
+    return;
+  }
+  if(!(new_file = fopen(new_fname, "w"))) {
+    sprintf(buf, "Error: Could not open %s for write.\r\n", new_fname);
+    send_to_char(buf, ch);
+    fclose(old_file);
+    return;
+  }
+  do {
+    get_line2(old_file, line);
+    if(strstr(line, "* Builder")) {
+      half_chop(line, arg3, arg4);
+      half_chop(arg4, arg5, arg6);
+      if(!isname(arg1, arg6))
+	 fprintf(new_file, "%s\n", line);
+    } else {
+      fprintf(new_file, "%s\n", line);
+    }
+  } while(*line != 'S');
+  fclose(old_file);
+  fclose(new_file);
+  remove(old_fname);
+  rename(new_fname, old_fname);
+  send_to_char("Zone file edited.\r\n", ch);
+  sprintf(buf, "olc: %s removes %s's zone #%i builder access.", GET_NAME(ch),
+    arg1, zone);
+  mudlog(buf, BRF, LVL_IMPL, TRUE);
+}
+
+void showplay()
+{
+/*
+   if (!(fl = fopen(filename, "r+"))) {
+      perror("error opening playerfile");
+      exit(1);
+   }
+
+   for (; ; ) {
+      fread(&player, sizeof(struct char_file_u ), 1, fl);
+      if (feof(fl)) {
+	 fclose(fl);
+	 exit(1);
+      }
+
+      switch (player.class) {
+      case CLASS_THIEF	: 
+	 strcpy(classname, "Th"); 
+	 break;
+      case CLASS_WARRIOR	: 
+	 strcpy(classname, "Wa"); 
+	 break;
+      case CLASS_MAGIC_USER	: 
+	 strcpy(classname, "Mu"); 
+	 break;
+      case CLASS_CLERIC	: 
+	 strcpy(classname, "Cl"); 
+	 break;
+      default : 
+	 strcpy(classname, "--"); 
+	 break;
+      }
+
+      switch (player.sex) {
+      case SEX_FEMALE	: 
+	 sexname = 'F'; 
+	 break;
+      case SEX_MALE	: 
+	 sexname = 'M'; 
+	 break;
+      case SEX_NEUTRAL: 
+	 sexname = 'N'; 
+	 break;
+      default : 
+	 sexname = '-'; 
+	 break;
+      }
+
+      printf("%5d. ID: %5ld (%c) [%2d %s] %-16s %9dg %9db\n", ++num,
+          player.char_specials_saved.idnum, sexname, player.level,
+          classname, player.name, player.points.gold,
+          player.points.bank_gold);
+   }
+*/
+}
