@@ -19,11 +19,16 @@
 #include <stdlib.h>
 #include <time.h>
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
 #include "db.h"
 #include "utils.h"
 #include "spells.h"
 #include "interpreter.h"
 
+void 	obj_from_char(struct obj_data *obj);
+void 	extract_obj(struct obj_data *obj);
+
 #define MIN_ATTRIBUTE_AVE 13
 /* Names first */
 
@@ -72,6 +77,15 @@ const char *race_abbrevs[] = {
   "LLY",
   "MIN",
   "PIX",
+  "HEN",
+  "DRA",
+  "NOR",
+  "UND",
+  "HMD",
+  "ANI",
+  "GIA",
+  "INS",
+  "DEM",
   "\n"
 };
 
@@ -86,24 +100,47 @@ const char *pc_race_types[] = {
   "Llyran",
   "Minotaur",
   "Pixie",
+  "Hengyokai",
+  "DragonLord",	/* last PC class */
+  "Other",
+  "Undead",
+  "Humanoid",
+  "Animal",
+  "Giant",
+  "Insectoid",
+  "Demon",
   "\n"
 };
 
 /* The menu for choosing a race in interpreter.c: */
-const char *race_menu =
+const char *race_menu1 =
 "\r\nYour race determines the modifiers to your attributes and what classes\r\n"
 "or multi-classes are available to you.\r\n"
-"    Race also decides what players you can player-kill or can PK you.\r\n"
-"  Race         Political Stance                 Attribute Mods\r\n"
-"  [H]uman      Neutral                          None\r\n"
-"  [E]lf        Member of Yllantra Collective    +1 Dex, -1 Con\r\n"
-"  H[A]lfelf    Neutral                          None\r\n"
-"  Ha[l]fling   Member of Yllantra Collective    +1 Dex, -1 Str\r\n"
-"  [G]nome      Neutral                          +1 Int, -1 Wis\r\n"
-"  Dark G[n]ome Member of Druhari Coalition      +1 Int, -1 Wis\r\n"
-"  [D]warf      Member of Druhari Coalition      +1 Con, -1 Cha\r\n"
-"  [M]inotaur   Neutral                          +2 Str +2 Con, -2 Wis -2 Cha\r\n"
-"  [P]ixie      Neutral                          +1 Int, -1 Con\r\n\r\n"
+"    Race also decides what players you can player-kill or can PK you.\n"
+"   Race        Political Stance                 Attribute Mods\n"
+"1) Human       Neutral                          None\n"
+"2) Elf         Member of Yllantra Collective    +1 Dex, -1 Con\r\n"
+"3) Halfelf     Neutral                          None\r\n"
+"4) Halfling    Member of Yllantra Collective    +1 Dex, -1 Str\r\n"
+"5) Gnome       Neutral                          +1 Int, -1 Wis\r\n"
+"6) Dark Gnome  Member of Druhari Coalition      +1 Int, -1 Wis\r\n"
+"7) Dwarf       Member of Druhari Coalition      +1 Con, -1 Cha\r\n";
+
+const char *race_menu2 =
+"8) Minotaur    Neutral                          +2 Str +2 Con, -2 Wis -2 Cha\r\n"
+"9) Pixie       Neutral                          +1 Int, -1 Con\r\n";
+
+const char *race_menu3 =
+"v) Hemnov      Neutral                          ???\r\n"
+"y) Llyran      Neutral                          ???\r\n";
+
+const char *race_menu4 =
+"w) Hengyokai   Neutral                          ???\r\n";
+
+const char *race_menu5 =
+"d) DragonLord  Neutral                          ???\r\n";
+
+const char *race_menu6 =
 "The Yllantra Collective are at war against the Druhari Coalition, and all\r\n"
 "their members are automatically registered for war.\r\n"
 "Non-members can register at the recruitment center in Capitol.\r\n"
@@ -118,39 +155,45 @@ int parse_race(char arg)
   arg = LOWER(arg);
 
   switch (arg) {
-  case 'a':
-    i = RACE_HALFELF;
-    break;
-  case 'd':
-    i = RACE_DWARF;
-    break;
-  case 'e':
-    i = RACE_ELF;
-    break;
-  case 'g':
-    i = RACE_GNOME;
-    break;
-  case 'n':
-    i = RACE_DARKGNOME;
-    break;
-  case 'h':
+  case '1':
     i = RACE_HUMAN;
     break;
-  case 'l':
+  case '2':
+    i = RACE_ELF;
+    break;
+  case '3':
+    i = RACE_HALFELF;
+    break;
+  case '4':
     i = RACE_HALFLING;
     break;
-  case 'm':
-    i = RACE_MINOTAUR;
-    break;
-  case 'p':
-    i = RACE_PIXIE;
+  case '5':
+    i = RACE_GNOME;
     break;
   case '6':
+    i = RACE_DARKGNOME;
+    break;
+  case '7':
+    i = RACE_DWARF;
+    break;
+  case '8':
+    i = RACE_MINOTAUR;
+    break;
+  case '9':
+    i = RACE_PIXIE;
+    break;
+  case 'v':
     i = RACE_HEMNOV; 
     break; 
-  case '7':
+  case 'y':
     i = RACE_LLYRA;
     break;
+  case 'w':
+    i = RACE_WEREFORM;
+    break;
+  case 'd':
+    i = RACE_DRAGON;
+    break;
   default:
     break;
   }
@@ -158,7 +201,7 @@ int parse_race(char arg)
 }
 
 /* The menu for choosing a class in interpreter.c: */
-const char *class_menu =
+const char *class_menu1 =
 "\r\n"
 "Select a class:\r\n\r\n"
 " [F]ighter     [B]arbarian   [S]amurai    [T]hief      [N]inja\r\n"
@@ -166,19 +209,30 @@ const char *class_menu =
 " [1] Mage-Cleric             [2] Mage-Thief            [3] Mage-Fighter\r\n"
 " [4] Mage-Cleric-Fighter     [5] Mage-Thief-Figher     [6] Cleric-Fighter\r\n"
 " [7] Fighter-Thief                                     [8] Monk\r\n"
-"        Race/Code:  F B S C D M W T N 1 2 3 4 5 6 7 8\r\n"
-"         Human   :  X X X X X X X X X               X\r\n"
-"         Elf     :  X     X X X   X     X X   X   X\r\n"
-"         Halfelf :  X     X X X   X   X X X X X X X\r\n"
-"         Dwarf   :  X X   X       X             X X\r\n"
-"         Halfling:  X X       X   X               X\r\n"
-"         Gnome   :  X     X   X X X     X X       X X\r\n"
-"         Minotaur:  X X     X X   X\r\n"
-"         Pixie   :  X             X               X\r\n"
+"    Race/Code    :  F B S C D M W T N 1 2 3 4 5 6 7 8\r\n"
+"     Human       :  X X X X X X X X X               X\r\n"
+"     Elf         :  X     X X X   X     X X   X   X\r\n"
+"     Halfelf     :  X     X X X   X   X X X X X X X\r\n"
+"     Dwarf       :  X X   X       X             X X\r\n"
+"     Halfling    :  X X       X   X               X\r\n"
+"     Gnome       :  X     X   X X X     X X       X X\r\n"
+"     Minotaur    :  X X     X X   X\r\n"
+"     Pixie       :  X             X               X\r\n";
+
+const char *class_menu2 = 
+"     Hemnov      :  X   X X X X   X   X     X       X\r\n"
+"     Llyran      :  X   X X   X   X X   X X       X\r\n";
+
+const char *class_menu3 =
+"     Hengyokai   :  X X     X X   X     X X   X     X\r\n"
+"     DragonLord  :  X   X X     X X       X   X     X\r\n";
+
+const char *class_menu4 =
+"    Race/Code    :  F B S C D M W T N 1 2 3 4 5 6 7 8\r\n"
 "        X == Available\r\n\r\n"
 " Only Single Class Fighters/Barbarians/Samurai can be BountyHunters.\r\n"
 " Only Single Class Thieves and Ninjas can register to be Assassins.\r\n"
-" [A]vatars are available only after reincarnation\r\n";
+" Avatars (A) are available at the last step upon the Golden Path.\r\n";
 
 /*
  * The code to interpret a class letter -- used in interpreter.c when a
@@ -192,22 +246,25 @@ int parse_class(char arg, int race)
 
   switch (arg) {
   case 'b':
-    if (race == RACE_HUMAN || race == RACE_DWARF || race == RACE_HALFLING || race == RACE_MINOTAUR || race == RACE_HEMNOV || race == RACE_LLYRA)
+    if (race == RACE_HUMAN || race == RACE_DWARF ||
+	  	race == RACE_HALFLING || race == RACE_MINOTAUR || race == RACE_WEREFORM)
 	retval = CLASS_BARBARIAN;
     break;
   case 'c':
-    if (!(race == RACE_HALFLING || race == RACE_MINOTAUR || race == RACE_PIXIE))
+    if (!(race == RACE_HALFLING || race == RACE_MINOTAUR || race == RACE_PIXIE ||
+		race == RACE_WEREFORM))
 	retval = CLASS_CLERIC;
     break;
   case 'd':
-    if (race == RACE_HUMAN || race == RACE_ELF || race == RACE_HALFELF || race == RACE_MINOTAUR || race == RACE_HEMNOV || race == RACE_LLYRA)
+    if (race == RACE_HUMAN || race == RACE_ELF || race == RACE_HALFELF ||
+		 race == RACE_MINOTAUR || race == RACE_HEMNOV || race == RACE_WEREFORM)
 	retval = CLASS_DRUID;
     break;
   case 'f':
     retval = CLASS_WARRIOR;
     break;
   case 'm':
-    if (!(race == RACE_DWARF || race == RACE_PIXIE))
+    if (!(race == RACE_DWARF || race == RACE_PIXIE || race == RACE_DRAGON))
 	retval = CLASS_MAGIC_USER;
     break;
   case 'n':
@@ -215,14 +272,15 @@ int parse_class(char arg, int race)
 	retval = CLASS_NINJA;
     break;
   case 's':
-    if (race == RACE_HUMAN || race == RACE_HEMNOV)
+    if (race == RACE_HUMAN || race == RACE_HEMNOV || race == RACE_WEREFORM ||
+		race == RACE_DRAGON)
 	retval = CLASS_SAMURAI;
     break;
   case 't':
     retval = CLASS_THIEF;
     break;
   case 'w':
-    if (race == RACE_HUMAN || race == RACE_GNOME || race == RACE_HEMNOV || race == RACE_LLYRA)
+    if (race == RACE_HUMAN || race == RACE_GNOME || race == RACE_DRAGON)
 	retval = CLASS_WIZARD;
     break;
   case '1':
@@ -230,11 +288,13 @@ int parse_class(char arg, int race)
 	retval = CLASS_MAG_CLE;
     break;
   case '2':
-    if (race == RACE_ELF || race == RACE_HALFELF || race == RACE_GNOME)
+    if (race == RACE_ELF || race == RACE_HALFELF || race == RACE_GNOME ||
+		race == RACE_LLYRA || race == RACE_WEREFORM)
 	retval = CLASS_MAG_THI;
     break;
   case '3':
-    if (race == RACE_ELF || race == RACE_HALFELF || race == RACE_GNOME)
+    if (race == RACE_ELF || race == RACE_HALFELF || race == RACE_GNOME ||
+		race == RACE_LLYRA || race == RACE_WEREFORM || race == RACE_DRAGON)
 	retval = CLASS_MAG_FIG;
     break;
   case '4':
@@ -242,7 +302,8 @@ int parse_class(char arg, int race)
 	retval = CLASS_MAG_CLE_FIG;
     break;
   case '5':
-    if (race == RACE_ELF || race == RACE_HALFELF)
+    if (race == RACE_ELF || race == RACE_HALFELF || race == RACE_WEREFORM || 
+		race == RACE_DRAGON)
 	retval = CLASS_MAG_THI_FIG;
     break;
   case '6':
@@ -250,15 +311,18 @@ int parse_class(char arg, int race)
 	retval = CLASS_CLE_FIG;
     break;
   case '7':
-    if (race != RACE_HUMAN)
+    if (!(race == RACE_HUMAN || race == RACE_MINOTAUR || race == RACE_HEMNOV ||
+		race == RACE_WEREFORM || race == RACE_DRAGON))
 	retval = CLASS_FIG_THI;
     break;
   case '8':
-    if (race == RACE_HUMAN || race == RACE_GNOME || race == RACE_HEMNOV || race == RACE_LLYRA)
+    if (race == RACE_HUMAN || race == RACE_GNOME || race == RACE_HEMNOV ||
+		 race == RACE_WEREFORM || race == RACE_DRAGON)
 	retval = CLASS_MONK;
     break;
   case 'a':
-    retval = CLASS_AVATAR;
+    if (!(race == RACE_WEREFORM || race == RACE_PIXIE))
+	    retval = CLASS_AVATAR;
     break;
   default:
     retval = CLASS_UNDEFINED;
@@ -323,7 +387,6 @@ long find_class_bitvector(char arg)
   }
 }
 
-
 /*
  * These are definitions which control the guildmasters for each class.
  *
@@ -375,25 +438,26 @@ int prac_params[2][NUM_CLASSES] = {
  * first line shows that from room 3017, only MAGIC_USERS are allowed
  * to go south.
  */
-int guild_info[][5] = {
+int guild_info[][7] = {
 
 /* Capitol */
-  {CLASS_CLERIC,	ANYRACE, ANYALIGN,		4263,	SCMD_NORTH},
-  {CLASS_DRUID,		ANYRACE, ANYALIGN,		4260,	SCMD_SOUTH},
+  {CLASS_CLERIC,	ANYRACE, ANYALIGN,		4263,	SCMD_NORTH, 1, LVL_IMPL},
+  {CLASS_DRUID,		ANYRACE, ANYALIGN,		4260,	SCMD_SOUTH, 1, LVL_IMPL},
   {CLASS_MAGIC_USER + CLASS_AVATAR + CLASS_WIZARD, ANYRACE, ANYALIGN, 4175,
-	SCMD_SOUTH},
+	SCMD_SOUTH, 1, LVL_IMPL},
   {CLASS_MONK + CLASS_AVATAR + CLASS_WARRIOR + CLASS_BARBARIAN + CLASS_SAMURAI,	
-	ANYRACE, ANYALIGN, 4204, SCMD_WEST},
-  {CLASS_THIEF + CLASS_NINJA,	ANYRACE, ANYALIGN, 4107, SCMD_SOUTH},
-  {ANYCLASS, RACE_GNOME, ANYALIGN, 4058, SCMD_NORTH},
-  {ANYCLASS, RACE_ELF, ANYALIGN, 4061, SCMD_NORTH},
-  {ANYCLASS, RACE_HALFLING, ANYALIGN, 4064, SCMD_NORTH},
-  {ANYCLASS, RACE_DWARF, ANYALIGN, 4093, SCMD_SOUTH},
-  {ANYCLASS, RACE_ELF, ANYALIGN, 4047, SCMD_WEST},
-  {ANYCLASS, RACE_HALFLING, ANYALIGN, 4047, SCMD_EAST},
+	ANYRACE, ANYALIGN, 4204, SCMD_WEST, 1, LVL_IMPL},
+  {CLASS_THIEF + CLASS_NINJA,	ANYRACE, ANYALIGN, 4107, SCMD_SOUTH, 1, LVL_IMPL},
+  {ANYCLASS, RACE_GNOME, ANYALIGN, 4058, SCMD_NORTH, 1, LVL_IMPL},
+  {ANYCLASS, RACE_ELF, ANYALIGN, 4061, SCMD_NORTH, 1, LVL_IMPL},
+  {ANYCLASS, RACE_HALFLING, ANYALIGN, 4064, SCMD_NORTH, 1, LVL_IMPL},
+  {ANYCLASS, RACE_DWARF, ANYALIGN, 4093, SCMD_SOUTH, 1, LVL_IMPL},
+  {ANYCLASS, RACE_ELF, ANYALIGN, 4047, SCMD_WEST, 1, LVL_IMPL},
+  {ANYCLASS, RACE_HALFLING, ANYALIGN, 4047, SCMD_EAST, 1, LVL_IMPL},
+  {ANYCLASS, ANYRACE, ANYALIGN, 4000, SCMD_NORTH, 1, 5},
 
 /* this must go last -- add new guards above! */
-{-1, -1, -1, -1, -1}};
+{-1, -1, -1, -1, -1, 1 , LVL_IMPL}};
 
 
 
@@ -536,19 +600,33 @@ void roll_real_abils(struct char_data * ch)
 	ch->real_abils.intel += 1;
 	ch->real_abils.con -= 1;
   } else if (IS_LLYRA(ch)) {
-	ch->real_abils.str = 22;
-	ch->real_abils.dex = 24;
-	ch->real_abils.wis = 23;
-	ch->real_abils.con = 20;
-	ch->real_abils.cha = 25;
-	ch->real_abils.intel = 24;
+	ch->real_abils.str = 18 + dice(1, 4);
+	ch->real_abils.dex = 18 + dice(1, 6);
+	ch->real_abils.wis = 19 + dice(1, 4);
+	ch->real_abils.con = 17 + dice(1, 3);
+	ch->real_abils.cha = 20 + dice(1, 5);
+	ch->real_abils.intel = 20 + dice(1, 4);
   } else if (IS_HEMNOV(ch)) {
-	ch->real_abils.str = 24;
-	ch->real_abils.dex = 23;
-	ch->real_abils.wis = 22;
-	ch->real_abils.con = 25;
-	ch->real_abils.cha = 25;
-	ch->real_abils.intel = 22;
+	ch->real_abils.str = 18 + dice(1, 6);
+	ch->real_abils.dex = 19 + dice(1, 4);
+	ch->real_abils.wis = 18 + dice(1, 4);
+	ch->real_abils.con = 20 + dice(1, 5);
+	ch->real_abils.cha = 19 + dice(1, 6);
+	ch->real_abils.intel = 17 + dice(1, 5);
+  } else if (IS_WEREFORM(ch)) {
+	ch->real_abils.str = 18 + dice(1, 4);
+	ch->real_abils.dex = 19 + dice(1, 4);
+	ch->real_abils.wis = 18 + dice(1, 3);
+	ch->real_abils.con = 20 + dice(1, 5);
+	ch->real_abils.cha = 20 + dice(1, 5);
+	ch->real_abils.intel = 17 + dice(1, 4);
+  } else if (IS_DRAGON(ch)) {
+	ch->real_abils.str = 17 + dice(1, 8);
+	ch->real_abils.dex = 18 + dice(1, 7);
+	ch->real_abils.wis = 17 + dice(1, 8);
+	ch->real_abils.con = 19 + dice(1, 6);
+	ch->real_abils.cha = 18 + dice(1, 7);
+	ch->real_abils.intel = 20 + dice(1, 5);
   }
   m = ch->real_abils.str + ch->real_abils.dex + ch->real_abils.con +
 	ch->real_abils.intel + ch->real_abils.wis + ch->real_abils.cha;
@@ -565,16 +643,24 @@ void roll_real_abils(struct char_data * ch)
 void do_start(struct char_data * ch)
 {
   void advance_level(struct char_data * ch);
+  struct obj_data *obj;
 
   int gold = 0;
   int divis = GET_NUM_OF_CLASS(ch);
   GET_LEVEL(ch) = 1;
   GET_EXP(ch) = 1;
+  GET_ALIGNMENT(ch) = 0;
 
   set_title(ch, NULL);
   ch->points.max_hit = 10;
   ch->points.max_mana = 50;  
 
+  while (ch->carrying) {	/* clear out inventory, just in case */
+	obj = ch->carrying;
+	obj_from_char(obj);
+	extract_obj(obj);
+  }
+	
   if (IS_AVATAR(ch)) {
 	if (ch->real_abils.str == 18) {
 		 ch->real_abils.str_add = number(1, 100);
@@ -645,6 +731,7 @@ void do_start(struct char_data * ch)
   }
 
   GET_GOLD(ch) = 100 * gold;
+  GET_BANK_GOLD(ch) = 0;
   GET_PRACTICES(ch) = 10;
   advance_level(ch);
 
@@ -652,8 +739,8 @@ void do_start(struct char_data * ch)
   GET_MANA(ch) = GET_MAX_MANA(ch);
   GET_MOVE(ch) = GET_MAX_MOVE(ch);
 
-  GET_COND(ch, THIRST) = 72;
-  GET_COND(ch, FULL) = 72;
+  GET_COND(ch, THIRST) = 70;
+  GET_COND(ch, FULL) = 70;
   GET_COND(ch, DRUNK) = 0;
 
   ch->player.time.played = 0;
@@ -665,7 +752,8 @@ void do_start(struct char_data * ch)
 	SET_BIT(PRF2_FLAGS(ch), PRF2_WAR_DRUHARI);
 
   SET_BIT(PRF_FLAGS(ch), PRF_DISPHP | PRF_DISPMANA | PRF_DISPMOVE | PRF_DISPGOLD | PRF_DISPXP);
-  SET_BIT(PRF_FLAGS(ch), PRF_SUMMONABLE | PRF_AUTOEXIT | PRF_NOWIZ | PRF_NOQUIT);
+  SET_BIT(PRF_FLAGS(ch), PRF_SUMMONABLE | PRF_AUTOEXIT | PRF_NOWIZ | PRF_NOCLAN | PRF_NOTEAM);
+  SET_BIT(PRF2_FLAGS(ch), PRF2_NOQUIT);
   SET_BIT(PLR_FLAGS(ch), PLR_NOTITLE);
 }
 
@@ -761,29 +849,36 @@ void advance_level(struct char_data * ch)
 int invalid_class(struct char_data *ch, struct obj_data *obj) {
   long temp_class = ~0;
 
-  REMOVE_BIT(temp_class, CLASS_DUAL | CLASS_TRIPLE);
-
-  if (IS_OBJ_STAT(obj, ITEM_ANTI_MAGIC_USER))
-	REMOVE_BIT(temp_class, CLASS_WIZARD | CLASS_MAGIC_USER);
-  if (IS_OBJ_STAT(obj, ITEM_ANTI_CLERIC))
-	REMOVE_BIT(temp_class, CLASS_CLERIC | CLASS_DRUID);
-  if (IS_OBJ_STAT(obj, ITEM_ANTI_THIEF))
-	REMOVE_BIT(temp_class, CLASS_THIEF | CLASS_NINJA);
-  if (IS_OBJ_STAT(obj, ITEM_ANTI_WARRIOR))
-	REMOVE_BIT(temp_class, CLASS_WARRIOR | CLASS_BARBARIAN | CLASS_SAMURAI | CLASS_MONK | CLASS_AVATAR);
-
   if (IS_MOB(ch))
 	return 0;
   if (GET_OBJ_TYPE(obj) == ITEM_WEAPON) {
+	  REMOVE_BIT(temp_class, CLASS_DUAL | CLASS_TRIPLE);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_MAGIC_USER))
+		REMOVE_BIT(temp_class, CLASS_WIZARD | CLASS_MAGIC_USER);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_CLERIC))
+		REMOVE_BIT(temp_class, CLASS_CLERIC | CLASS_DRUID);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_THIEF))
+		REMOVE_BIT(temp_class, CLASS_THIEF | CLASS_NINJA);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_WARRIOR))
+		REMOVE_BIT(temp_class, CLASS_WARRIOR | CLASS_BARBARIAN | CLASS_SAMURAI | CLASS_MONK | CLASS_AVATAR);
      if IS_SET(GET_CLASS(ch), temp_class)
 	return 0;
      else
 	return 1;
   } else {
-     if IS_SET(GET_CLASS(ch), ~temp_class)
+	temp_class = 0;
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_MAGIC_USER))
+		SET_BIT(temp_class, CLASS_WIZARD | CLASS_MAGIC_USER);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_CLERIC))
+		SET_BIT(temp_class, CLASS_CLERIC | CLASS_DRUID);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_THIEF))
+		SET_BIT(temp_class, CLASS_THIEF | CLASS_NINJA);
+	  if (IS_OBJ_STAT(obj, ITEM_ANTI_WARRIOR))
+		SET_BIT(temp_class, CLASS_WARRIOR | CLASS_BARBARIAN | CLASS_SAMURAI | CLASS_MONK | CLASS_AVATAR);
+     if (GET_CLASS(ch) & temp_class)
 	return 1;
      else
-	return 0;
+	return 0; 
   }
 }
 
