@@ -26,6 +26,9 @@
 #include <signal.h>
 #include <netinet/in.h>
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
+#include "rooms.h"
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -33,6 +36,7 @@
 #include "db.h"
 #include "house.h"
 #include "screen.h"
+#include "teleport.h"
 
 /* externs */
 extern int restrict;
@@ -366,13 +370,16 @@ int get_avail_descs(void)
  * output and sending it out to players, and calling "heartbeat" functions
  * such as mobile_activity().
  */
+
+int pulse = 0;
+
 void game_loop(int mother_desc)
 {
   fd_set input_set, output_set, exc_set;
   struct timeval last_time, now, timespent, timeout, opt_time;
   char comm[MAX_INPUT_LENGTH];
   struct descriptor_data *d, *next_d;
-  int pulse = 0, mins_since_crashsave = 0, maxdesc, aliased;
+  int mins_since_crashsave = 0, maxdesc, aliased;
 
   /* initialize various time values */
   null_time.tv_sec = 0;
@@ -382,7 +389,7 @@ void game_loop(int mother_desc)
   gettimeofday(&last_time, (struct timezone *) 0);
 
   /* The Main Loop.  The Big Cheese.  The Top Dog.  The Head Honcho.  The.. */
-  while (!circle_shutdown) {
+  while (circle_shutdown != 1) {
 
     /* Sleep if we don't have any connections */
     if (descriptor_list == NULL) {
@@ -502,6 +509,71 @@ void game_loop(int mother_desc)
       }
     }
 
+    if (circle_shutdown > 1) {
+	switch (circle_shutdown) {
+	case (3600 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 1 HOUR\r\n");
+		break;
+	case (1800 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 30 MINUTES\r\n");
+		break;
+	case (1200 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 20 MINUTES\r\n");
+		send_to_all("We suggest you go and RENT out soon.  We won't reimburse crash-lost items.\r\n");
+		break;
+	case (600 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 10 MINUTES\r\n");
+		send_to_all("We suggest you go and RENT out soon.  We won't reimburse crash-lost items.\r\n");
+		break;
+	case (300 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 5 MINUTES\r\n");
+		send_to_all("We suggest you go and RENT out soon.  We won't reimburse crash-lost items.\r\n");
+		break;
+	case (180 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 3 MINUTES\r\n");
+		send_to_all("We suggest you go and RENT out soon.  We won't reimburse crash-lost items.\r\n");
+		break;
+	case (60 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 1 MINUTE\r\n");
+		send_to_all("We suggest you go and RENT out soon.  We won't reimburse crash-lost items.\r\n");
+		break;
+	case (30 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 30 SECONDS\r\n");
+		break;
+	case (10 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 10 SECONDS\r\n");
+		break;
+	case (9 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 9 SECONDS\r\n");
+		break;
+	case (8 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 8 SECONDS\r\n");
+		break;
+	case (7 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 7 SECONDS\r\n");
+		break;
+	case (6 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 6 SECONDS\r\n");
+		break;
+	case (5 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 5 SECONDS\r\n");
+		break;
+	case (4 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 4 SECONDS\r\n");
+		break;
+	case (3 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 3 SECONDS\r\n");
+		break;
+	case (2 RL_SEC):
+		send_to_all("\007\007LOSTLANDS WILL SHUTDOWN IN 2 SECONDS\r\n");
+		break;
+	case (1 RL_SEC):
+		send_to_all("\007\007SHUTTING DOWN\007\007\r\n");
+		send_to_all("Come back soon, ya hear?\r\n");
+		break;
+	}
+	circle_shutdown--;
+    }
     /* send queued output out to the operating system (ultimately to user) */
     for (d = descriptor_list; d; d = next_d) {
       next_d = d->next;
@@ -538,10 +610,10 @@ void game_loop(int mother_desc)
       mobile_activity();
     if (!(pulse % PULSE_VIOLENCE)) 
       perform_violence();
-
+    if (!(pulse % PULSE_TELEPORT))
+      TeleportPulseStuff(pulse);
     if (!(pulse % (SECS_PER_MUD_HOUR * PASSES_PER_SEC / 3)))
       point_update();
- 
     if (!(pulse % (SECS_PER_MUD_HOUR * PASSES_PER_SEC))) {
       weather_and_time(1);
       affect_update();
@@ -656,6 +728,9 @@ void make_prompt(struct descriptor_data * d)
     if (GET_INVIS_LEV(d->character))
       sprintf(prompt, "i%d ", GET_INVIS_LEV(d->character));
 
+    if (PRF2_FLAGGED(d->character, PRF2_AFK))
+      sprintf(prompt, "%sAFK ", prompt);
+
     i = 100 * GET_HIT(d->character) / GET_MAX_HIT(d->character);
     STATUS_COLOR(i, colorbuf, d->character, C_SPR);
     if (PRF_FLAGGED(d->character, PRF_DISPHP))
@@ -685,7 +760,7 @@ void make_prompt(struct descriptor_data * d)
 	      CCNRM(d->character, C_SPR));
 
     if (PRF_FLAGGED(d->character, PRF_DISPXP))
-	if (!IS_IMMORT(d->character))
+	if (GET_LEVEL(d->character) < (LVL_IMMORT - 1) && !IS_MOB(d->character))
 	      sprintf(prompt, "%sXPtoLv %d: %d ", prompt, 
 		 GET_LEVEL(d->character) + 1,
 		 (titles[(int) GET_CLASS_NUM_FULL(d->character)][GET_LEVEL(d->character) + 1].exp) - GET_EXP(d->character) );
@@ -1294,6 +1369,10 @@ void checkpointing()
     tics = 0;
 }
 
+void shuttingdown()
+{
+   send_to_all("SHUTTING DOWN NOW!!!");  
+}
 
 void reread_wizlists()
 {
@@ -1390,7 +1469,7 @@ void signal_setup(void)
   my_signal(SIGINT, hupsig);
   my_signal(SIGTERM, hupsig);
   my_signal(SIGPIPE, SIG_IGN);
-  my_signal(SIGALRM, SIG_IGN);
+  my_signal(SIGALRM, shuttingdown);
 }
 
 
@@ -1430,7 +1509,21 @@ void send_to_outdoor(char *messg)
       SEND_TO_Q(messg, i);
 }
 
+void send_to_channel(char *messg, int channel)
+{
+   struct descriptor_data *i;
+   char buf[256];
 
+   if (!messg || !*messg)
+	return;
+
+   sprintf(buf, "Channel %d: %s", channel, messg);
+
+   for (i = descriptor_list; i; i = i->next)
+	if (!i->connected && i->character && AWAKE(i->character) &&
+		IS_SET(GET_CHANNEL(i->character), 1 << (channel - 1)));
+	SEND_TO_Q(buf, i);
+}
 
 void send_to_room(char *messg, int room)
 {
@@ -1440,10 +1533,17 @@ void send_to_room(char *messg, int room)
     for (i = world[room].people; i; i = i->next_in_room)
       if (i->desc)
 	SEND_TO_Q(messg, i->desc);
+
+  if (ROOM_FLAGGED(room, ROOM_BROADCAST)) {
+	if (world[room].broad->channel > 0);
+		send_to_channel(messg, world[room].broad->channel);
+	if (real_room(world[room].broad->targ1) > 0)
+		send_to_room(messg, real_room(world[room].broad->targ1));
+	if (real_room(world[room].broad->targ2) > 0)
+		send_to_room(messg, real_room(world[room].broad->targ2));
+  }
 }
 
-
-
 char *ACTNULL = "<NULL>";
 
 #define CHECK_NULL(pointer, expression) \
@@ -1458,6 +1558,7 @@ void perform_act(char *orig, struct char_data * ch, struct obj_data * obj,
   static char lbuf[MAX_STRING_LENGTH];
 
   buf = lbuf;
+  i = NULL;  
 
   for (;;) {
     if (*orig == '$') {
@@ -1538,7 +1639,11 @@ void perform_act(char *orig, struct char_data * ch, struct obj_data * obj,
 }
 
 
-#define SENDOK(ch) ((ch)->desc && (AWAKE(ch) || sleep) && \
+/* #define SENDOK(ch) ((ch)->desc && (AWAKE(ch) || sleep) && \
+		    !PLR_FLAGGED((ch), PLR_WRITING) && \
+		    !PLR_FLAGGED((ch), PLR_EDITING))
+*/
+#define SENDOK(ch) ((AWAKE(ch) || sleep) && \
 		    !PLR_FLAGGED((ch), PLR_WRITING) && \
 		    !PLR_FLAGGED((ch), PLR_EDITING))
 
