@@ -19,6 +19,44 @@ const char circlemud_version[] = {
 
 /* (Note: strings for class definitions in class.c instead of here) */
 
+/* char channel_bits[] will be read in from a file */
+/* char clan_names[] will also be read in from a file */
+
+const char *channel_bits[] = {
+  "God Channel",
+  "Radio Free LostLands",
+  "Arena battles",
+  "4",
+  "5",
+  "6",
+  "7",
+  "8",
+  "9",
+  "10",
+  "11",
+  "12",
+  "13",
+  "14",
+  "15",
+  "16",
+  "17",
+  "18",
+  "19",
+  "20",
+  "21",
+  "22",
+  "23",
+  "24",
+  "25",
+  "26",
+  "27",
+  "28",
+  "29",
+  "30",
+  "31",
+  "32",
+  "\n"
+};
 
 /* cardinal directions */
 const char *dirs[] =
@@ -32,6 +70,27 @@ const char *dirs[] =
   "\n"
 };
 
+const char *dirs2[] = 
+{
+  "the north",
+  "the east",
+  "the south",
+  "the west",
+  "above",
+  "below",
+  "\n"
+};
+
+const char *teleport_bits[] = {
+  "Force Look",
+  "Timed",
+  "Random",
+  "Spin(NOT AVAIL)",
+  "Has Obj",
+  "No Obj",
+  "No MSG",
+  "\n"
+};
 
 /* ROOM_x */
 const char *room_bits[] = {
@@ -46,12 +105,15 @@ const char *room_bits[] = {
   "TUNNEL",
   "PRIVATE",
   "GODROOM",
-  "HOUSE",
-  "HCRSH",
-  "ATRIUM",
-  "OLC",
-  "*",				/* BFS MARK */
+  "HOUSE (R)",
+  "HCRSH (R)",
+  "ATRIUM(R)",
+  "OLC   (R)",
+  "* (R)",		/* BFS MARK */
   "MURKY",
+  "COURT",
+  "BROADCAST",
+  "RECEIVE",
   "\n"
 };
 
@@ -65,7 +127,6 @@ const char *exit_bits[] = {
   "\n"
 };
 
-
 /* SECT_ */
 const char *sector_types[] = {
   "Inside",
@@ -92,6 +153,12 @@ const char *genders[] =
   "Female"
 };
 
+const char *gend_he[] =
+{
+  "it",
+  "he",
+  "she"
+};
 
 /* POS_x */
 const char *position_types[] = {
@@ -128,10 +195,15 @@ const char *player_bits[] = {
   "CRYO",
   "EDITING",
   "NOAUTOTITLE",
+  "JUSTREIN",
   "\n"
 };
 
 const char *player_bits2[] = {
+  "CLAN BIT1",
+  "CLAN BIT2",
+  "CLAN BIT3",
+  "CLAN LEAD",
   "\n"
 };
 
@@ -212,10 +284,10 @@ const char *preference_bits[] = {
   "!GOS",
   "!GTZ",
   "RMFLG",
-  "NOQUIT",
-  "BOUNTY",
-  "UNUSED",
-  "ASSASSIN",
+  "!CLAN",
+  "!CHAT",
+  "!WAR",
+  "!ARENA",
   "A_DIR",
   "A_SAC",
   "D_GOLD",
@@ -233,6 +305,10 @@ const char *preference_bits2[] = {
   "RETIRED",
   "ARENA_RED",
   "ARENA_BLUE",
+  "AFK",
+  "BOUNTY",
+  "ASSASSIN",
+  "NOQUIT",
   "\n"
 };
 
@@ -288,8 +364,8 @@ const char *affected_bits2[] =
   "FREEZING",
   "ACIDED",
   "PROT COLD",
-  "UNUSED",
-  "UNUSED",
+  "BLINK",
+  "HASTE",
   "UNUSED",
   "UNUSED",
   "UNUSED",
@@ -369,8 +445,11 @@ const char *connected_types[] = {
   "Self-Delete 2",
   "Get Attrib",
   "Get Race",
-  "I-EDITING"
-  "R-EDITING"
+  "I-EDITING",
+  "R-EDITING",
+  "Z-EDITING"
+  "Reincarn 1",
+  "Reincarn 2",
   "\n"
 };
 
@@ -388,12 +467,13 @@ const char *where[] = {
   "<worn on feet>       ",
   "<worn on hands>      ",
   "<worn on arms>       ",
-  "<worn as shield>     ",
+  "<worn as a shield>   ",
   "<worn about body>    ",
   "<worn about waist>   ",
   "<worn around wrist>  ",
   "<worn around wrist>  ",
   "<wielded>            ",
+  "<dual-wielded>       ",
   "<held>               ",
   "<worn on back>       ",
   "<worn on face>       "
@@ -413,12 +493,13 @@ const char *equipment_types[] = {
   "Worn on feet",
   "Worn on hands",
   "Worn on arms",
-  "Worn as shield",
+  "Worn as a shield",
   "Worn about body",
   "Worn around waist",
   "Worn around right wrist",
   "Worn around left wrist",
   "Wielded",
+  "Dual-Wielded",
   "Held",
   "Worn on the back",
   "Worn on face",
@@ -674,46 +755,46 @@ const char *fullness[] =
 /* [ch] strength apply (all) */
 const struct str_app_type str_app[41] = {
   {-5, -4, 0, 0},
-  {-5, -4, 3, 1},
-  {-3, -2, 3, 2},
-  {-3, -1, 10, 3},
-  {-2, -1, 25, 4},
-  {-2, -1, 55, 5},
-  {-1, 0, 80, 6},
-  {-1, 0, 90, 7},
-  {0, 0, 100, 8},
-  {0, 0, 100, 9},
-  {0, 0, 115, 10},	/* 10 */
-  {0, 0, 115, 11},
-  {0, 0, 140, 12},
-  {0, 0, 140, 13},
-  {0, 0, 170, 14},
-  {0, 0, 170, 15},
-  {0, 1, 195, 16},
-  {1, 1, 220, 18},
-  {1, 2, 255, 20},	/* 18 */
-  {3, 7, 640, 40},      
-  {3, 8, 700, 40},
-  {4, 9, 810, 40},
-  {4, 10, 970, 40},
-  {5, 11, 1130, 40},
-  {5, 12, 1340, 40},
-  {6, 13, 1550, 40},    /* 25 */
-  {6, 14, 2000, 60},	
-  {7, 15, 2500, 60},
-  {7, 16, 3500, 60},
-  {8, 17, 4000, 60},
-  {8, 18, 5000, 60},	/* 30 */
-  {9, 19, 6000, 60},
-  {9, 20, 7000, 60},
-  {10,20, 8000, 60},
-  {10,20, 9000, 60},
-  {10,20, 9000, 60},	/* 35 */
-  {1, 3, 280, 22},      /* 18/01 */
-  {2, 3, 305, 24},
-  {2, 4, 330, 26},
-  {2, 5, 380, 28},
-  {3, 6, 480, 30}       /* 18/100 (40) */
+  {-5, -4, 30, 10},
+  {-3, -2, 30, 20},
+  {-3, -1, 100, 30},
+  {-2, -1, 250, 40},
+  {-2, -1, 550, 50},
+  {-1, 0, 800, 60},
+  {-1, 0, 900, 70},
+  {0, 0, 1000, 80},
+  {0, 0, 1000, 90},
+  {0, 0, 1150, 100},	/* 10 */
+  {0, 0, 1150, 110},
+  {0, 0, 1400, 120},
+  {0, 0, 1400, 130},
+  {0, 0, 1700, 140},
+  {0, 0, 1700, 150},
+  {0, 1, 1950, 160},
+  {1, 1, 2200, 180},
+  {1, 2, 2550, 200},	/* 18 */
+  {3, 7, 6400, 400},      
+  {3, 8, 7000, 400},
+  {4, 9, 8100, 400},
+  {4, 10, 9700, 400},
+  {5, 11, 11300, 400},
+  {5, 12, 13400, 400},
+  {6, 13, 15500, 400},    /* 25 */
+  {6, 14, 20000, 600},	
+  {7, 15, 25000, 600},
+  {7, 16, 35000, 600},
+  {8, 17, 40000, 600},
+  {8, 18, 50000, 600},	/* 30 */
+  {9, 19, 60000, 600},
+  {9, 20, 70000, 600},
+  {10,20, 80000, 600},
+  {10,20, 90000, 600},
+  {10,20, 90000, 600},	/* 35 */
+  {1, 3, 2800, 220},      /* 18/01 */
+  {2, 3, 3050, 240},
+  {2, 4, 3300, 260},
+  {2, 5, 3800, 280},
+  {3, 6, 4800, 300}       /* 18/100 (40) */
 };
 
 
@@ -1091,6 +1172,21 @@ const char *npc_class_types[] = {
 };
 
 
+const char *align_types[] = {
+   "Evil Incarnate",
+   "Demonic",
+   "Evil",
+   "Sinister",
+   "Nasty",
+   "Neutral",
+   "Helpful",
+   "Nice",
+   "Good",
+   "Saintly",
+   "Godly",
+   "\n"
+};
+
 const int rev_dir[] =
 {
   2,
@@ -1147,3 +1243,11 @@ const char *month_name[17] = {
   "Month of the Ancient Darkness",
   "Month of the Great Evil"
 };
+
+const char * rmode[] =    {
+  "Never resets",
+  "Resets only when deserted",
+  "Always resets",
+  "\n"
+};
+
