@@ -18,6 +18,9 @@
 #include <errno.h>
 
 #include "structs.h"
+#include "rooms.h"
+#include "objs.h"
+#include "class.h"
 #include "utils.h"
 #include "db.h"
 #include "comm.h"
@@ -49,6 +52,12 @@ struct index_data *obj_index;	/* index table for object file	 */
 struct obj_data *obj_proto;	/* prototypes for objs		 */
 int top_of_objt = 0;		/* top of object index table	 */
 
+extern int max_new_objects;
+extern int max_new_rooms;
+extern int max_new_zones;
+extern int max_new_zcmds;
+extern int max_new_mobs;
+
 struct zone_data *zone_table;	/* zone table			 */
 int top_of_zone_table = 0;	/* top element of zone tab	 */
 struct message_list fight_messages[MAX_MESSAGES];	/* fighting messages	 */
@@ -69,6 +78,8 @@ sh_int r_immort_start_room;	/* rnum of immort start room	 */
 sh_int r_frozen_start_room;	/* rnum of frozen start room	 */
 
 char *buglist = NULL;		/* bug list 			 */
+char *idealist = NULL;		/* idea list 			 */
+char *typolist = NULL;		/* typo list 			 */
 char *credits = NULL;		/* game credits			 */
 char *news = NULL;		/* mud news			 */
 char *motd = NULL;		/* message of the day - mortals */
@@ -115,6 +126,8 @@ void renum_zone_table(void);
 void log_zone_error(int zone, int cmd_no, char *message);
 void reset_time(void);
 void clear_char(struct char_data * ch);
+int read_file_to_32strings_alloc(char *filename, char **buf);
+int read_file_to_32strings(char *filename, char **buf);
 
 /* external functions */
 extern struct descriptor_data *descriptor_list;
@@ -129,6 +142,8 @@ void load_banned(void);
 void Read_Invalid_List(void);
 void boot_the_shops(FILE * shop_f, char *filename, int rec_count);
 struct help_index_element *build_help_index(FILE * fl, int *num);
+void remove(char *filename);
+void rename(char *filename, char *filename2);
 
 /* external vars */
 extern int no_specials;
@@ -165,6 +180,9 @@ ACMD(do_reboot)
     file_to_string_alloc(HANDBOOK_FILE, &handbook);
     file_to_string_alloc(BACKGROUND_FILE, &background);
     file_to_string_alloc(BUGLIST_FILE, &buglist);
+    file_to_string_alloc(IDEALIST_FILE, &idealist);
+    file_to_string_alloc(TYPOLIST_FILE, &typolist);
+/*    read_file_to_32strings_alloc(CHANNEL_FILE, channel_bits); */
   } else if (!str_cmp(arg, "wizlist"))
     file_to_string_alloc(WIZLIST_FILE, &wizlist);
   else if (!str_cmp(arg, "immlist"))
@@ -185,8 +203,12 @@ ACMD(do_reboot)
     file_to_string_alloc(POLICIES_FILE, &policies);
   else if (!str_cmp(arg, "handbook"))
     file_to_string_alloc(HANDBOOK_FILE, &handbook);
-  else if (!str_cmp(arg, "handbook"))
+  else if (!str_cmp(arg, "buglist"))
     file_to_string_alloc(BUGLIST_FILE, &buglist);
+  else if (!str_cmp(arg, "idealist"))
+    file_to_string_alloc(IDEALIST_FILE, &idealist);
+  else if (!str_cmp(arg, "typolist"))
+    file_to_string_alloc(TYPOLIST_FILE, &typolist);
   else if (!str_cmp(arg, "background"))
     file_to_string_alloc(BACKGROUND_FILE, &background);
   else if (!str_cmp(arg, "xhelp")) {
@@ -244,6 +266,7 @@ void boot_world(void)
 void boot_db(void)
 {
   int i;
+  extern int no_specials;
 
   log("Boot db -- BEGIN.");
 
@@ -263,6 +286,9 @@ void boot_db(void)
   file_to_string_alloc(HANDBOOK_FILE, &handbook);
   file_to_string_alloc(BACKGROUND_FILE, &background);
   file_to_string_alloc(BUGLIST_FILE, &buglist);
+  file_to_string_alloc(TYPOLIST_FILE, &typolist);
+  file_to_string_alloc(IDEALIST_FILE, &idealist);
+/*  read_file_to_32strings_alloc(CHANNEL_FILE, channel_bits); */
 
   log("Opening help file.");
   if (!(help_fl = fopen(HELP_KWRD_FILE, "r")))
@@ -452,6 +478,7 @@ void index_boot(int mode)
 {
   char *index_filename, *prefix;
   FILE *index, *db_file;
+  int *temp_pt;
   int rec_count = 0;
 
   switch (mode) {
@@ -514,15 +541,27 @@ void index_boot(int mode)
 
   switch (mode) {
   case DB_BOOT_WLD:
-    CREATE(world, struct room_data, rec_count);
+    CREATE(world, struct room_data, rec_count + max_new_rooms);
     break;
   case DB_BOOT_MOB:
+    rec_count += max_new_mobs;
     CREATE(mob_proto, struct char_data, rec_count);
     CREATE(mob_index, struct index_data, rec_count);
+    for (; rec_count >= 0; rec_count--) {
+	CREATE(temp_pt, int, top_of_zone_table + 1);
+	mob_index[rec_count].number = temp_pt;
+	temp_pt = NULL;
+    }
     break;
   case DB_BOOT_OBJ:
+    rec_count += max_new_objects;
     CREATE(obj_proto, struct obj_data, rec_count);
     CREATE(obj_index, struct index_data, rec_count);
+    for (; rec_count >= 0; rec_count--) {
+	CREATE(temp_pt, int, top_of_zone_table + 1);
+	obj_index[rec_count].number = temp_pt;
+	temp_pt = NULL;
+    }
     break;
   case DB_BOOT_ZON:
     CREATE(zone_table, struct zone_data, rec_count);
@@ -645,6 +684,8 @@ void parse_room(FILE * fl, int virtual_nr)
   int t[10], i;
   char line[256], flags[128];
   struct extra_descr_data *new_descr;
+  struct teleport_data *new_tele;
+  struct broadcast_data *new_broad;
 
   sprintf(buf2, "room #%d", virtual_nr);
 
@@ -666,6 +707,7 @@ void parse_room(FILE * fl, int virtual_nr)
     fprintf(stderr, "Format error in room #%d\n", virtual_nr);
     exit(1);
   }
+
   /* t[0] is the zone number; ignored with the zone-file system */
   world[room_nr].room_flags = asciiflag_conv(flags);
   world[room_nr].sector_type = t[2];
@@ -673,6 +715,8 @@ void parse_room(FILE * fl, int virtual_nr)
   world[room_nr].func = NULL;
   world[room_nr].contents = NULL;
   world[room_nr].people = NULL;
+  world[room_nr].tele = NULL;
+  world[room_nr].broad = NULL;
   world[room_nr].light = 0;	/* Zero light sources */
 
   for (i = 0; i < NUM_OF_DIRS; i++)
@@ -702,6 +746,32 @@ void parse_room(FILE * fl, int virtual_nr)
       top_of_world = room_nr++;
       return;
       break;
+    case 'T':
+      if (!get_line(fl, line) || sscanf(line, " %d %d %d %d", t, t + 1, t + 2, t + 3) != 4) {
+	fprintf(stderr, "Format error in room #%d's section T\n", virtual_nr);
+	exit(1);
+      }
+      CREATE(new_tele, struct teleport_data, 1);
+      world[room_nr].tele = new_tele;
+      world[room_nr].tele->targ = t[0];
+      world[room_nr].tele->mask = t[1];
+      world[room_nr].tele->time = t[2];
+/*      if (IS_SET(world[room_nr].tele->mask, TELE_COUNT))
+	 world[room_nr].tele->cnt = t[3]; */
+      world[room_nr].tele->obj = t[3];
+      break;
+    case 'B':
+      if (!get_line(fl, line) || sscanf(line, " %d %d %d", t, t + 1, t + 2) != 3) {
+	fprintf(stderr, "Format error in room #%d's section B\n", virtual_nr);
+	exit(1);
+      }
+      CREATE(new_broad, struct broadcast_data, 1);
+      world[room_nr].broad = new_broad;
+      new_broad = NULL;
+      world[room_nr].broad->channel = t[0];
+      world[room_nr].broad->targ1 = t[1];
+      world[room_nr].broad->targ2 = t[2];
+      break;
     default:
       fprintf(stderr, "%s\n", buf);
       exit(1);
@@ -892,10 +962,6 @@ void parse_simple_mob(FILE *mob_f, int i, int nr)
     mob_proto[i].mob_specials.default_pos = t[1];
     mob_proto[i].player.sex = t[2];
 
-    mob_proto[i].player.class = 0;
-    mob_proto[i].player.weight = 200;
-    mob_proto[i].player.height = 198;
-
     for (j = 0; j < 3; j++)
       GET_COND(mob_proto + i, j) = -1;
 
@@ -928,7 +994,14 @@ void interpret_espec(char *keyword, char *value, int i, int nr)
     RANGE(0, 99);
     mob_proto[i].mob_specials.attack_type = num_arg;
   }
-
+  CASE("Class") {
+	  RANGE(0, ANYCLASS);
+	  mob_proto[i].player.class = num_arg;
+  }	
+  CASE("Race") {
+	  RANGE(0, MAX_NPC_RACE);
+	  mob_proto[i].player.race = num_arg;
+  }
   CASE("Attacks") {
     RANGE(0, 99);
     mob_proto[i].mob_specials.attack_num = num_arg;
@@ -1022,7 +1095,6 @@ void parse_mobile(FILE * mob_f, int nr)
   char f1[128], f2[128], f3[128];
 
   mob_index[i].virtual = nr;
-  mob_index[i].number = 0;
   mob_index[i].func = NULL;
 
   clear_char(mob_proto + i);
@@ -1042,6 +1114,11 @@ void parse_mobile(FILE * mob_f, int nr)
   mob_proto[i].player.long_descr = fread_string(mob_f, buf2);
   mob_proto[i].player.description = fread_string(mob_f, buf2);
   mob_proto[i].player.title = NULL;
+  mob_proto[i].player.class = CLASS_WARRIOR;
+  mob_proto[i].player.race = RACE_NORMAL;
+  mob_proto[i].player.weight = 200;
+  mob_proto[i].player.height = 198;
+
 
   /* *** Numeric data *** */
   get_line(mob_f, line);
@@ -1115,7 +1192,6 @@ char *parse_object(FILE * obj_f, int nr)
   struct extra_descr_data *new_descr;
 
   obj_index[i].virtual = nr;
-  obj_index[i].number = 0;
   obj_index[i].func = NULL;
   obj_proto[i].obj_flags.bitvector = 0;
   obj_proto[i].obj_flags.bitvector2 = 0;
@@ -1200,7 +1276,7 @@ char *parse_object(FILE * obj_f, int nr)
     obj_proto[i].affected[j].modifier = 0;
   }
 
-  strcat(buf2, ", after numeric constants (expecting E/A/#xxx)");
+  strcat(buf2, ", after numeric constants (expecting E/A/F/C/#xxx)");
   j = 0;
 
   for (;;) {
@@ -1234,6 +1310,10 @@ char *parse_object(FILE * obj_f, int nr)
       obj_proto[i].obj_flags.bitvector2 = asciiflag_conv(f2);
       obj_proto[i].obj_flags.bitvector3 = asciiflag_conv(f3);
       break;
+    case 'C':
+      tmpptr = obj_proto[i].attack_verb = fread_string(obj_f, buf2);
+	*tmpptr = LOWER(*tmpptr);
+      break;
     case '$':
     case '#':
       top_of_objt = i++;
@@ -1256,7 +1336,8 @@ void load_zones(FILE * fl, char *zonename)
   static int zone = 0;
   int cmd_no = 0, num_of_cmds = 0, line_num = 0, tmp, error;
   char *ptr, buf[256], zname[256];
-
+  char s1[256];
+  int d[2];
   strcpy(zname, zonename);
 
   while (get_line(fl, buf))
@@ -1271,10 +1352,17 @@ void load_zones(FILE * fl, char *zonename)
 
   line_num += get_line(fl, buf);
 
-  if (sscanf(buf, "#%d", &Z.number) != 1) {
-    fprintf(stderr, "Format error in %s, line %d\n", zname, line_num);
-    exit(0);
+  if (sscanf(buf, "#%d %s %d %d", &Z.number, s1, d, d + 1) == 1) {
+	/* old style */
+	Z.creator = strdup("Anon");
+	Z.lvl_low = 1;
+	Z.lvl_high = LVL_IMMORT;
+  } else {
+	Z.creator = strdup(s1);
+	Z.lvl_low = d[0];
+	Z.lvl_high = d[1];
   }
+
   sprintf(buf2, "beginning of zone #%d", Z.number);
 
   line_num += get_line(fl, buf);
@@ -1283,7 +1371,7 @@ void load_zones(FILE * fl, char *zonename)
   Z.name = str_dup(buf);
 
   line_num += get_line(fl, buf);
-  if (sscanf(buf, " %d %d %d ", &Z.top, &Z.lifespan, &Z.reset_mode) != 3) {
+  if (sscanf(buf, " %d %d %d", &Z.top, &Z.lifespan, &Z.reset_mode) != 3) {
     fprintf(stderr, "Format error in 3-constant line of %s", zname);
     exit(0);
   }
@@ -1388,7 +1476,7 @@ struct char_data *create_char(void)
 
 
 /* create a new mobile from a prototype */
-struct char_data *read_mobile(int nr, int type)
+struct char_data *read_mobile(int nr, int type, int rznum)
 {
   int i;
   struct char_data *mob;
@@ -1421,7 +1509,8 @@ struct char_data *read_mobile(int nr, int type)
   mob->player.time.played = 0;
   mob->player.time.logon = time(0);
 
-  mob_index[i].number++;
+  mob_index[i].number[rznum] += 1;
+  mob->mob_specials.orig_zone = rznum;
 
   return mob;
 }
@@ -1442,7 +1531,7 @@ struct obj_data *create_obj(void)
 
 
 /* create a new object from a prototype */
-struct obj_data *read_object(int nr, int type)
+struct obj_data *read_object(int nr, int type, int rzone)
 {
   struct obj_data *obj;
   int i;
@@ -1465,7 +1554,8 @@ struct obj_data *read_object(int nr, int type)
   obj->next = object_list;
   object_list = obj;
 
-  obj_index[i].number++;
+  obj_index[i].number[rzone] += 1;
+  obj->orig_zone = rzone;
 
   return obj;
 }
@@ -1566,7 +1656,7 @@ void log_zone_error(int zone, int cmd_no, char *message)
 void reset_zone(int zone)
 {
   int cmd_no, last_cmd = 0;
-  struct char_data *mob;
+  struct char_data *mob = NULL;
   struct obj_data *obj, *obj_to;
 
   for (cmd_no = 0; ZCMD.command != 'S'; cmd_no++) {
@@ -1580,8 +1670,8 @@ void reset_zone(int zone)
       break;
 
     case 'M':			/* read a mobile */
-      if (mob_index[ZCMD.arg1].number < ZCMD.arg2) {
-	mob = read_mobile(ZCMD.arg1, REAL);
+      if (mob_index[ZCMD.arg1].number[zone] < ZCMD.arg2) {
+	mob = read_mobile(ZCMD.arg1, REAL, zone);
 	char_to_room(mob, ZCMD.arg3);
 	last_cmd = 1;
       } else
@@ -1589,13 +1679,13 @@ void reset_zone(int zone)
       break;
 
     case 'O':			/* read an object */
-      if (obj_index[ZCMD.arg1].number < ZCMD.arg2) {
+      if (obj_index[ZCMD.arg1].number[zone] < ZCMD.arg2) {
 	if (ZCMD.arg3 >= 0) {
-	  obj = read_object(ZCMD.arg1, REAL);
+	  obj = read_object(ZCMD.arg1, REAL, zone);
 	  obj_to_room(obj, ZCMD.arg3);
 	  last_cmd = 1;
 	} else {
-	  obj = read_object(ZCMD.arg1, REAL);
+	  obj = read_object(ZCMD.arg1, REAL, zone);
 	  obj->in_room = NOWHERE;
 	  last_cmd = 1;
 	}
@@ -1604,8 +1694,8 @@ void reset_zone(int zone)
       break;
 
     case 'P':			/* object to object */
-      if (obj_index[ZCMD.arg1].number < ZCMD.arg2) {
-	obj = read_object(ZCMD.arg1, REAL);
+      if (obj_index[ZCMD.arg1].number[zone] < ZCMD.arg2) {
+	obj = read_object(ZCMD.arg1, REAL, zone);
 	if (!(obj_to = get_obj_num(ZCMD.arg3))) {
 	  ZONE_ERROR("target obj not found");
 	  break;
@@ -1621,8 +1711,8 @@ void reset_zone(int zone)
 	ZONE_ERROR("attempt to give obj to non-existant mob");
 	break;
       }
-      if (obj_index[ZCMD.arg1].number < ZCMD.arg2) {
-	obj = read_object(ZCMD.arg1, REAL);
+      if (obj_index[ZCMD.arg1].number[zone] < ZCMD.arg2) {
+	obj = read_object(ZCMD.arg1, REAL, zone);
 	obj_to_char(obj, mob);
 	last_cmd = 1;
       } else
@@ -1634,11 +1724,11 @@ void reset_zone(int zone)
 	ZONE_ERROR("trying to equip non-existant mob");
 	break;
       }
-      if (obj_index[ZCMD.arg1].number < ZCMD.arg2) {
+      if (obj_index[ZCMD.arg1].number[zone] < ZCMD.arg2) {
 	if (ZCMD.arg3 < 0 || ZCMD.arg3 >= NUM_WEARS) {
 	  ZONE_ERROR("invalid equipment pos number");
 	} else {
-	  obj = read_object(ZCMD.arg1, REAL);
+	  obj = read_object(ZCMD.arg1, REAL, zone);
 	  equip_char(mob, obj, ZCMD.arg3);
 	  last_cmd = 1;
 	}
@@ -1823,6 +1913,10 @@ void store_to_char(struct char_file_u * st, struct char_data * ch)
   if (ch->points.max_mana < 100)
     ch->points.max_mana = 100;
 
+  ch->player.obj_buffer = NULL;
+  ch->player.mob_buf = NULL;
+  ch->player.zone_edit = NOWHERE;
+
   ch->char_specials.carry_weight = 0;
   ch->char_specials.carry_items = 0;
   ch->points.armor = 100;
@@ -2057,6 +2151,10 @@ void free_char(struct char_data * ch)
       free(ch->player_specials->poofin);
     if (ch->player_specials->poofout)
       free(ch->player_specials->poofout);
+    while ((a = GET_ALIASES(ch)) != NULL) {
+	    GET_ALIASES(ch) = (GET_ALIASES(ch))->next;
+	    free_alias(a);
+    }
     free(ch->player_specials);
     if (IS_NPC(ch))
       log("SYSERR: Mob had player_specials allocated!");
@@ -2089,10 +2187,6 @@ void free_char(struct char_data * ch)
   while (ch->affected)
     affect_remove(ch, ch->affected);
 
-  while ((a = GET_ALIASES(ch)) != NULL) {
-    GET_ALIASES(ch) = (GET_ALIASES(ch))->next;
-    free_alias(a);
-  }
   while ((q = GET_QUESTS(ch)) != NULL) {
     GET_QUESTS(ch) = (GET_QUESTS(ch))->next;
     free_quest(q);
@@ -2129,6 +2223,10 @@ void free_room(struct room_data *room)
 	free(this->description);
       free(this);
     }
+  if (room->broad)
+	free(room->broad);
+  if (room->tele)
+	free(room->tele);
 }
 
 /* release memory allocated for an obj struct */
@@ -2203,14 +2301,28 @@ int file_to_string(char *name, char *buf)
 {
   FILE *fl;
   char tmp[128];
+  char tname[80];
 
   *buf = '\0';
+  *tname = '\0';
 
   if (!(fl = fopen(name, "r"))) {
     sprintf(tmp, "Error reading %s", name);
     perror(tmp);
     return (-1);
   }
+  if(feof(fl)) { /*reditmod* if empty file, load the backup*/
+    fclose(fl);
+    fprintf(stderr, "olc: empty file %s, using backup", name);
+    sprintf(tname, "%s.back", name);
+    remove(name);
+    rename(tname, name);
+    if (!(fl = fopen(name, "r"))) {
+      sprintf(tmp, "Error reading %s", name);
+      perror(tmp);
+      return (-1);
+    }
+  }
   do {
     fgets(tmp, 128, fl);
     tmp[strlen(tmp) - 1] = '\0';/* take off the trailing \n */
@@ -2231,8 +2343,41 @@ int file_to_string(char *name, char *buf)
   return (0);
 }
 
+int read_file_to_32strings_alloc(char *filename, char **buf) {
+	int counter = 0;
 
+	while (counter < 32 && *buf && *buf[counter])
+		free(buf[counter]);
 
+	if (read_file_to_32strings(filename, buf) < 0)
+		return -1;
+
+	return 0;
+}
+		
+
+int read_file_to_32strings(char *filename, char **buf) {
+	FILE *file;
+	char temp[256];
+	int counter = 0;
+
+	if (!(file = fopen(filename, "r"))) {
+		sprintf(temp, "Error reading %s", filename);
+		perror(temp);
+		return(-1);
+        }
+	while (counter == 0 && !feof(file)) {
+		get_line(file, temp);
+		buf[counter] = strdup(temp);
+		counter++;
+        }
+	while (counter < 32) {
+		sprintf(temp, "NONE");
+		buf[counter] = strdup(temp);
+		counter++;
+        }
+	return 0;
+}
 
 /* clear some of the the working variables of a char */
 void reset_char(struct char_data * ch)
@@ -2263,7 +2408,7 @@ void reset_char(struct char_data * ch)
     GET_MANA(ch) = 1;
 
   GET_LAST_TELL(ch) = NOBODY;
-  SET_BIT(PRF_FLAGS(ch), PRF_NOQUIT);
+  SET_BIT(PRF2_FLAGS(ch), PRF2_NOQUIT);
 }
 
 
@@ -2339,7 +2484,7 @@ void init_char(struct char_data * ch)
     ch->player.height = number(150, 180);
   }
 
-  ch->points.max_mana = 1000;
+  ch->points.max_mana = 5000;
   ch->points.mana = GET_MAX_MANA(ch);
   ch->points.hit = GET_MAX_HIT(ch);
   ch->points.max_move = 100;
@@ -2369,12 +2514,11 @@ void init_char(struct char_data * ch)
 /* returns the real number of the room with given virtual number */
 int real_room(int virtual)
 {
-  int bot, top, mid;
+/*  int bot, top, mid;
 
   bot = 0;
   top = top_of_world;
 
-  /* perform binary search on world-table */
   for (;;) {
     mid = (bot + top) >> 1;
 
@@ -2388,18 +2532,30 @@ int real_room(int virtual)
       bot = mid + 1;
   }
 }
+Note: reditmod - changed binary search to incremental */
+
+ int nr, found = -1;
+
+  for(nr = 0; nr <= top_of_world; nr++) {
+    if(world[nr].number == virtual) {
+      found = nr;
+      break;
+    }
+  }
+  return(found);
+}
 
 
+/* meditmod */
 
 /* returns the real number of the monster with given virtual number */
 int real_mobile(int virtual)
 {
-  int bot, top, mid;
+/*   int bot, top, mid;
 
   bot = 0;
   top = top_of_mobt;
 
-  /* perform binary search on mob-table */
   for (;;) {
     mid = (bot + top) / 2;
 
@@ -2412,19 +2568,27 @@ int real_mobile(int virtual)
     else
       bot = mid + 1;
   }
-}
+*/
+  int nr, found = -1;
 
+  for(nr = 0; nr <= top_of_mobt; nr++) {
+    if(mob_index[nr].virtual == virtual) {
+      found = nr;
+      break;
+    }
+  }
+  return(found);
+}
 
 
 /* returns the real number of the object with given virtual number */
 int real_object(int virtual)
 {
-  int bot, top, mid;
+/*  int bot, top, mid;
 
   bot = 0;
   top = top_of_objt;
 
-  /* perform binary search on obj-table */
   for (;;) {
     mid = (bot + top) / 2;
 
@@ -2437,6 +2601,16 @@ int real_object(int virtual)
     else
       bot = mid + 1;
   }
+*/
+  int nr, found = -1;
+
+  for(nr = 0; nr <= top_of_objt; nr++) {
+    if(obj_index[nr].virtual == virtual) {
+      found = nr;
+      break;
+    }
+  }
+  return(found);
 }
 
 /* the functions */
