@@ -14,6 +14,9 @@
 #include <assert.h>
 
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
+#include "rooms.h"
 #include "utils.h"
 #include "spells.h"
 #include "comm.h"
@@ -21,6 +24,7 @@
 #include "interpreter.h"
 #include "db.h"
 #include "screen.h"
+#include "fight.h"
 
 /* Structures */
 struct char_data *combat_list = NULL;	/* head of l-list of fighting chars */
@@ -34,13 +38,14 @@ extern int pk_allowed;		/* see config.c */
 extern int auto_save;		/* see config.c */
 extern int max_exp_gain;	/* see config.c */
 extern char *dirs[];		/* see constants.c */
+extern char *dirs2[];		/* see constants.c */
 extern int rev_dir[];		/* see constants.c */
+extern struct char_data *mob_proto;
+
 /* External procedures */
 char *fread_action(FILE * fl, int nr);
 char *fread_string(FILE * fl, char *error);
 void stop_follower(struct char_data * ch);
-bool CAN_DAMAGE(struct char_data * ch, struct char_data * vict);
-bool CAN_MURDER(struct char_data * ch, struct char_data * vict);
 
 ACMD(do_flee);
 void hit(struct char_data * ch, struct char_data * victim, int type);
@@ -231,7 +236,7 @@ void make_corpse(struct char_data * ch)
   struct obj_data *create_money(int amount);
 
   corpse = create_obj();
-  corpse->item_number = NOTHING;
+  corpse->item_number = 0;
   corpse->in_room = NOWHERE;
   corpse->name = str_dup("corpse");
 
@@ -241,6 +246,7 @@ void make_corpse(struct char_data * ch)
   sprintf(buf2, "the corpse of %s", GET_NAME(ch));
   corpse->short_description = str_dup(buf2);
 
+  corpse->orig_zone = world[ch->in_room].zone;
   GET_OBJ_TYPE(corpse) = ITEM_CONTAINER;
   GET_OBJ_WEAR(corpse) = 0;
   GET_OBJ_EXTRA(corpse) = ITEM_NODONATE;
@@ -434,7 +440,7 @@ char *replace_string(char *str, char *weapon_singular, char *weapon_plural)
 
 /* message for doing damage with a weapon */
 void dam_message(int dam, struct char_data * ch, struct char_data * victim,
-		      int w_type)
+		      int w_type, struct obj_data *obj)
 {
   char *buf;
   int msgnum;
@@ -448,105 +454,105 @@ void dam_message(int dam, struct char_data * ch, struct char_data * victim,
     /* use #w for singular (i.e. "slash") and #W for plural (i.e. "slashes") */
 
     {
-      "$n tries to #w $N, but misses.",	/* 0: 0     */
-      "You try to #w $N, but miss.",
-      "$n tries to #w you, but misses."
+      "$n tries to #w $N with $p, but misses.",	/* 0: 0     */
+      "You try to #w $N with $p, but miss.",
+      "$n tries to #w you with $p, but misses."
     },
 
     {
-      "$n tickles $N as $e #W $M.",	/* 1: 1..2  */
-      "You tickle $N as you #w $M.",
-      "$n tickles you as $e #W you."
+      "$n tickles $N as $e #W $M with $p.",	/* 1: 1..2  */
+      "You tickle $N as you #w $M with $p.",
+      "$n tickles you as $e #W you with $p."
     },
 
     {
-      "$n barely #W $N.",		/* 2: 3..4  */
-      "You barely #w $N.",
-      "$n barely #W you."
+      "$n barely #W $N with $p.",		/* 2: 3..4  */
+      "You barely #w $N with $p.",
+      "$n barely #W you with $p."
     },
 
     {
-      "$n #W $N.",			/* 3: 5..6  */
-      "You #w $N.",
-      "$n #W you."
+      "$n #W $N with $p.",			/* 3: 5..6  */
+      "You #w $N with $p.",
+      "$n #W you with $p."
     },
 
     {
-      "$n #W $N hard.",			/* 4: 7..10  */
-      "You #w $N hard.",
-      "$n #W you hard."
+      "$n #W $N hard with $p.",			/* 4: 7..10  */
+      "You #w $N hard with $p.",
+      "$n #W you hard with $p."
     },
 
     {
-      "$n #W $N very hard.",		/* 5: 11..14  */
-      "You #w $N very hard.",
-      "$n #W you very hard."
+      "$n #W $N very hard with $p.",		/* 5: 11..14  */
+      "You #w $N very hard with $p.",
+      "$n #W you very hard with $p."
     },
 
     {
-      "$n #W $N extremely hard.",	/* 6: 15..19  */
-      "You #w $N extremely hard.",
-      "$n #W you extremely hard."
+      "$n #W $N extremely hard with $p.",	/* 6: 15..19  */
+      "You #w $N extremely hard with $p.",
+      "$n #W you extremely hard with $p."
     },
 
     {
-      "$n mauls $N to smithereens with $s #w.",	/* 7: 19..23 */
-      "You maul $N to smithereens with your #w.",
-      "$n mauls you to smithereens with $s #w."
+      "With $p, $n mauls $N to smithereens with $s #w.",	/* 7: 19..23 */
+      "With $p, you maul $N to smithereens with your #w.",
+      "With $p, $n mauls you to smithereens with $s #w."
     },
 
     {
-      "$n decimates $N with $s brutal #w.",	/* 8: 23..28 */
-      "You decimate $N with your brutal #w.",
-      "$n decimates you with $s brutal #w."
+      "With $p, $n decimates $N with a brutal #w.",	/* 8: 23..28 */
+      "With $p, you decimate $N with a brutal #w.",
+      "With $p, $n decimates you with a brutal #w."
     },
 
     {
-      "$n devastates $N with $s #w.",	/* 9: 28..32 */
-      "Your #w totally devastates $N.",
-      "$n devastates you fully with $s #w."
+      "With a mighty #w, $n devastates $N with $p.",	/* 9: 28..32 */
+      "You totally devastate $N with $p.",
+      "$n devastates you fully with $p."
     },
 
     {
-      "$n maims $N with $s #w.",	/* 10: 32..36 */
-      "You maim $N with your #w.",
-      "$n maims you with $s #w."
+      "$n #W $N with $p and maims $M.",	/* 10: 32..36 */
+      "You #w $N with $p and maim $M.",
+      "$n #W you with $p and maims you."
     },
 
     {
-      "$n #W and mutilates $N.",	/* 11: 36..44 */
-      "You #w and mutilate $N.",
-      "$n #W and mutilates you."
+      "$n #W and mutilates $N with $p.",	/* 11: 36..44 */
+      "You #w and mutilate $N with $p.",
+      "$n #W and mutilates you with $p."
     },
 
     {
-      "$n massacres $N to small fragments with $s #w.",	/* 12: 44..50 */
-      "You massacre $N to small fragments with your #w.",
-      "$n massacres you to small fragments with $s #w."
+      "#wing with $p, $n massacres $N to small fragments.",	/* 12: 44..50 */
+      "#wing with $p, you massacre $N to small fragments.",
+      "#wing with $p, $n massacres you to small fragments."
     },
 
     {
-      "$n's #w critically DEMOLISHES $N.",	/* 13: 50..100 */
-      "Your #w critically DEMOLISHES $N.",
-      "$n's #w critically DEMOLISHES you."
+      "$n's #w with $p critically DEMOLISHES $N.",	/* 13: 50..100 */
+      "Your #w with $p critically DEMOLISHES $N.",
+      "$n's #w with $p critically DEMOLISHES you."
     },
 
     {
-      "$n's #w ANNIHILATES $N's vitals.",	/* 14: 100..150 */
-      "Your #w ANNIHILATES $N's vitals.",
-      "$n's #w ANNIHILATES your vitals."
+      "With $p, $n's #w ANNIHILATES $N's vitals.",	/* 14: 100..150 */
+      "With $p, your #w ANNIHILATES $N's vitals.",
+      "With $p, $n's #w ANNIHILATES your vitals."
     },
  
     {
-      "$n OBLITERATES $N with $s deadly #w!!",	/* 15: 150..250   */
-      "You OBLITERATE $N with your deadly #w!!",
-      "$n OBLITERATES you with $s deadly #w!!"
+      "$n OBLITERATES $N with a deadly #w, using $p!!",	/* 15: 150..250   */
+      "You OBLITERATE $N with a deadly #w, using $p!!",
+      "$n OBLITERATES you with a deadly #w, using $p!!"
     },
 
     {
-      "$n OBLITERATES $N with $s LEATHAL #w!!",	/* 16: > 250   */
-      "You DISINTEGRATES $N with your LEATHAL #w!!",
-      "$n DISINTEGRATES you with $s LEATHAL #w!!"
+      "$n uses $p to DISINTERATE $N with a LETHAL #w!!",	/* 16: > 250   */
+      "You use $p to DISINTEGRATE $N with a LETHAL #w!!",
+      "$n uses $p to DISINTEGRATE you with a LETHAL #w!!"
     }
 
   };
@@ -575,25 +581,24 @@ void dam_message(int dam, struct char_data * ch, struct char_data * victim,
   /* damage message to onlookers */
   buf = replace_string(dam_weapons[msgnum].to_room,
 	  attack_hit_text[w_type].singular, attack_hit_text[w_type].plural);
-  act(buf, FALSE, ch, NULL, victim, TO_NOTVICT);
-
+  act(buf, FALSE, ch, obj, victim, TO_NOTVICT);
 
   /* damage message to damager */
   send_to_char(CCYEL(ch, C_CMP), ch);
   buf = replace_string(dam_weapons[msgnum].to_char,
 	  attack_hit_text[w_type].singular, attack_hit_text[w_type].plural);
-  if (GET_LEVEL(ch) >= 20) 
+  if (GET_LEVEL(ch) >= 22) 
 	  sprintf(buf, "%s  You do %s%d%s points of damage.", buf, CCRED(ch, C_CMP), dam, CCYEL(ch, C_CMP));
-  act(buf, FALSE, ch, NULL, victim, TO_CHAR);
+  act(buf, FALSE, ch, obj, victim, TO_CHAR);
   send_to_char(CCNRM(ch, C_CMP), ch);
 
   /* damage message to damagee */
   send_to_char(CCRED(victim, C_CMP), victim);
   buf = replace_string(dam_weapons[msgnum].to_victim,
 	  attack_hit_text[w_type].singular, attack_hit_text[w_type].plural);
-  if (GET_LEVEL(ch) >= 20) 
+  if (GET_LEVEL(ch) >= 18) 
 	  sprintf(buf, "%s  You got hit for %s%d%s points of damage.", buf, CCYEL(ch, C_CMP), dam, CCRED(ch, C_CMP));
-  act(buf, FALSE, ch, NULL, victim, TO_VICT | TO_SLEEP);
+  act(buf, FALSE, ch, obj, victim, TO_VICT | TO_SLEEP);
   send_to_char(CCNRM(victim, C_CMP), victim);
 }
 
@@ -603,14 +608,12 @@ void dam_message(int dam, struct char_data * ch, struct char_data * victim,
  *  C3.0: Also used for weapon damage on miss and death blows
  */
 int skill_message(int dam, struct char_data * ch, struct char_data * vict,
-		      int attacktype)
+		      int attacktype, struct obj_data *weap)
 {
   char buf[256];
   int i, j, nr;
   struct message_type *msg;
 
-  struct obj_data *weap = GET_EQ(ch, WEAR_WIELD);
-
   for (i = 0; i < MAX_MESSAGES; i++) {
     if (fight_messages[i].a_type == attacktype) {
       nr = dice(1, fight_messages[i].number_of_attacks);
@@ -618,47 +621,59 @@ int skill_message(int dam, struct char_data * ch, struct char_data * vict,
 	msg = msg->next;
 
       if (!IS_NPC(vict) && (GET_LEVEL(vict) >= LVL_IMMORT)) {
-	act(msg->god_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
-	act(msg->god_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT);
-	act(msg->god_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
+	if (msg->god_msg.attacker_msg)
+		act(msg->god_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
+	if (msg->god_msg.victim_msg)
+		act(msg->god_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT);
+	if (msg->god_msg.room_msg)
+		act(msg->god_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
       } else if (dam != 0) {
 	if (GET_POS(vict) == POS_DEAD) {
-	  send_to_char(CCYEL(ch, C_CMP), ch);
-	  act(msg->die_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
-	  send_to_char(CCNRM(ch, C_CMP), ch);
-
-	  send_to_char(CCRED(vict, C_CMP), vict);
-	  act(msg->die_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
-	  send_to_char(CCNRM(vict, C_CMP), vict);
-
-	  act(msg->die_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
+	  if (msg->die_msg.attacker_msg) {
+		  send_to_char(CCYEL(ch, C_CMP), ch);
+		  act(msg->die_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
+		  send_to_char(CCNRM(ch, C_CMP), ch);
+	  }
+	  if (msg->die_msg.victim_msg) {
+		  send_to_char(CCRED(vict, C_CMP), vict);
+		  act(msg->die_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
+		  send_to_char(CCNRM(vict, C_CMP), vict);
+	  }
+	  if (msg->die_msg.room_msg) 
+		  act(msg->die_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
 	} else {
-	  send_to_char(CCYEL(ch, C_CMP), ch);
-  	  strcpy(buf, msg->hit_msg.attacker_msg);
-          if (GET_LEVEL(ch) >= 20)
-   	  	sprintf(buf, "%s  You do %s%d%s points of damage.", buf, CCRED(ch, C_CMP), dam, CCYEL(ch, C_CMP));
-	  act(buf, FALSE, ch, weap, vict, TO_CHAR);
-	  send_to_char(CCNRM(ch, C_CMP), ch);
-
-	  send_to_char(CCRED(vict, C_CMP), vict);
-	  strcpy(buf, msg->hit_msg.victim_msg);
-	  if (GET_LEVEL(ch) >= 20) 
-		  sprintf(buf, "%s  You take %s%d%s points of damage.", buf, CCYEL(ch, C_CMP), dam, CCRED(ch, C_CMP));
-	  act(buf, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
-	  send_to_char(CCNRM(vict, C_CMP), vict);
-
-	  act(msg->hit_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
+	  if (msg->hit_msg.attacker_msg) {
+		  send_to_char(CCYEL(ch, C_CMP), ch);
+	  	  strcpy(buf, msg->hit_msg.attacker_msg);
+	          if (GET_LEVEL(ch) >= 20 && attacktype != 399)
+	   	  	sprintf(buf, "%s  You do %s%d%s points of damage.", buf, CCRED(ch, C_CMP), dam, CCYEL(ch, C_CMP));
+		  act(buf, FALSE, ch, weap, vict, TO_CHAR);
+		  send_to_char(CCNRM(ch, C_CMP), ch);
+	  }
+	  if (msg->hit_msg.victim_msg) {
+		  send_to_char(CCRED(vict, C_CMP), vict);
+		  strcpy(buf, msg->hit_msg.victim_msg);
+		  if (GET_LEVEL(ch) >= 20 && attacktype != 399) 
+			  sprintf(buf, "%s  You take %s%d%s points of damage.", buf, CCYEL(ch, C_CMP), dam, CCRED(ch, C_CMP));
+		  act(buf, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
+		  send_to_char(CCNRM(vict, C_CMP), vict);
+	  }
+	  if (msg->hit_msg.room_msg)
+		  act(msg->hit_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
 	}
       } else if (ch != vict) {	/* Dam == 0 */
-	send_to_char(CCYEL(ch, C_CMP), ch);
-	act(msg->miss_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
-	send_to_char(CCNRM(ch, C_CMP), ch);
-
-	send_to_char(CCRED(vict, C_CMP), vict);
-	act(msg->miss_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
-	send_to_char(CCNRM(vict, C_CMP), vict);
-
-	act(msg->miss_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
+	  if (msg->miss_msg.attacker_msg) {
+		send_to_char(CCYEL(ch, C_CMP), ch);
+		act(msg->miss_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
+		send_to_char(CCNRM(ch, C_CMP), ch);
+	  }
+	  if (msg->miss_msg.victim_msg) {
+		send_to_char(CCRED(vict, C_CMP), vict);
+		act(msg->miss_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
+		send_to_char(CCNRM(vict, C_CMP), vict);
+	  }
+	  if (msg->miss_msg.room_msg)
+		act(msg->miss_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
       }
       return 1;
     }
@@ -668,9 +683,10 @@ int skill_message(int dam, struct char_data * ch, struct char_data * vict,
 
 
 void damage(struct char_data * ch, struct char_data * victim, int dam,
-	    int attacktype)
+	    int attacktype, struct obj_data *obj)
 {
   int exp;
+  bool freeflag = FALSE;
 
   if (GET_POS(victim) <= POS_DEAD) {
     log("SYSERR: Attempt to damage a corpse.");
@@ -721,11 +737,10 @@ void damage(struct char_data * ch, struct char_data * victim, int dam,
   if ((attacktype >= 0) && (IS_WEAPON(attacktype) || IS_MONK(ch))) {
 	if (number(1, 101) <= GET_SKILL(ch, SKILL_ENH_DAMAGE)) {
 		dam += lvD5(ch);
-		dam += number(1, dam / 2);
 	}
   }
   if (AFF_FLAGGED(ch, AFF_DEATHDANCE))
-	dam += number(1, dam / 2);
+	dam += number(1, dam / 4);
   dam = MAX(MIN(dam, 500), 0);
 
   /*
@@ -740,14 +755,21 @@ void damage(struct char_data * ch, struct char_data * victim, int dam,
    * dam_message. Otherwise, always send a dam_message.
    */
 
+   if (!obj) {
+	exp = MAX(39, MIN(56, attacktype + 40));
+	obj = read_object(exp, VIRTUAL, 0);
+	obj_to_room(obj, 0);
+	exp = 0;
+	freeflag = TRUE;
+   }
    if (!IS_WEAPON(attacktype))
-      skill_message(dam, ch, victim, attacktype);
+      skill_message(dam, ch, victim, attacktype, obj);
     else {
       if (GET_POS(victim) == POS_DEAD || dam == 0) {
-        if (!skill_message(dam, ch, victim, attacktype))
-  	  dam_message(dam, ch, victim, attacktype);
+        if (!skill_message(dam, ch, victim, attacktype, obj))
+  	  dam_message(dam, ch, victim, attacktype, obj);
       } else
-         dam_message(dam, ch, victim, attacktype);
+         dam_message(dam, ch, victim, attacktype, obj);
       if (!CAN_DAMAGE(ch, victim))
 	dam = (GET_HIT(victim) == GET_MAX_HIT(victim));
     }
@@ -755,6 +777,11 @@ void damage(struct char_data * ch, struct char_data * victim, int dam,
   if (ch != victim)
 	  gain_exp(ch, MAX(0, (GET_LEVEL(victim) - lvD2(ch)) * dam));
 
+  if (freeflag) {
+	extract_obj(obj);
+	freeflag = FALSE;
+  }
+
   update_pos(victim);
 
   /* Use send_to_char -- act() doesn't send message if you are DEAD. */
@@ -872,6 +899,10 @@ void hit(struct char_data * ch, struct char_data * victim, int type)
   mprog_hitprcnt_trigger(ch, FIGHTING(ch));
   mprog_fight_trigger(ch, FIGHTING(ch));
 
+  if (type == SKILL_DUAL_WIELD) {
+	wielded = GET_EQ(ch, WEAR_DUALWIELD);
+	type = TYPE_UNDEFINED;
+  }
   if (wielded && GET_OBJ_TYPE(wielded) == ITEM_WEAPON)
     w_type = GET_OBJ_VAL(wielded, 3) + TYPE_HIT;
   else if (wielded && GET_OBJ_TYPE(wielded) == ITEM_FIREWEAPON)
@@ -907,9 +938,9 @@ void hit(struct char_data * ch, struct char_data * victim, int type)
   if ((((diceroll < 20) && AWAKE(victim)) &&
        ((diceroll == 1) || ((calc_thaco - diceroll) > victim_ac)))) {
     if (type == SKILL_BACKSTAB)
-      damage(ch, victim, 0, SKILL_BACKSTAB);
+      damage(ch, victim, 0, SKILL_BACKSTAB, wielded);
     else
-      damage(ch, victim, 0, w_type);
+      damage(ch, victim, 0, w_type, wielded);
   } else {
     if (number(1, 121) <= GET_SKILL(victim, SKILL_PARRY)) {
 	act("$N parries your attack.", FALSE, ch, 0, victim, TO_CHAR);
@@ -917,10 +948,12 @@ void hit(struct char_data * ch, struct char_data * victim, int type)
     } else if (AFF2_FLAGGED(victim, AFF2_BLINK) && 
 	       (number(1, 50) <= GET_LEVEL(victim))) {
 		act("$N blinks away from your attack.", FALSE, ch, 0, victim, TO_CHAR);
+		act("$N blinks out of $n's way.", FALSE, ch, 0, victim, TO_NOTVICT);
 	        act("You blink out of $n's way.", FALSE, ch, 0, victim, TO_VICT);
     } else if (AFF2_FLAGGED(victim, AFF2_MIRRORIMAGE) &&
 	       (number(1, 40) > GET_INT(ch)) && (number(1, 40) <= GET_INT(victim))) {
-		act("One of $N's false images dissipate and is instantly replaced!", FALSE, ch, 0, victim, TO_CHAR);
+		act("One of $N's false images dissipates and is instantly replaced!", FALSE, ch, 0, victim, TO_CHAR);
+		act("One of $N's false images takes the blow from $n and is instantly replaced!", FALSE, ch, 0, victim, TO_NOTVICT);
 	        act("One of your images takes the blow from $n and is replaced by another image.", FALSE, ch, 0, victim, TO_VICT);
     } else {
     /* okay, we know the guy has been hit.  now calculate damage. */
@@ -951,9 +984,9 @@ void hit(struct char_data * ch, struct char_data * victim, int type)
 
     if (type == SKILL_BACKSTAB) {
       dam *= backstab_mult[(int) GET_LEVEL(ch)];
-      damage(ch, victim, dam, SKILL_BACKSTAB);
+      damage(ch, victim, dam, SKILL_BACKSTAB, wielded);
     } else
-      damage(ch, victim, dam, w_type);
+      damage(ch, victim, dam, w_type, wielded);
     }
   }
 }
@@ -963,12 +996,14 @@ void perform_violence(void)
 {
   struct char_data *ch;
   extern struct index_data *mob_index;
-  unsigned int attacks = 1;
+  unsigned int attacks, dual_wield_flag;
 
   for (ch = combat_list; ch; ch = next_combat_list) {
     next_combat_list = ch->next_fighting;
 
     attacks = 1;
+    dual_wield_flag = 0;
+
     if (IS_NPC(ch)) 
         attacks = dice(1, MAX(1, ch->mob_specials.attack_num));
     else { /* PC */
@@ -976,16 +1011,20 @@ void perform_violence(void)
 		attacks++;
 	if (number(1, 101) <= GET_SKILL(ch, SKILL_THIRD_ATTACK))
 		attacks++;
-	if ((IS_WARRIOR(ch) && GET_NUM_OF_CLASS(ch) == 1) || 
+	if ((IS_WARRIOR(ch) && GET_NUM_OF_CLASS(ch) == 1) || IS_AVATAR(ch) ||
 	     IS_SAMURAI(ch) || IS_BARBARIAN(ch) || IS_MONK(ch))
 		attacks += MAX(0, (lvD8(ch) - 2));
 	if (IS_MONK(ch))
 		attacks++;
-	if (IS_AVATAR(ch))
-		attacks += lvD8(ch);
-	if (AFF_FLAGGED(ch, AFF_DEATHDANCE))
-		attacks += number(lvD3(ch), lvD2(ch));
+	if (AFF_FLAGGED(ch, AFF_DEATHDANCE) && !AFF2_FLAGGED(ch, AFF2_HASTE))
+		attacks += dice(1,lvD3(ch)) + dice(1, lvD3(ch));
+        if (AFF2_FLAGGED(ch, AFF2_HASTE))
+		attacks *= 2;
+	if ( (number(1, 101) <= GET_SKILL(ch, SKILL_DUAL_WIELD)) &&
+		(GET_EQ(ch, WEAR_DUALWIELD)) )
+		dual_wield_flag = 1;
     }
+    attacks = MIN(10, attacks);
 
     while(attacks-- > 0) {
       if (FIGHTING(ch) == NULL || ch->in_room != FIGHTING(ch)->in_room) {
@@ -1007,7 +1046,11 @@ void perform_violence(void)
         send_to_char("You can't fight while sitting!!\r\n", ch);
         continue;
       }
-      hit(ch, FIGHTING(ch), TYPE_UNDEFINED);
+      if ((dual_wield_flag == 1) && (attacks < 2)) {
+		hit(ch, FIGHTING(ch), SKILL_DUAL_WIELD);
+		dual_wield_flag = 0;
+      } else
+	      hit(ch, FIGHTING(ch), TYPE_UNDEFINED);
     }
     if (MOB_FLAGGED(ch, MOB_SPEC) && mob_index[GET_MOB_RNUM(ch)].func != NULL)
       (mob_index[GET_MOB_RNUM(ch)].func) (ch, ch, 0, "");
@@ -1016,7 +1059,7 @@ void perform_violence(void)
 
 bool range_hit(struct char_data *ch, struct char_data * victim, struct obj_data *obj)
 {
-  int w_type = 315, victim_ac, calc_thaco, dam, diceroll;
+  int w_type = 315, victim_ac, calc_thaco, dam = 0, diceroll;
   extern int thaco[NUM_CLASSES][LVL_IMPL+1];
   extern struct str_app_type str_app[];
   extern struct dex_app_type dex_app[];
@@ -1072,7 +1115,7 @@ bool range_hit(struct char_data *ch, struct char_data * victim, struct obj_data
             dam += number(0, (GET_OBJ_WEIGHT(obj) / 5));    
     }
     dam = MAX(1, dam);          /* at least 1 hp damage min per hit */
-    damage(ch, victim, dam, w_type);
+    damage(ch, victim, dam, w_type, obj);
     return TRUE;
   }
   return FALSE;
@@ -1085,26 +1128,23 @@ bool fire_at_char(struct char_data *ch, struct char_data * list, struct obj_data
 
   while (vict) {	/* while there's a victim */
     /* check to hit */
-    if (!pk_allowed && !IS_NPC(vict) && !IS_NPC(ch) && !CAN_MURDER(ch, vict))
-		send_to_char("A strange wall of force protects you!!!\r\n", vict);
-    else {
-         if (range_hit(ch, vict, obj)) {
-		if (vict) {
-		  sprintf(msgbuf, "$p flew from the %s.", dirs[rev_dir[dir]]);
-		  act(msgbuf, FALSE, vict, obj, 0, TO_CHAR);
-		  if (IS_NPC(vict) && !IS_NPC(ch) && GET_POS(ch) > POS_STUNNED) {
-			  SET_BIT(MOB_FLAGS(vict), MOB_MEMORY);
-			  remember(vict, ch);
-			  HUNTING(vict) = ch;
-		  }
-		}
- 		return FALSE;
-	}
-    }
-    sprintf(msgbuf, "$p flies past you coming from the %s.", dirs[rev_dir[dir]]);
+    sprintf(msgbuf, "$p just flew in from %s.", dirs2[rev_dir[dir]]);
     act(msgbuf, FALSE, vict, obj, 0, TO_CHAR);
-    if (!(vict = vict->next_in_room))
-	return TRUE;
+    if (ROOM_FLAGGED(vict->in_room, ROOM_PEACEFUL) || 
+	(!pk_allowed && !IS_NPC(vict) && !IS_NPC(ch) && !CAN_MURDER(ch, vict)))
+		send_to_char("A strange wall of force protects you!!!\r\n", vict);
+    else if (range_hit(ch, vict, obj)) {
+	if (vict) {
+	  if (IS_NPC(vict) && !IS_NPC(ch) && GET_POS(ch) > POS_STUNNED) {
+		  SET_BIT(MOB_FLAGS(vict), MOB_MEMORY);
+		  remember(vict, ch);
+		  HUNTING(vict) = ch;
+	  }
+	}
+	return FALSE;
+    }
+    send_to_char("It flies past you harmlessly.", vict);
+    vict = vict->next_in_room;
   }
   return TRUE;	/* missed everyone */
 }
