@@ -15,6 +15,9 @@
 #include <assert.h>
 
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
+#include "rooms.h"
 #include "utils.h"
 #include "comm.h"
 #include "db.h"
@@ -24,6 +27,7 @@
 
 /* external vars */
 extern int top_of_world;
+extern int pulse;
 extern struct room_data *world;
 extern struct obj_data *object_list;
 extern struct char_data *character_list;
@@ -81,6 +85,37 @@ int isname(char *str, char *namelist)
   }
 }
 
+int myisname(char *str, char *namelist)
+{
+  register char *curname, *curstr;
+
+  curname = namelist;
+  for (;;) {
+    for (curstr = str;; curstr++, curname++) {
+/*      if (!*curstr && !isalpha(*curname))
+	return (1);
+
+      if (!*curname)
+	return (0);
+
+      if (!*curstr || *curname == ' ')
+	break;
+*/
+      if (!*curstr)
+	return (1);
+
+      if (LOWER(*curstr) != LOWER(*curname))
+	break;
+    }
+
+    /* skip to next name */
+
+    for (; isalpha(*curname); curname++);
+    if (!*curname)
+      return (0);
+    curname++;			/* first char of new name */
+  }
+}
 
 
 void affect_modify(struct char_data * ch, byte loc, sbyte mod, long bitv,
@@ -393,6 +428,22 @@ void char_to_room(struct char_data * ch, int room)
       if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_LIGHT)) == ITEM_LIGHT)
 	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_LIGHT), 2))	/* Light ON */
 	  world[room].light++;
+
+/* --------------------- NEW CODE added below ------------------ */
+/* the rest of this procedure hasn't been changed */
+
+    if (!IS_NPC(ch) && world[ch->in_room].tele != NULL) {
+      if (world[ch->in_room].tele->cnt > 0 && world[ch->in_room].tele->time == 0) {
+      /* this is a teleport countdown room */
+        world[ch->in_room].tele->time = pulse + world[ch->in_room].tele->cnt; /* now round up */
+        if (world[ch->in_room].tele->time % 10)
+           world[ch->in_room].tele->time += 10 - (world[ch->in_room].tele->time % 10);
+        if (world[ch->in_room].tele->time > 2400) {
+           world[ch->in_room].tele->time = world[ch->in_room].tele->cnt;  
+		/* start of next day */
+        }
+      }
+    }
   }
 }
 
@@ -777,13 +828,21 @@ void extract_obj(struct obj_data * obj)
 
   REMOVE_FROM_LIST(obj, object_list, next);
 
-  if (GET_OBJ_RNUM(obj) >= 0)
-    (obj_index[GET_OBJ_RNUM(obj)].number)--;
+/*
+    if (GET_OBJ_RNUM(obj) >= 0)
+	if (obj->worn_by)
+	    (obj_index[GET_OBJ_RNUM(obj)].number[world[obj->worn_by->in_room].zone]) -= 1;
+	else if (obj->carried_by)
+	    (obj_index[GET_OBJ_RNUM(obj)].number[world[obj->carried_by->in_room].zone]) -= 1;
+	else
+	    (obj_index[GET_OBJ_RNUM(obj)].number[0]) -= 1;
+*/
+  (obj_index[GET_OBJ_RNUM(obj)].number[obj->orig_zone]) -= 1;
+  if (obj_index[GET_OBJ_RNUM(obj)].number[obj->orig_zone] < 0) 
+	obj_index[GET_OBJ_RNUM(obj)].number[obj->orig_zone] = 0;
   free_obj(obj);
 }
 
-
-
 void update_object(struct obj_data * obj, int use)
 {
   if (GET_OBJ_TIMER(obj) > 0)
@@ -877,6 +936,9 @@ void extract_char(struct char_data * ch)
   if (FIGHTING(ch))
     stop_fighting(ch);
 
+  if (GET_MOB_RNUM(ch) > -1)		/* if mobile */
+      mob_index[GET_MOB_RNUM(ch)].number[ch->mob_specials.orig_zone] -= 1;
+
   for (k = combat_list; k; k = temp) {
     temp = k->next_fighting;
     if (FIGHTING(k) == ch)
@@ -895,8 +957,6 @@ void extract_char(struct char_data * ch)
     save_char(ch, NOWHERE);
     Crash_delete_crashfile(ch);
   } else {
-    if (GET_MOB_RNUM(ch) > -1)		/* if mobile */
-      mob_index[GET_MOB_RNUM(ch)].number--;
     clearMemory(ch);		/* Only NPC's can have memory */
     free_char(ch);
     freed = 1;
@@ -925,7 +985,8 @@ struct char_data *get_player_vis(struct char_data * ch, char *name, int inroom)
 
   for (i = character_list; i; i = i->next)
     if (!IS_NPC(i) && (!inroom || i->in_room == ch->in_room) &&
-	!str_cmp(i->player.name, name) && CAN_SEE(ch, i))
+/*	!str_cmp(i->player.name, name) && CAN_SEE(ch, i)) */
+	myisname(i->player.name, name) && CAN_SEE(ch, i))
       return i;
 
   return NULL;
@@ -949,11 +1010,11 @@ struct char_data *get_char_room_vis(struct char_data * ch, char *name)
     return get_player_vis(ch, tmp, 1);
 
   for (i = world[ch->in_room].people; i && j <= number; i = i->next_in_room)
-    if (isname(tmp, i->player.name))
+    if (myisname(tmp, i->player.name))
       if (CAN_SEE(ch, i))
-	if (++j == number)
-	  return i;
-
+	if (!MOB_FLAGGED(i, MOB_ETHEREAL) || GET_LEVEL(ch) > LVL_IMMORT)
+		if (++j == number)
+		  return i;
   return NULL;
 }
 
@@ -974,7 +1035,8 @@ struct char_data *get_char_vis(struct char_data * ch, char *name)
     return get_player_vis(ch, tmp, 0);
 
   for (i = character_list; i && (j <= number); i = i->next)
-    if (isname(tmp, i->player.name) && CAN_SEE(ch, i))
+    if (myisname(tmp, i->player.name) && CAN_SEE(ch, i) && 
+	(!MOB_FLAGGED(i, MOB_ETHEREAL) || GET_LEVEL(ch) > LVL_IMMORT))
       if (++j == number)
 	return i;
 
@@ -1061,35 +1123,35 @@ char *money_desc(int amount)
     return NULL;
   }
   if (amount == 1)
-    strcpy(buf, "a gold coin");
+    strcpy(buf, "a coin");
   else if (amount <= 10)
-    strcpy(buf, "a tiny pile of gold coins");
-  else if (amount <= 20)
-    strcpy(buf, "a handful of gold coins");
-  else if (amount <= 75)
-    strcpy(buf, "a little pile of gold coins");
+    strcpy(buf, "a tiny pile of coins");
+  else if (amount <= 50)
+    strcpy(buf, "a handful of coins");
+  else if (amount <= 100)
+    strcpy(buf, "a little pile of coins");
   else if (amount <= 200)
-    strcpy(buf, "a small pile of gold coins");
+    strcpy(buf, "a small pile of coins");
   else if (amount <= 1000)
-    strcpy(buf, "a pile of gold coins");
+    strcpy(buf, "a pile of coins");
   else if (amount <= 5000)
-    strcpy(buf, "a big pile of gold coins");
+    strcpy(buf, "a big pile of coins");
   else if (amount <= 10000)
-    strcpy(buf, "a large heap of gold coins");
+    strcpy(buf, "a large heap of coins");
   else if (amount <= 20000)
-    strcpy(buf, "a huge mound of gold coins");
+    strcpy(buf, "a huge mound of coins");
   else if (amount <= 75000)
-    strcpy(buf, "an enormous mound of gold coins");
+    strcpy(buf, "an enormous mound of coins");
   else if (amount <= 150000)
-    strcpy(buf, "a small mountain of gold coins");
+    strcpy(buf, "a small mountain of coins");
   else if (amount <= 250000)
-    strcpy(buf, "a mountain of gold coins");
+    strcpy(buf, "a mountain of coins");
   else if (amount <= 500000)
-    strcpy(buf, "a huge mountain of gold coins");
+    strcpy(buf, "a huge mountain of coins");
   else if (amount <= 1000000)
-    strcpy(buf, "an enormous mountain of gold coins");
+    strcpy(buf, "an enormous mountain of coins");
   else
-    strcpy(buf, "an absolutely colossal mountain of gold coins");
+    strcpy(buf, "an absolutely colossal mountain of coins");
 
   return buf;
 }
@@ -1110,10 +1172,10 @@ struct obj_data *create_money(int amount)
 
   if (amount == 1) {
     obj->name = str_dup("coin gold");
-    obj->short_description = str_dup("a gold coin");
-    obj->description = str_dup("One miserable gold coin is lying here.");
+    obj->short_description = str_dup("a coin");
+    obj->description = str_dup("One miserable coin is lying here.");
     new_descr->keyword = str_dup("coin gold");
-    new_descr->description = str_dup("It's just one miserable little gold coin.");
+    new_descr->description = str_dup("It's just one miserable little coin.");
   } else {
     obj->name = str_dup("coins gold");
     obj->short_description = str_dup(money_desc(amount));
@@ -1146,6 +1208,7 @@ struct obj_data *create_money(int amount)
   GET_OBJ_VAL(obj, 0) = amount;
   GET_OBJ_COST(obj) = amount;
   obj->item_number = NOTHING;
+  GET_OBJ_RNUM(obj) = 0;
 
   return obj;
 }
