@@ -1,12 +1,16 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
 #include "interpreter.h"
 #include "comm.h"
 #include "spells.h"
 #include "utils.h"
 #include "db.h"
 #include "boards.h"
+#include "olc.h"
 
 /* external variables */
 extern struct obj_data *obj_proto;
@@ -613,7 +617,6 @@ iedit_parse(struct descriptor_data * d, char *arg)
 		/* eureka. insert now */
 		/*---------*/
 		new_obj_index[counter].virtual = d->edit_number;
-		new_obj_index[counter].number = 0;
 		new_obj_index[counter].func = NULL;
 		/*---------*/
 		new_obj_proto[counter] = *(d->edit_obj);
@@ -644,7 +647,6 @@ iedit_parse(struct descriptor_data * d, char *arg)
 	   * the other objects.. so insert at end */
 	  if (!found) {
 	    new_obj_index[top_of_objt + 1].virtual = d->edit_number;
-	    new_obj_index[top_of_objt + 1].number = 0;
 	    new_obj_index[top_of_objt + 1].func = NULL;
 
 	    clear_object(new_obj_proto + top_of_objt + 1);
@@ -691,6 +693,8 @@ iedit_parse(struct descriptor_data * d, char *arg)
       {
 	int             counter, counter2, realcounter;
 	FILE           *fp;
+	char newname[50], oldname[50];
+        char buf[33], buf2[33], buf3[33];
 	struct obj_data *obj;
 	struct extra_descr_data *ex_desc;
 
@@ -699,9 +703,11 @@ iedit_parse(struct descriptor_data * d, char *arg)
 	 * obj_proto is, plus it's short to type in :P */
 	CREATE(obj, struct obj_data, 1);
 
-	sprintf(buf, "%s/%d.obj", OBJ_PREFIX,
+	sprintf(newname, "%s/%d.obj.back", OBJ_PREFIX,
 		zone_table[d->edit_zone].number);
-	fp = fopen(buf, "w+");
+	sprintf(oldname, "%s/%d.obj", OBJ_PREFIX,
+		zone_table[d->edit_zone].number);
+	fp = fopen(newname, "w+");
 
 	/* start running through all objects in this zone */
 	for (counter = zone_table[d->edit_zone].number * 100;
@@ -719,8 +725,10 @@ iedit_parse(struct descriptor_data * d, char *arg)
 	      fprintf(fp, "%s~\n", obj->action_description);
 	    else
 	      fprintf(fp, "~\n");
-	    fprintf(fp, "%d %ld %ld %ld\n", GET_OBJ_TYPE(obj),
-		    GET_OBJ_EXTRA(obj), GET_OBJ_EXTRA2(obj), GET_OBJ_WEAR(obj));
+	    fprintf(fp, "%d %s %s %s\n", GET_OBJ_TYPE(obj),
+		    sprintbitascii(GET_OBJ_EXTRA(obj), buf),
+		    sprintbitascii(GET_OBJ_EXTRA2(obj), buf2),
+		    sprintbitascii(GET_OBJ_WEAR(obj), buf3) );
 	    fprintf(fp, "%d %d %d %d\n", GET_OBJ_VAL(obj, 0),
 		    GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2),
 		    GET_OBJ_VAL(obj, 3));
@@ -736,8 +744,10 @@ iedit_parse(struct descriptor_data * d, char *arg)
 	    }
             if (obj->obj_flags.bitvector || obj->obj_flags.bitvector2 || obj->obj_flags.bitvector3) {
 		fprintf(fp, "F\n");
-		fprintf(fp, "%ld %ld %ld\n", obj->obj_flags.bitvector,
-			obj->obj_flags.bitvector2, obj->obj_flags.bitvector3);
+		fprintf(fp, "%s %s %s\n",
+		    sprintbitascii(obj->obj_flags.bitvector, buf),
+		    sprintbitascii(obj->obj_flags.bitvector2, buf2),
+		    sprintbitascii(obj->obj_flags.bitvector3, buf3));
             }	
 	    /* do we have affects? */
 	    for (counter2 = 0; counter2 < MAX_OBJ_AFFECT; counter2++) {
@@ -752,6 +762,8 @@ iedit_parse(struct descriptor_data * d, char *arg)
 	/* write final line, close */
 	fprintf(fp, "$~\n");
 	fclose(fp);
+	remove(oldname);
+	rename(newname, oldname);
 	/* nuke temp object */
 	free_obj(obj);
       }
