@@ -16,6 +16,7 @@
 #include <ctype.h>
 
 #include "structs.h"
+#include "class.h"
 #include "comm.h"
 #include "interpreter.h"
 #include "db.h"
@@ -54,6 +55,8 @@ int isbanned(char *hostname);
 int Valid_Name(char *newname);
 void iedit_parse(struct descriptor_data * d, char *arg);
 void redit_parse(struct descriptor_data * d, char *arg);
+void zedit_parse(struct descriptor_data * d, char *arg);
+void ADD_REINCARN(struct char_data *ch);
 
 /* prototypes for all do_x functions. */
 ACMD(do_action);
@@ -66,6 +69,7 @@ ACMD(do_backstab);
 ACMD(do_ban);
 ACMD(do_bash);
 ACMD(do_cast);
+ACMD(do_channel);
 ACMD(do_color);
 ACMD(do_commands);
 ACMD(do_consider);
@@ -136,7 +140,7 @@ ACMD(do_mpforce);
 ACMD(do_move);
 ACMD(do_not_here);
 ACMD(do_offer);
-/*ACMD(do_olc);*/
+/* ACMD(do_olc); */
 ACMD(do_order);
 ACMD(do_page);
 ACMD(do_poofset);
@@ -155,8 +159,10 @@ ACMD(do_rescue);
 ACMD(do_rest);
 ACMD(do_restore);
 ACMD(do_return);
+ACMD(do_rstat);
 ACMD(do_save);
 ACMD(do_say);
+ACMD(do_scan);
 ACMD(do_score);
 ACMD(do_send);
 ACMD(do_set);
@@ -202,9 +208,13 @@ ACMD(do_wizlock);
 ACMD(do_wiznet);
 ACMD(do_wizutil);
 ACMD(do_write);
+ACMD(do_zstat);
+ACMD(do_zallow);
+ACMD(do_zdeny);
 ACMD(do_zreset);
 ACMD(do_iedit);
 ACMD(do_redit);
+ACMD(do_zedit);
 
 /* This is the Master Command List(tm).
  *
@@ -232,10 +242,11 @@ const struct command_info cmd_info[] = {
 
   /* now, the main list */
   { "at"       , POS_DEAD    , do_at       , LVL_IMMORT, 0 },
-  { "advance"  , POS_DEAD    , do_advance  , LVL_IMPL, 0 },
+  { "advance"  , POS_RESTING , do_advance  , 0, 0 },
+  { "afk"      , POS_DEAD    , do_gen_tog  , 0, SCMD_AFK },
   { "alias"    , POS_DEAD    , do_alias    , 0, 0 },
   { "accuse"   , POS_SITTING , do_action   , 0, 0 },
-  { "addquest" , POS_DEAD    , do_setquest , LVL_IMMORT, SCMD_ADD_QUEST_BIT },
+  { "addquest" , POS_DEAD    , do_setquest , LVL_GOD, SCMD_ADD_QUEST_BIT },
   { "affects"  , POS_FIGHTING, do_affects  , 0, 0 },
   { "agree"    , POS_SITTING , do_action   , 0, 0 },
   { "applaud"  , POS_RESTING , do_action   , 0, 0 },
@@ -255,6 +266,8 @@ const struct command_info cmd_info[] = {
   { "beg"      , POS_RESTING , do_action   , 0, 0 },
   { "bleed"    , POS_RESTING , do_action   , 0, 0 },
   { "blush"    , POS_RESTING , do_action   , 0, 0 },
+  { "bonk"     , POS_RESTING , do_action   , 0, 0 },
+  { "boink"    , POS_RESTING , do_action   , 0, 0 },
   { "bow"      , POS_STANDING, do_action   , 0, 0 },
   { "brb"      , POS_RESTING , do_action   , 0, 0 },
   { "brief"    , POS_DEAD    , do_gen_tog  , 0, SCMD_BRIEF },
@@ -267,8 +280,11 @@ const struct command_info cmd_info[] = {
   { "cast"     , POS_SITTING , do_cast     , 1, 0 },
   { "cackle"   , POS_RESTING , do_action   , 0, 0 },
   { "chat"     , POS_SLEEPING, do_gen_comm , 0, SCMD_CHAT },
+  { "channel"  , POS_DEAD    , do_channel  , 0, 0 },
   { "check"    , POS_STANDING, do_not_here , 1, 0 },
+  { "cheer"    , POS_SLEEPING, do_action   , 0, 0 },
   { "chuckle"  , POS_RESTING , do_action   , 0, 0 },
+  { "clan"     , POS_RESTING , do_gen_comm , 0, SCMD_CLAN },
   { "clap"     , POS_RESTING , do_action   , 0, 0 },
   { "clear"    , POS_DEAD    , do_gen_ps   , 0, SCMD_CLEAR },
   { "close"    , POS_SITTING , do_gen_door , 0, SCMD_CLOSE },
@@ -303,7 +319,7 @@ const struct command_info cmd_info[] = {
   { "drool"    , POS_RESTING , do_action   , 0, 0 },
 
   { "eat"      , POS_RESTING , do_eat      , 0, SCMD_EAT },
-  { "echo"     , POS_SLEEPING, do_echo     , LVL_IMMORT, SCMD_ECHO },
+  { "echo"     , POS_SLEEPING, do_echo     , LVL_GOD, SCMD_ECHO },
   { "emote"    , POS_RESTING , do_echo     , 1, SCMD_EMOTE },
   { ":"        , POS_RESTING, do_echo      , 1, SCMD_EMOTE },
   { "embrace"  , POS_STANDING, do_action   , 0, 0 },
@@ -318,6 +334,8 @@ const struct command_info cmd_info[] = {
   { "examine"  , POS_SITTING , do_examine  , 0, 0 },
 
   { "force"    , POS_SLEEPING, do_force    , LVL_GOD, 0 },
+  { "fade"     , POS_RESTING , do_action   , 0, 0 },
+  { "faint"    , POS_RESTING , do_action   , 0, 0 },
   { "fart"     , POS_RESTING , do_action   , 0, 0 },
   { "fill"     , POS_STANDING, do_pour     , 0, SCMD_FILL },
   { "fillet"   , POS_RESTING , do_fillet   , 0, 0 },
@@ -332,6 +350,7 @@ const struct command_info cmd_info[] = {
   { "french"   , POS_RESTING , do_action   , 0, 0 },
   { "frown"    , POS_RESTING , do_action   , 0, 0 },
   { "fume"     , POS_RESTING , do_action   , 0, 0 },
+  { "fwap"     , POS_STANDING, do_action   , 0, 0 },
 
   { "get"      , POS_RESTING , do_get      , 0, 0 },
   { "gasp"     , POS_RESTING , do_action   , 0, 0 },
@@ -347,6 +366,7 @@ const struct command_info cmd_info[] = {
   { "grats"    , POS_SLEEPING, do_gen_comm , 0, SCMD_GRATZ },
   { "greet"    , POS_RESTING , do_action   , 0, 0 },
   { "grin"     , POS_RESTING , do_action   , 0, 0 },
+  { "grimace"  , POS_RESTING , do_action   , 0, 0 },
   { "groan"    , POS_RESTING , do_action   , 0, 0 },
   { "grope"    , POS_RESTING , do_action   , 0, 0 },
   { "grovel"   , POS_RESTING , do_action   , 0, 0 },
@@ -371,13 +391,15 @@ const struct command_info cmd_info[] = {
 
   { "inventory", POS_DEAD    , do_inventory, 0, 0 },
   { "idea"     , POS_DEAD    , do_gen_write, 0, SCMD_IDEA },
+  { "idealist" , POS_DEAD    , do_gen_ps   , LVL_IMMORT, SCMD_IDEALIST },
   { "imotd"    , POS_DEAD    , do_gen_ps   , LVL_IMMORT, SCMD_IMOTD },
   { "immlist"  , POS_DEAD    , do_gen_ps   , 0, SCMD_IMMLIST },
   { "info"     , POS_SLEEPING, do_gen_ps   , 0, SCMD_INFO },
   { "insult"   , POS_RESTING , do_insult   , 0, 0 },
   { "invis"    , POS_DEAD    , do_invis    , LVL_IMMORT, 0 },
-  { "iedit"    , POS_DEAD    , do_iedit    , LVL_GOD, 0 },
+  { "iedit"    , POS_DEAD    , do_iedit    , LVL_GRGOD, 0 },
 
+  { "jeer"    , POS_SLEEPING, do_action   , 0, 0 },
   { "junk"     , POS_RESTING , do_drop     , 0, SCMD_JUNK },
 
   { "kill"     , POS_FIGHTING, do_kill     , 0, 0 },
@@ -389,6 +411,7 @@ const struct command_info cmd_info[] = {
   { "land"     , POS_FIGHTING, do_land     , 0, 0 },
   { "last"     , POS_DEAD    , do_last     , LVL_GOD, 0 },
   { "leave"    , POS_STANDING, do_leave    , 0, 0 },
+  { "leer"     , POS_SLEEPING, do_action   , 0, 0 },
   { "levels"   , POS_DEAD    , do_levels   , 0, 0 },
   { "list"     , POS_STANDING, do_not_here , 0, 0 },
   { "lick"     , POS_RESTING , do_action   , 0, 0 },
@@ -400,6 +423,7 @@ const struct command_info cmd_info[] = {
   { "motd"     , POS_DEAD    , do_gen_ps   , 0, SCMD_MOTD },
   { "mail"     , POS_STANDING, do_not_here , 1, 0 },
   { "massage"  , POS_RESTING , do_action   , 0, 0 },
+/*  { "medit"    , POS_DEAD    , do_medit    , LVL_IMPL, 0 }, meditmod*/ 
   { "mumble"   , POS_RESTING , do_action   , 0, 0 },
   { "mute"     , POS_DEAD    , do_wizutil  , LVL_GOD, SCMD_SQUELCH },
   { "murder"   , POS_FIGHTING, do_hit      , 0, SCMD_MURDER },
@@ -416,11 +440,15 @@ const struct command_info cmd_info[] = {
   { "mpat"     , POS_DEAD    , do_mpat	   , 0, 0 },
   { "mptransfer" , POS_DEAD  , do_mptransfer, 0, 0 },
   { "mpforce"  , POS_DEAD    , do_mpforce  , 0, 0 },
+  { "muhaha"   , POS_RESTING , do_action   , 0, 0 },
 
   { "news"     , POS_SLEEPING, do_gen_ps   , 0, SCMD_NEWS },
   { "nibble"   , POS_RESTING , do_action   , 0, 0 },
   { "nod"      , POS_RESTING , do_action   , 0, 0 },
   { "noauction", POS_DEAD    , do_gen_tog  , 0, SCMD_NOAUCTION },
+  { "noarena"  , POS_DEAD    , do_gen_tog  , 0, SCMD_NOTEAM },
+  { "nochat"   , POS_DEAD    , do_gen_tog  , 0, SCMD_NOCHAT },
+  { "noclan"   , POS_DEAD    , do_gen_tog  , 0, SCMD_NOCLAN },
   { "nogossip" , POS_DEAD    , do_gen_tog  , 0, SCMD_NOGOSSIP },
   { "nograts"  , POS_DEAD    , do_gen_tog  , 0, SCMD_NOGRATZ },
   { "nohassle" , POS_DEAD    , do_gen_tog  , LVL_IMMORT, SCMD_NOHASSLE },
@@ -431,11 +459,11 @@ const struct command_info cmd_info[] = {
   { "notell"   , POS_DEAD    , do_gen_tog  , 1, SCMD_NOTELL },
   { "notitle"  , POS_DEAD    , do_wizutil  , LVL_GOD, SCMD_NOTITLE },
   { "nowar"    , POS_DEAD    , do_gen_tog  , 0, SCMD_NOWAR },
-  { "nowiz"    , POS_DEAD    , do_gen_tog  , LVL_IMMORT, SCMD_NOWIZ },
+  { "nowiz"    , POS_DEAD    , do_gen_tog  , LVL_GRGOD, SCMD_NOWIZ },
   { "nudge"    , POS_RESTING , do_action   , 0, 0 },
   { "nuzzle"   , POS_RESTING , do_action   , 0, 0 },
 
-  /*  { "olc"      , POS_DEAD    , do_olc      , LVL_IMPL, 0 },*/
+/*  { "olc"      , POS_DEAD    , do_olc      , LVL_IMPL, 0 }, */
   { "order"    , POS_RESTING , do_order    , 1, 0 },
   { "offer"    , POS_STANDING, do_not_here , 1, 0 },
   { "open"     , POS_SITTING , do_gen_door , 0, SCMD_OPEN },
@@ -473,11 +501,13 @@ const struct command_info cmd_info[] = {
   { "reply"    , POS_SLEEPING, do_reply    , 0, 0 },
   { "rest"     , POS_RESTING , do_rest     , 0, 0 },
   { "read"     , POS_RESTING , do_look     , 0, SCMD_READ },
-  { "reload"   , POS_DEAD    , do_reboot   , LVL_IMPL, 0 },
   { "recite"   , POS_RESTING , do_use      , 0, SCMD_RECITE },
   { "receive"  , POS_STANDING, do_not_here , 1, 0 },
   { "register" , POS_STANDING, do_not_here , 0, 0 },
+  { "reincarn" , POS_STANDING, do_not_here , 0, 0 },
+  { "reload"   , POS_DEAD    , do_reboot   , LVL_IMMORT, 0 },
   { "remove"   , POS_RESTING , do_remove   , 0, 0 },
+  { "ren"      , POS_RESTING , do_action   , 0, 0 },
   { "rent"     , POS_STANDING, do_not_here , 1, 0 },
   { "report"   , POS_RESTING , do_report   , 0, 0 },
   { "reroll"   , POS_DEAD    , do_wizutil  , LVL_GRGOD, SCMD_REROLL },
@@ -485,20 +515,24 @@ const struct command_info cmd_info[] = {
   { "restore"  , POS_DEAD    , do_restore  , LVL_GOD, 0 },
   { "return"   , POS_DEAD    , do_return   , 0, 0 },
   { "roll"     , POS_RESTING , do_action   , 0, 0 },
+  { "rofl"     , POS_RESTING , do_action   , 0, 0 },
   { "roomflags", POS_DEAD    , do_gen_tog  , LVL_IMMORT, SCMD_ROOMFLAGS },
+  { "rstat"    , POS_DEAD    , do_rstat    , LVL_GOD, 0 },
   { "ruffle"   , POS_STANDING, do_action   , 0, 0 },
   { "redit"    , POS_DEAD    , do_redit    , LVL_GOD, 0 },
+/*  { "rzedit"    , POS_DEAD   , do_rzedit   , LVL_IMPL, 0 }, */
 
 /*  { "sacrifice", POS_RESTING , do_ */
   { "say"      , POS_RESTING , do_say      , 0, 0 },
   { "'"        , POS_RESTING , do_say      , 0, 0 },
   { "save"     , POS_SLEEPING, do_save     , 0, 0 },
   { "score"    , POS_DEAD    , do_score    , 0, 0 },
+  { "scan"     , POS_RESTING , do_scan     , 0, 0 },
   { "scream"   , POS_RESTING , do_action   , 0, 0 },
   { "sell"     , POS_STANDING, do_not_here , 0, 0 },
   { "send"     , POS_SLEEPING, do_send     , LVL_GOD, 0 },
   { "set"      , POS_DEAD    , do_set      , LVL_GOD, 0 },
-  { "setquest" , POS_DEAD    , do_setquest , LVL_IMMORT, SCMD_SET_QUEST_BIT },
+  { "setquest" , POS_DEAD    , do_setquest , LVL_GOD, SCMD_SET_QUEST_BIT },
   { "shout"    , POS_RESTING , do_gen_comm , 0, SCMD_SHOUT },
   { "shake"    , POS_RESTING , do_action   , 0, 0 },
   { "shishkabob",POS_RESTING , do_action   , 0, 0 },
@@ -506,8 +540,9 @@ const struct command_info cmd_info[] = {
   { "show"     , POS_DEAD    , do_show     , 0, 0 },
   { "showquest", POS_DEAD    , do_showquest, LVL_IMMORT, 0 },
   { "shrug"    , POS_RESTING , do_action   , 0, 0 },
+  { "shudder"  , POS_RESTING , do_action   , 0, 0 },
   { "shutdow"  , POS_DEAD    , do_shutdown , LVL_IMPL, 0 },
-  { "shutdown" , POS_DEAD    , do_shutdown , LVL_IMPL, SCMD_SHUTDOWN },
+  { "shutdown" , POS_DEAD    , do_shutdown , 0, SCMD_SHUTDOWN },
   { "sigh"     , POS_RESTING , do_action   , 0, 0 },
   { "sing"     , POS_RESTING , do_action   , 0, 0 },
   { "sip"      , POS_RESTING , do_drink    , 0, SCMD_SIP },
@@ -532,6 +567,7 @@ const struct command_info cmd_info[] = {
   { "socials"  , POS_DEAD    , do_commands , 0, SCMD_SOCIALS },
   { "split"    , POS_SITTING , do_split    , 1, 0 },
   { "spank"    , POS_RESTING , do_action   , 0, 0 },
+  { "spew"     , POS_RESTING , do_action   , 0, 0 },
   { "spit"     , POS_STANDING, do_action   , 0, 0 },
   { "squeeze"  , POS_RESTING , do_action   , 0, 0 },
   { "stand"    , POS_RESTING , do_stand    , 0, 0 },
@@ -539,6 +575,7 @@ const struct command_info cmd_info[] = {
   { "stat"     , POS_DEAD    , do_stat     , LVL_IMMORT, 0 },
   { "steal"    , POS_STANDING, do_steal    , 1, 0 },
   { "steam"    , POS_RESTING , do_action   , 0, 0 },
+  { "stimpy"   , POS_RESTING , do_action   , 0, 0 },
   { "stroke"   , POS_RESTING , do_action   , 0, 0 },
   { "strut"    , POS_STANDING, do_action   , 0, 0 },
   { "sulk"     , POS_RESTING , do_action   , 0, 0 },
@@ -549,6 +586,7 @@ const struct command_info cmd_info[] = {
   { "tackle"   , POS_RESTING , do_action   , 0, 0 },
   { "take"     , POS_RESTING , do_get      , 0, 0 },
   { "tango"    , POS_STANDING, do_action   , 0, 0 },
+  { "tap"      , POS_STANDING, do_action   , 0, 0 },
   { "taunt"    , POS_RESTING , do_action   , 0, 0 },
   { "taste"    , POS_RESTING , do_eat      , 0, SCMD_TASTE },
   { "teleport" , POS_DEAD    , do_teleport , LVL_GOD, 0 },
@@ -566,6 +604,7 @@ const struct command_info cmd_info[] = {
   { "transfer" , POS_SLEEPING, do_trans    , LVL_GOD, 0 },
   { "twiddle"  , POS_RESTING , do_action   , 0, 0 },
   { "typo"     , POS_DEAD    , do_gen_write, 0, SCMD_TYPO },
+  { "typolist" , POS_DEAD    , do_gen_ps   , LVL_IMMORT, SCMD_TYPOLIST },
 
   { "unlock"   , POS_SITTING , do_gen_door , 0, SCMD_UNLOCK },
   { "ungroup"  , POS_DEAD    , do_ungroup  , 0, 0 },
@@ -586,6 +625,8 @@ const struct command_info cmd_info[] = {
   { "wave"     , POS_RESTING , do_action   , 0, 0 },
   { "wear"     , POS_RESTING , do_wear     , 0, 0 },
   { "weather"  , POS_RESTING , do_weather  , 0, 0 },
+  { "wee"      , POS_RESTING , do_action   , 0, 0 },
+  { "weep"     , POS_RESTING , do_action   , 0, 0 },
   { "who"      , POS_DEAD    , do_who      , 0, 0 },
   { "whoami"   , POS_DEAD    , do_gen_ps   , 0, SCMD_WHOAMI },
   { "where"    , POS_RESTING , do_where    , 1, 0 },
@@ -610,6 +651,10 @@ const struct command_info cmd_info[] = {
   { "yodel"    , POS_RESTING , do_action   , 0, 0 },
 
   { "zreset"   , POS_DEAD    , do_zreset   , LVL_GRGOD, 0 },
+  { "zedit"    , POS_DEAD    , do_zedit    , LVL_GOD, 0 },
+  { "zstat"    , POS_DEAD    , do_zstat    , LVL_GOD, 0 },  
+  { "zallow"   , POS_DEAD    , do_zallow   , LVL_IMPL, 0 }, 
+  { "zdeny"    , POS_DEAD    , do_zdeny    , LVL_IMPL, 0 }, 
 
   { "\n", 0, 0, 0, 0 } };	/* this must be last */
 
@@ -1171,8 +1216,16 @@ void nanny(struct descriptor_data * d, char *arg)
   extern sh_int r_mortal_start_room;
   extern sh_int r_immort_start_room;
   extern sh_int r_frozen_start_room;
-  extern const char *class_menu;
-  extern const char *race_menu;
+  extern const char *class_menu1;
+  extern const char *class_menu2;
+  extern const char *class_menu3;
+  extern const char *class_menu4;
+  extern const char *race_menu1;
+  extern const char *race_menu2;
+  extern const char *race_menu3;
+  extern const char *race_menu4;
+  extern const char *race_menu5;
+  extern const char *race_menu6;
   extern int max_bad_pws;
   sh_int load_room;
 
@@ -1191,6 +1244,10 @@ void nanny(struct descriptor_data * d, char *arg)
     /* return control to iedit_parse */
     iedit_parse(d, arg);
     break;
+  case CON_ZEDIT:
+    /* goto zedit_parse */
+    zedit_parse(d, arg);
+    break;
   case CON_GET_NAME:		/* wait for input of name */
     if (d->character == NULL) {
       CREATE(d->character, struct char_data, 1);
@@ -1473,17 +1530,46 @@ void nanny(struct descriptor_data * d, char *arg)
       return;
       break;
     }
-    SEND_TO_Q(race_menu, d);
+    SEND_TO_Q(race_menu1, d);
+    if (GET_REINCARN(d->character) >= 2)
+	SEND_TO_Q(race_menu2, d);
+    if (GET_REINCARN(d->character) >= 3)
+	SEND_TO_Q(race_menu3, d);
+    if (GET_REINCARN(d->character) >= 5)
+	SEND_TO_Q(race_menu4, d);
+    if (GET_REINCARN(d->character) >= 7)
+	SEND_TO_Q(race_menu5, d);
+    SEND_TO_Q(race_menu6, d);
     SEND_TO_Q("\r\nRace: ", d);
     STATE(d) = CON_QRACE;
     break;
 
   case CON_QRACE:
     load_result = parse_race(*arg);
-    if (load_result == CLASS_UNDEFINED || load_result == RACE_HEMNOV || load_result == RACE_LLYRA) {
-	SEND_TO_Q("\r\nThat is not an available race.\r\nRace: ", d);
+/* BY STROM: Extra race parsing... */
+    if (load_result == RACE_LLYRA && GET_REINCARN(d->character) < 3)
+	load_result = CLASS_UNDEFINED;
+    if (load_result == RACE_HEMNOV && GET_REINCARN(d->character) < 3)
+	load_result = CLASS_UNDEFINED;
+    if (load_result == RACE_WEREFORM && GET_REINCARN(d->character) < 5)
+	load_result = CLASS_UNDEFINED;
+    if (load_result == RACE_DRAGON && GET_REINCARN(d->character) < 7)
+	load_result = CLASS_UNDEFINED;
+    if (load_result == CLASS_UNDEFINED) {
+	SEND_TO_Q("\r\nThat race is not available to you.\r\nRace: ", d);
+	SEND_TO_Q(race_menu1, d);
+        if (GET_REINCARN(d->character) >= 2)
+		SEND_TO_Q(race_menu2, d);
+        if (GET_REINCARN(d->character) >= 3)
+		SEND_TO_Q(race_menu3, d);
+        if (GET_REINCARN(d->character) >= 5)
+		SEND_TO_Q(race_menu4, d);
+        if (GET_REINCARN(d->character) >= 7)
+		SEND_TO_Q(race_menu5, d);
+    	SEND_TO_Q(race_menu6, d);
+	SEND_TO_Q("\r\nRace: ", d);
 	return;
-        } 
+    } 
     if (load_result == RACE_DARKGNOME) {
 	load_result = RACE_GNOME;
 	SET_BIT(PRF2_FLAGS(d->character), PRF2_WAR_DRUHARI);
@@ -1508,16 +1594,28 @@ void nanny(struct descriptor_data * d, char *arg)
 	    SEND_TO_Q(buf, d);
       return;
       break; }
-    SEND_TO_Q(class_menu, d);
+    SEND_TO_Q(class_menu1, d);
+    if (GET_REINCARN(d->character) >= 3)
+	    SEND_TO_Q(class_menu2, d);   
+    if (GET_REINCARN(d->character) >= 5)
+	    SEND_TO_Q(class_menu3, d);   
+    SEND_TO_Q(class_menu4, d);
     SEND_TO_Q("\r\nClass: ", d);
     STATE(d) = CON_QCLASS;
     break;
 
   case CON_QCLASS:
     load_result = parse_class(*arg, GET_RACE(d->character));
-    if (load_result == CLASS_UNDEFINED || load_result == CLASS_AVATAR) {
+    if (load_result == CLASS_AVATAR && GET_REINCARN(d->character) < 8)
+	load_result = CLASS_UNDEFINED;
+    if (load_result == CLASS_UNDEFINED) {
       SEND_TO_Q("\r\nSorry: that class is not available to you.\r\n", d);
-      SEND_TO_Q(class_menu, d);
+      SEND_TO_Q(class_menu1, d);
+      if (GET_REINCARN(d->character) >= 3)
+	    SEND_TO_Q(class_menu2, d);    
+      if (GET_REINCARN(d->character) >= 5)
+	    SEND_TO_Q(class_menu2, d);    
+      SEND_TO_Q(class_menu4, d);
       SEND_TO_Q("\r\nClass: ", d);
       return;
     } else
@@ -1617,7 +1715,6 @@ BY STROM */
       read_aliases(d->character);
       read_quests(d->character);
       break;
-
     case '2':
       SEND_TO_Q("Enter the text you'd like others to see when they look at you.\r\n", d);
       SEND_TO_Q("Terminate with a '@' on a new line.\r\n", d);
@@ -1648,13 +1745,16 @@ BY STROM */
       echo_off(d);
       STATE(d) = CON_DELCNF1;
       break;
-
+    case '6':
+      SEND_TO_Q("\r\nEnter your password for verification: ", d);
+      echo_off(d);
+      STATE(d) = CON_REINCNF1;
+      break;
     default:
       SEND_TO_Q("\r\nThat's not a menu choice!\r\n", d);
       SEND_TO_Q(MENU, d);
       break;
     }
-
     break;
 
   case CON_CHPWD_GETOLD:
@@ -1670,6 +1770,28 @@ BY STROM */
       return;
     }
     break;
+  case CON_REINCNF1:
+    echo_on(d);
+    if (strncmp(CRYPT(arg, GET_PASSWD(d->character)), GET_PASSWD(d->character), MAX_PWD_LENGTH)) {
+      SEND_TO_Q("\r\nIncorrect password.\r\n", d);
+      SEND_TO_Q(MENU, d);
+      STATE(d) = CON_MENU;
+    } else {
+      if (GET_LEVEL(d->character) < 29) {
+        SEND_TO_Q("\r\nYOU ARE ABOUT TO REINCARNATE THIS CHARACTER.\r\n"
+		"SINCE YOUR LEVEL IS LESS THEN 29, YOU WILL NOT ADVANCE ON THE GOLDEN PATH.\r\n"
+		"ARE YOU ABSOLUTELY SURE YOU WANT TO GO BACK TO THE BEGINNING OF THIS CIRCLE?\r\n\r\n"
+		"Please type \"yes\" to confirm: ", d);
+      } else {
+	SEND_TO_Q("\r\nYOU ARE ABOUT TO BE REINCARNATED.  YOU WILL BEGIN ONCE MORE AT\r\n"
+		  "LEVEL ONE.  HOWEVER, YOU WILL ADVANCE ONE STEP UP THE GOLDEN PATH,\r\n"
+		  "ONWARD TOWARD IMMORTALITY.\r\n"
+		  "ARE YOU ABSOLUTELY PREPARED TO STEP INTO THE NEXT CIRCLE?\r\n"
+		  "Please type \"yes\" to confirm: ", d); 
+      }
+      STATE(d) = CON_REINCNF2;
+    }
+    break;
 
   case CON_DELCNF1:
     echo_on(d);
@@ -1685,6 +1807,38 @@ BY STROM */
     }
     break;
 
+  case CON_REINCNF2:
+    if (!strcmp(arg, "yes") || !strcmp(arg, "YES")) {
+      if (PLR_FLAGGED(d->character, PLR_FROZEN)) {
+	SEND_TO_Q("You try to force-reincarnate yourself, but the ice stops you.\r\n", d);
+	SEND_TO_Q("Character not deleted.\r\n\r\n", d);
+	STATE(d) = CON_CLOSE;
+	return;
+      }
+      if (GET_LEVEL(d->character) >= LVL_IMMORT) {
+	sprintf(buf, "\r\nSorry, %s, greater gods and up can not restart their characters.", GET_NAME(d->character));
+	SEND_TO_Q(buf, d);
+	return;
+      }
+
+      if (GET_LEVEL(d->character) >= 29 && GET_REINCARN(d->character) < 8) {
+	ADD_REINCARN(d->character);
+	GET_LEVEL(d->character) = 0;
+	/* Some dramatic rumbling is called for */
+      }
+
+      SET_BIT(PLR_FLAGS(d->character), PLR_JUSTREIN);
+      sprintf(buf, "\r\n%s, you are being rerolled from scratch.  Good Luck.\r\nWhat is your gender? ", GET_NAME(d->character));
+      SEND_TO_Q(buf, d);
+      STATE(d) = CON_QSEX;
+      return;
+    } else {
+      SEND_TO_Q("\r\nCharacter not reincarnated.\r\n", d);
+      SEND_TO_Q(MENU, d);
+      STATE(d) = CON_MENU;
+    }
+    break;
+
   case CON_DELCNF2:
     if (!strcmp(arg, "yes") || !strcmp(arg, "YES")) {
       if (PLR_FLAGGED(d->character, PLR_FROZEN)) {
