@@ -12,6 +12,9 @@
 #include <stdio.h>
 #include <string.h>
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "utils.h"
 #include "comm.h"
 #include "spells.h"
@@ -42,7 +45,7 @@ void act(char *str, int i, struct char_data * c, struct obj_data * o,
 	      void *vict_obj, int j);
 
 void damage(struct char_data * ch, struct char_data * victim,
-	         int damage, int weapontype);
+	         int damage, int weapontype, struct obj_data *obj);
 
 void weight_change_object(struct obj_data * obj, int weight);
 void add_follower(struct char_data * ch, struct char_data * leader);
@@ -51,7 +54,7 @@ extern struct spell_info_type spell_info[];
 
 bool CAN_MURDER(struct char_data * ch, struct char_data * victim);
 
-struct char_data *read_mobile(int, int);
+struct char_data *read_mobile(int, int, int);
 
 
 /*
@@ -477,9 +480,11 @@ int mag_materials(struct char_data * ch, int item0, int item1, int item2,
 		      int extract, int verbose)
 {
   struct obj_data *tobj;
-  struct obj_data *obj0, *obj1, *obj2;
+  struct obj_data *obj0 = NULL;
+  struct obj_data *obj1 = NULL;
+  struct obj_data *obj2 = NULL;
 
-  for (tobj = ch->carrying; tobj; tobj = tobj->next) {
+  for (tobj = ch->carrying; tobj; tobj = tobj->next_content) {
     if ((item0 > 0) && (GET_OBJ_VNUM(tobj) == item0)) {
       obj0 = tobj;
       item0 = -1;
@@ -528,9 +533,6 @@ int mag_materials(struct char_data * ch, int item0, int item1, int item2,
   return (TRUE);
 }
 
-
-
-
 /*
  * Every spell that does damage comes through here.  This calculates the
  * amount of damage, adds in any modifiers, determines what the saves are,
@@ -694,7 +696,7 @@ void mag_damage(int level, struct char_data * ch, struct char_data * victim,
   if (!IS_NPC(ch) && !IS_NPC(victim) && !CAN_MURDER(ch, victim))
       act("The Mark of Neutrality protects $N.", FALSE, ch, 0, victim, TO_CHAR);
   else
-      damage(ch, victim, dam, spellnum);
+      damage(ch, victim, dam, spellnum, NULL);
 }
 
 /*
@@ -1137,6 +1139,12 @@ void mag_affects(int level, struct char_data * ch, struct char_data * victim,
     to_vict = "You feel much more restful.";
     break;
 
+  case SPELL_HASTE:
+    af.duration = 2;
+    af.bitvector2 = AFF2_HASTE;
+    to_vict = "You feel yourself moving much faster.";
+    break;
+
   
   /* end of spells */
   }
@@ -1530,13 +1538,14 @@ void mag_creations(int level, struct char_data * ch, int spellnum)
   case SPELL_CREATE_FOOD:
     z = 10;
     break;
+  case SPELL_CREATE_WATER:
   default:
     send_to_char("Spell unimplemented, it would seem.\r\n", ch);
     return;
     break;
   }
 
-  if (!(tobj = read_object(z, VIRTUAL))) {
+  if (!(tobj = read_object(z, VIRTUAL, 0))) {
     send_to_char("I seem to have goofed.\r\n", ch);
     sprintf(buf, "SYSERR: spell_creations, spell %d, obj %d: obj not found",
 	    spellnum, z);
@@ -1574,18 +1583,18 @@ static char *mag_summon_msgs[] = {
   "$n animates a corpse!\r\n"
 };
 
-#define MOB_MONSUM_I		4020	/* lesser level */
-#define MOB_MONSUM_II		4020	/* regular */
-#define MOB_MONSUM_III		4020	/* greater */
-#define MOB_MONSUM_IV		4020	/* dragon  */
-#define MOB_MONSUM_V		4020	/* ancient */
-#define MOB_GATE_I		4020
-#define MOB_GATE_II		4020
-#define MOB_GATE_III		4020
-#define MOB_ELEMENTAL_BASE	4020
-#define MOB_CLONE		4020
+#define MOB_MONSUM_I		116	/* lesser level */
+#define MOB_MONSUM_II		10	/* regular */
+#define MOB_MONSUM_III		10	/* greater */
+#define MOB_MONSUM_IV		273	/* dragon  */
+#define MOB_MONSUM_V		275	/* ancient */
+#define MOB_GATE_I		10
+#define MOB_GATE_II		10
+#define MOB_GATE_III		10
+#define MOB_ELEMENTAL_BASE	10
+#define MOB_CLONE		10
 #define MOB_ZOMBIE		1176
-#define MOB_AERIALSERVANT	4020
+#define MOB_AERIALSERVANT	19
 
 
 static char *mag_summon_fail_msgs[] = {
@@ -1686,7 +1695,7 @@ void mag_summons(int level, struct char_data * ch, struct obj_data * obj,
     send_to_char(mag_summon_fail_msgs[fmsg], ch);
     return;
   }
-  if (!(mob = read_mobile(real_mobile(mob_num), REAL))) {
+  if (!(mob = read_mobile(real_mobile(mob_num), REAL, world[ch->in_room].zone))) {
 	send_to_char("I seem to have goofed.\r\n", ch);
 	sprintf(buf, "SYSERR: spell_summons, spell %d, mob %d: mob not found",
 		spellnum, mob_num);
