@@ -36,7 +36,11 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
+#include "rooms.h"
 #include "utils.h"
 #include "interpreter.h"
 #include "handler.h"
