@@ -16,6 +16,9 @@
 #include <errno.h>
 
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "comm.h"
 #include "handler.h"
 #include "db.h"
@@ -41,13 +44,13 @@ ACMD(do_tell);
 SPECIAL(receptionist);
 SPECIAL(cryogenicist);
 
-struct obj_data *Obj_from_store(struct obj_file_elem object)
+struct obj_data *Obj_from_store(struct obj_file_elem object, int zone)
 {
   struct obj_data *obj;
   int j;
 
   if (real_object(object.item_number) > -1) {
-    obj = read_object(object.item_number, VIRTUAL);
+    obj = read_object(object.item_number, VIRTUAL, zone);
     GET_OBJ_VAL(obj, 0) = object.value[0];
     GET_OBJ_VAL(obj, 1) = object.value[1];
     GET_OBJ_VAL(obj, 2) = object.value[2];
@@ -266,7 +269,7 @@ void Crash_listrent(struct char_data * ch, char *name)
     }
     if (!feof(fl))
       if (real_object(object.item_number) > -1) {
-	obj = read_object(object.item_number, VIRTUAL);
+	obj = read_object(object.item_number, VIRTUAL, 0);
 	sprintf(buf, "%s [%5d] (%5dau) %-20s\r\n", buf,
 		object.item_number, GET_OBJ_RENT(obj),
 		obj->short_description);
@@ -334,6 +337,12 @@ int Crash_load(struct char_data * ch)
       Crash_crashsave(ch);
       return 2;
     } else {
+      if (PLR_FLAGGED(ch, PLR_JUSTREIN)) {
+	REMOVE_BIT(PLR_FLAGS(ch), PLR_JUSTREIN);
+	fclose(fl);
+	save_char(ch, NOWHERE);
+	return 1;
+      }
       GET_BANK_GOLD(ch) -= MAX(cost - GET_GOLD(ch), 0);
       GET_GOLD(ch) = MAX(GET_GOLD(ch) - cost, 0);
       save_char(ch, NOWHERE);
@@ -371,7 +380,7 @@ int Crash_load(struct char_data * ch)
       return 1;
     }
     if (!feof(fl))
-      obj_to_char(Obj_from_store(object), ch);
+      obj_to_char(Obj_from_store(object, world[ch->in_room].zone), ch);
   }
 
   /* turn this into a crash file by re-writing the control block */
@@ -676,7 +685,7 @@ void Crash_rent_deadline(struct char_data * ch, struct char_data * recep,
 
   rent_deadline = ((GET_GOLD(ch) + GET_BANK_GOLD(ch)) / cost);
   sprintf(buf,
-      "$n tells you, 'You can rent for %ld day%s with the gold you have\r\n"
+      "$n tells you, 'You can rent for %ld day%s with the coins you have\r\n"
 	  "on hand and in the bank.'\r\n",
 	  rent_deadline, (rent_deadline > 1) ? "s" : "");
   act(buf, FALSE, recep, 0, ch, TO_VICT);
@@ -813,9 +822,9 @@ int gen_receptionist(struct char_data * ch, struct char_data * recep,
     if (!(cost = Crash_offer_rent(ch, recep, FALSE, mode)))
       return TRUE;
     if (mode == RENT_FACTOR)
-      sprintf(buf, "$n tells you, 'Rent will cost you %d gold coins per day.'", cost);
+      sprintf(buf, "$n tells you, 'Rent will cost you %d coins per day.'", cost);
     else if (mode == CRYO_FACTOR)
-      sprintf(buf, "$n tells you, 'It will cost you %d gold coins to be frozen.'", cost);
+      sprintf(buf, "$n tells you, 'It will cost you %d coins to be frozen.'", cost);
     act(buf, FALSE, recep, 0, ch, TO_VICT);
     if (cost > GET_GOLD(ch)) {
       act("$n tells you, '...which I see you can't afford.'",
