@@ -1,5 +1,8 @@
 #include <stdio.h>
+#include <ctype.h>
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
 #include "interpreter.h"
 #include "comm.h"
 #include "spells.h"
@@ -7,18 +10,18 @@
 #include "db.h"
 #include <stdlib.h>
 #include "boards.h"
+#include "olc.h"
+#include "constants.h"
 
 /* change this depending on the number of flags you have defined */
 #define NUM_ROOM_SECTORS 12
-#define NUM_ROOM_FLAGS   17
+#define NUM_ROOM_FLAGS   20
+#define TELEPORT_NUM	  7
 
 extern int      top_of_world;
 extern struct room_data *world;
 extern struct obj_data *obj_proto;
 extern struct char_data *mob_proto;
-extern char    *room_bits[];
-extern char    *sector_types[];
-extern char    *exit_bits[];
 extern struct zone_data *zone_table;
 extern sh_int r_mortal_start_room;
 extern sh_int r_immort_start_room;
@@ -31,6 +34,7 @@ extern sh_int frozen_start_room;
 void redit_disp_extradesc_menu(struct descriptor_data * d);
 void redit_disp_exit_menu(struct descriptor_data * d);
 void redit_disp_exit_flag_menu(struct descriptor_data * d);
+void redit_disp_teleport_menu(struct descriptor_data * d);
 void redit_disp_flag_menu(struct descriptor_data * d);
 void redit_disp_sector_menu(struct descriptor_data * d);
 void redit_disp_menu(struct descriptor_data * d);
@@ -138,6 +142,26 @@ void redit_disp_flag_menu(struct descriptor_data * d)
   d->edit_mode = REDIT_FLAGS;
 }
 
+/* For teleport flags */
+void redit_disp_teleport_menu(struct descriptor_data * d)
+{
+  int             counter;
+
+  send_to_char("[H[J", d->character);
+  for (counter = 0; counter < TELEPORT_NUM; counter += 2) {
+    sprintf(buf, "%2d) %20.20s %2d) %20.20s\r\n",
+	    counter, teleport_bits[counter],
+	    counter + 1, counter + 1 < NUM_ROOM_FLAGS ?
+	    teleport_bits[counter + 1] : "");
+    send_to_char(buf, d->character);
+  }
+  sprintbit(d->edit_room->tele->mask, teleport_bits, buf1);
+  sprintf(buf, "Teleport flags: %s\r\n", buf1);
+  send_to_char(buf, d->character);
+  send_to_char("Enter teleport flags, 'q' to quit:", d->character);
+  d->edit_mode = REDIT_TELEPORT_MENU;
+}
+
 /* for sector type */
 void redit_disp_sector_menu(struct descriptor_data * d)
 {
@@ -214,6 +238,41 @@ void redit_disp_menu(struct descriptor_data * d)
   sprintf(buf, "a) Exit down to:  %s\r\n", buf2);
   send_to_char(buf, d->character);
   send_to_char("b) Extra descriptions\r\n", d->character);
+
+  if (IS_SET(d->edit_room->room_flags, ROOM_BROADCAST)) {
+    send_to_char("  Broadcasting Control \r\n", d->character);
+    sprintf(buf, "p) Broadcasting Channel: %s\r\n", channel_bits[(d->edit_room->broad->channel) - 1]);
+    send_to_char(buf, d->character);
+    sprintf(buf, "r) Broadcasting Target Room1: %s \r\n",
+		world[real_room(d->edit_room->broad->targ1)].name);
+    send_to_char(buf, d->character);
+    sprintf(buf, "s) Broadcasting Target Room2: %s \r\n",
+		world[real_room(d->edit_room->broad->targ2)].name);
+    send_to_char(buf, d->character);    
+  }
+  if (d->edit_room->tele != NULL) {
+    send_to_char("  Teleportation Control \r\n"
+	"   Set target to 0 for non-teleporting rooms\r\n", d->character);
+    sprintf(buf, "t) Teleport Target   :  %s\r\n", 
+		world[real_room(d->edit_room->tele->targ)].name);
+    send_to_char(buf, d->character);
+    sprintf(buf, "u) Teleport Frequency:  ~%d*10 seconds\r\n",
+		d->edit_room->tele->time);
+    send_to_char(buf, d->character);
+    sprintbit(d->edit_room->tele->mask, teleport_bits, buf2);
+    sprintf(buf, "v) Teleport Flags    : %s\r\n", buf2);
+    send_to_char(buf, d->character);
+    if (IS_SET(d->edit_room->tele->mask, TELE_OBJ) ||
+		IS_SET(d->edit_room->tele->mask, TELE_NOOBJ)) {
+	if (d->edit_room->tele->obj == 0)  
+	  sprintf(buf, "w) Teleport Obj-VNUM : (NULL) [CHANGE IT]\r\n");
+	else
+	  sprintf(buf, "w) Teleport Obj-VNUM : %s\r\n",
+		obj_proto[real_object(d->edit_room->tele->obj)].short_description);
+    } else
+	  sprintf(buf, "w) Teleport Obj-VNUM : (NULL) [not used]\r\n");
+    send_to_char(buf, d->character);
+  }
   send_to_char("q) Quit\r\n", d->character);
   send_to_char("Enter your choice:\r\n", d->character);
 }
@@ -227,6 +286,7 @@ void redit_disp_menu(struct descriptor_data * d)
 void redit_parse(struct descriptor_data * d, char *arg)
 {
   extern struct room_data *world;
+  char buf[33];
   int             number;
   int             room_num;
   switch (d->edit_mode) {
@@ -261,28 +321,29 @@ void redit_parse(struct descriptor_data * d, char *arg)
 	/* this code writes the entire zone containing
 	   the edited room to disk */
 	int             counter, counter2, realcounter;
+	char	newname[50], oldname[50];
 	FILE           *fp;
-	struct room_data *room;
+	struct room_data *room = NULL;
 	struct extra_descr_data *ex_desc;
 
-	/* not really necessary, but I"m lazy :) */
-	CREATE(room, struct room_data, 1);
-
-	sprintf(buf, "%s/%d.wld", WLD_PREFIX,
+	sprintf(newname, "%s/%d.wld.back", WLD_PREFIX,
 		zone_table[d->edit_zone].number);
-	fp = fopen(buf, "w+");
+	sprintf(oldname, "%s/%d.wld", WLD_PREFIX,
+		zone_table[d->edit_zone].number);
+	fp = fopen(newname, "w+");
 	for (counter = zone_table[d->edit_zone].number * 100;
 	     counter <= zone_table[d->edit_zone].top;
 	     counter++) {
 	  realcounter = real_room(counter);
 	  if (realcounter >= 0) {
-	    *room = world[realcounter];
+	    room = &world[realcounter];
 	    fprintf(fp, "#%d\n", counter);
 	    fprintf(fp, "%s~\n", room->name);
 	    fprintf(fp, "%s~\n", room->description);
-	    fprintf(fp, "%d %d %d\n",
+	    fprintf(fp, "%d %s %d\n",
 		    zone_table[room->zone].number,
-		    room->room_flags, room->sector_type);
+		    sprintbitascii(room->room_flags, buf),
+		    room->sector_type);
 	    for (counter2 = 0; counter2 < NUM_OF_DIRS; counter2++) {
 	      if (room->dir_option[counter2]) {
 		int             temp_door_flag;
@@ -317,14 +378,29 @@ void redit_parse(struct descriptor_data * d, char *arg)
 		fprintf(fp, "%s~\n", ex_desc->description);
 	      }
 	    }
+	    if (room->tele != NULL) {
+		fprintf(fp, "T\n");
+		fprintf(fp, "%d %ld %d %d\n",
+			room->tele->targ,
+			room->tele->mask,
+			room->tele->time,
+			room->tele->obj);
+	    }
+            if (IS_SET(room->room_flags, ROOM_BROADCAST) && room->broad != NULL) {
+		fprintf(fp, "B\n");
+		fprintf(fp, "%d %d %d\n",
+			room->broad->channel,
+			room->broad->targ1,
+			room->broad->targ2);
+	    }
 	    fprintf(fp, "S\n");
 	  }
 	}
-	/* free temporary */
-	free(room);
 	/* write final line and close */
 	fprintf(fp, "$~\n");
 	fclose(fp);
+        remove(oldname);
+	rename(newname, oldname);
 	send_to_char("Saved.\r\n", d->character);
 	/* do NOT free strings! just the room structure */
 	free(d->edit_room);
@@ -357,7 +433,6 @@ void redit_parse(struct descriptor_data * d, char *arg)
     case 'Y':
       {
 	int             counter;
-	struct extra_descr_data *this, *next_one;
 	room_num = real_room(d->edit_number);
 	if (room_num > 0) {
 	  /* copy people/object pointers over to the temp room
@@ -365,30 +440,17 @@ void redit_parse(struct descriptor_data * d, char *arg)
 	  d->edit_room->contents = world[room_num].contents;
 	  d->edit_room->people = world[room_num].people;
 	  /* find the original room, and free all strings/pointers */
-	  if (world[room_num].name)
-	    free(world[room_num].name);
-	  if (world[room_num].description)
-	    free(world[room_num].description);
-	  for (counter = 0; counter < NUM_OF_DIRS; counter++) {
-	    if (world[room_num].dir_option[counter]) {
-	      if (world[room_num].dir_option[counter]->general_description)
-		free(world[room_num].dir_option[counter]->general_description);
-	      if (world[room_num].dir_option[counter]->keyword)
-		free(world[room_num].dir_option[counter]->keyword);
-	    }
-	  }
-	  if (world[room_num].ex_description)
-	    for (this = world[room_num].ex_description; this; this = next_one) 
-	      {
-		next_one = this->next;
-		if (this->keyword)
-		  free(this->keyword);
-		if (this->description)
-		  free(this->description);
-		free(this);
-	      }
+	  free_room(&world[room_num]);
 	  /* now copy everything over! */
 	  world[room_num] = *d->edit_room;
+	  if (d->edit_room->tele->targ <= 0) {
+		free(world[room_num].tele);
+		world[room_num].tele = NULL;
+	  }
+          if (!IS_SET(d->edit_room->room_flags, ROOM_BROADCAST)) {
+		free(world[room_num].broad);
+		world[room_num].broad = NULL;
+	  }
 	  /* resolve all vnum doors to rnum doors in the newly edited room */
 	  for (counter = 0; counter < NUM_OF_DIRS; counter++) {
 	    if (world[room_num].dir_option[counter]) {
@@ -397,7 +459,7 @@ void redit_parse(struct descriptor_data * d, char *arg)
 	    }
 	  }
 
-	} else {
+	} else {  /* SHOULDN"T GET HERE... BAD, MEANS WE"RE NOT USING STUBS */
 	  /* hm, we can't just copy.. gotta insert a new room */
 	  int             counter;
 	  int             counter2;
@@ -570,12 +632,151 @@ void redit_parse(struct descriptor_data * d, char *arg)
       d->misc_data = (void **) &(d->edit_room->ex_description);
       redit_disp_extradesc_menu(d);
       break;
+    case 'p':
+    case 'P':
+      if (IS_SET(d->edit_room->room_flags, ROOM_BROADCAST)) {
+	send_to_char("Enter channel number: ", d->character);
+        d->edit_mode = REDIT_BROADCAST_CHANNEL;
+      } else {
+	send_to_char("But this room isn't flagged as a Broadcasting room", d->character);
+      }
+      break;
+    case 'r':
+    case 'R':
+      if (IS_SET(d->edit_room->room_flags, ROOM_BROADCAST)) {
+        send_to_char("Enter target vnum: ", d->character);
+        d->edit_mode = REDIT_BROADCAST_TARG1;
+      } else {
+	send_to_char("But this room isn't flagged as a Broadcasting room", d->character);
+      }
+      break;
+    case 's':
+    case 'S':
+      if (IS_SET(d->edit_room->room_flags, ROOM_BROADCAST)) {
+	send_to_char("Enter target vnum: ", d->character);
+        d->edit_mode = REDIT_BROADCAST_TARG2;
+      } else {
+	send_to_char("But this room isn't flagged as a Broadcasting room", d->character);
+      }
+      break;
+    case 't':
+    case 'T':
+      send_to_char("Enter target vnum: ", d->character);
+      d->edit_mode = REDIT_TELEPORT_TARGET;
+      break;
+    case 'u':
+    case 'U':
+      send_to_char("Enter teleport frequency: ", d->character);
+      d->edit_mode = REDIT_TELEPORT_FREQ;
+      break;
+    case 'v':
+    case 'V':
+      redit_disp_teleport_menu(d);
+      break;
+    case 'w':
+    case 'W':
+      send_to_char("Enter Object's Vnum: ", d->character);
+      d->edit_mode = REDIT_TELEPORT_OBJ;
+      break;
     default:
       send_to_char("Invalid choice!", d->character);
       redit_disp_menu(d);
       break;
     }
     break;
+  case REDIT_BROADCAST_TARG1:
+    if (isdigit(*arg)) {
+	    number = atoi(arg);
+	    if (real_room(number) != -1) {
+		d->edit_room->broad->targ1 = number;
+		redit_disp_menu(d);
+		break;
+            }
+    }
+    send_to_char("That was not a valid room, please try again or enter -1: ", d->character);
+    break;
+  case REDIT_BROADCAST_TARG2:
+    if (isdigit(*arg)) {
+	    number = atoi(arg);
+	    if (real_room(number) != -1) {
+		d->edit_room->broad->targ2 = number;
+		redit_disp_menu(d);
+		break;
+            }
+    }
+    send_to_char("That was not a valid room, please try again or enter -1: ", d->character);
+    break;
+  case REDIT_BROADCAST_CHANNEL:
+    if (isdigit(*arg)) {
+	    number = atoi(arg);
+	    if (number <= 32 && number >= 0) {
+		d->edit_room->broad->channel = number;
+		redit_disp_menu(d);
+		break;
+            }
+    }
+    send_to_char("That was not a valid number, please enter something between 0 and 32: ", d->character);
+    break;	
+  case REDIT_TELEPORT_TARGET:
+    if (!strcmp("-1", arg)) {
+		d->edit_room->tele->targ = -1;
+		redit_disp_menu(d);
+		break;
+    }
+    if (isdigit(*arg)) {
+	    number = atoi(arg);
+	    if (real_room(number) != -1) {
+		d->edit_room->tele->targ = number;
+		redit_disp_menu(d);
+		break;
+            }
+    }
+    send_to_char("That was not a valid room, please try again or enter -1: ", d->character);
+    break;
+  case REDIT_TELEPORT_FREQ:
+    if (isdigit(*arg)) {
+	    number = atoi(arg);
+	    if (number >= 2 && number <= 7200) {
+		d->edit_room->tele->time = number;
+	        redit_disp_menu(d);
+		break;
+   	        }
+            }
+    send_to_char("That was not a valid number, please enter something between 2 and 7200: ", d->character);
+    break;	
+  case REDIT_TELEPORT_OBJ:
+    if (isdigit(*arg)) {
+	    number = atoi(arg);
+	    if (real_object(number) != -1) {
+		d->edit_room->tele->obj = number;
+	        redit_disp_menu(d);
+		break;
+   	        }
+            }
+    send_to_char("That was not a valid number, please enter the object's VNUM: ", d->character);
+    break;	
+  case REDIT_TELEPORT_MENU:
+    if (isdigit(*arg)) {
+	number = atoi(arg);
+	if (number >= 0 && number < TELEPORT_NUM) {
+		if (IS_SET(d->edit_room->tele->mask, 1 << number))
+			REMOVE_BIT(d->edit_room->tele->mask, 1 << number);
+		else
+			SET_BIT(d->edit_room->tele->mask, 1 << number);
+		redit_disp_teleport_menu(d);
+		break;
+	}
+    } 
+    switch (*arg) {
+    case 'q':
+    case 'Q':
+	redit_disp_menu(d);
+        break;
+    default:
+        send_to_char("Invalid Choice... please try again: ", d->character);
+	break;
+    }
+    break;
   case REDIT_NAME:
     if (d->edit_room->name)
       free(d->edit_room->name);
