@@ -17,6 +17,9 @@
 #include <string.h>
 #include <ctype.h>
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "comm.h"
 #include "handler.h"
 #include "db.h"
@@ -43,6 +46,8 @@ ACMD(do_echo);
 ACMD(do_say);
 char *fname(char *namelist);
 void sort_keeper_objs(struct char_data * keeper, int shop_nr);
+int invalid_class(struct char_data * ch, struct obj_data * obj);
+int invalid_race(struct char_data * ch, struct obj_data * obj);
 
 /* Local variables */
 struct shop_data *shop_index;
@@ -407,7 +412,7 @@ void shopping_buy(char *arg, struct char_data * ch,
 		       struct char_data * keeper, int shop_nr)
 {
   char tempstr[200], buf[MAX_STRING_LENGTH];
-  struct obj_data *obj, *last_obj;
+  struct obj_data *obj, *last_obj = NULL;
   int goldamt = 0, buynum, bought = 0;
 
   if (!(is_ok(keeper, ch, shop_nr)))
@@ -463,7 +468,7 @@ void shopping_buy(char *arg, struct char_data * ch,
     bought++;
     /* Test if producing shop ! */
     if (shop_producing(obj, shop_nr))
-      obj = read_object(GET_OBJ_RNUM(obj), REAL);
+      obj = read_object(GET_OBJ_RNUM(obj), REAL, world[ch->in_room].zone);
     else {
       obj_from_char(obj);
       SHOP_SORT(shop_nr)--;
@@ -522,12 +527,29 @@ struct obj_data *get_selling_obj(struct char_data * ch, char *name,
   int result;
 
   if (!(obj = get_obj_in_list_vis(ch, name, ch->carrying))) {
-    if (msg) {
+     if ((obj = get_obj_in_list_vis(ch, name, keeper->carrying))) {
+	 if (invalid_class(ch, obj)) {
+		sprintf(buf, "%s Sorry, your class is not allowed to use that.",
+			GET_NAME(ch));
+		do_tell(keeper, buf, cmd_tell, 0);
+	 } else if (invalid_race(ch, obj)) {
+		sprintf(buf, "%s Sorry, your race is not allowed to use that.",
+			GET_NAME(ch));
+		do_tell(keeper, buf, cmd_tell, 0);
+	 } else {
+		sprintf(buf, "%s You can use that.", GET_NAME(ch));
+		do_tell(keeper, buf, cmd_tell, 0);
+         }
+     } else if (msg) {
       sprintf(buf, shop_index[shop_nr].no_such_item2, GET_NAME(ch));
       do_tell(keeper, buf, cmd_tell, 0);
-    }
+     } else {
+      sprintf(buf, "%s Sorry, I don't see such an item.", GET_NAME(ch));
+      do_tell(keeper, buf, cmd_tell, 0);
+     }
     return (0);
   }
+
   if ((result = trade_with(obj, shop_nr)) == OBJECT_OK)
     return (obj);
 
@@ -705,18 +727,16 @@ void shopping_value(char *arg, struct char_data * ch,
     return;
 
   if (!(*arg)) {
-    sprintf(buf, "%s What do you want me to valuate??", GET_NAME(ch));
+    sprintf(buf, "%s What do you want me to evaluate??", GET_NAME(ch));
     do_tell(keeper, buf, cmd_tell, 0);
     return;
   }
   one_argument(arg, name);
-  if (!(obj = get_selling_obj(ch, name, keeper, shop_nr, TRUE)))
-    return;
-
-  sprintf(buf, "%s I'll give you %d gold coins for that!", GET_NAME(ch),
-	  sell_price(ch, obj, shop_nr));
-  do_tell(keeper, buf, cmd_tell, 0);
-
+  if ((obj = get_selling_obj(ch, name, keeper, shop_nr, TRUE))) {
+	  sprintf(buf, "%s I'll give you %d coins for that!", GET_NAME(ch),
+		  sell_price(ch, obj, shop_nr));
+	  do_tell(keeper, buf, cmd_tell, 0);
+  }
   return;
 }
 
@@ -839,6 +859,7 @@ SPECIAL(shop_keeper)
     sprintf(argm, "$N shouts '%s'", MSG_NO_STEAL_HERE);
     do_action(keeper, GET_NAME(ch), cmd_slap, 0);
     act(argm, FALSE, ch, 0, keeper, TO_CHAR);
+    SET_BIT(PLR_FLAGS(ch), PLR_THIEF);
     return (TRUE);
   }
   if (CMD_IS("buy")) {
