@@ -15,6 +15,9 @@
 #include <sys/types.h>
 
 #include "structs.h"
+#include "class.h"
+#include "rooms.h"
+#include "objs.h"
 #include "utils.h"
 #include "comm.h"
 #include "interpreter.h"
@@ -34,7 +37,7 @@ extern struct command_info cmd_info[];
 
 /* extern functions */
 void add_follower(struct char_data * ch, struct char_data * leader);
-
+void advance_char(struct char_data * ch);
 
 struct social_type {
   char *cmd;
@@ -115,11 +118,13 @@ extern int prac_params[2][NUM_CLASSES];
 #define LEARNED(ch) (prac_params[LEARNED_LEVEL][(int)GET_CLASS_NUM_FULL(ch)])
 #define SPLSKL(ch) (prac_types[prac_params[PRAC_TYPE][(int)GET_CLASS_NUM_FULL(ch)]])
 
+int mag_manacost(struct char_data * ch, int spellnum);
+
 void list_skills(struct char_data * ch)
 {
   extern char *spells[];
   extern struct spell_info_type spell_info[];
-  int i, j, sortpos;
+  int i, j, sortpos, temp;
 
   if (!GET_PRACTICES(ch))
     strcpy(buf, "You have no practice sessions remaining.\r\n");
@@ -139,7 +144,12 @@ void list_skills(struct char_data * ch)
     }
     for (j = 0; j < NUM_CLASSES; j++) {
 	if ( IS_SET(GET_CLASS(ch), (1 << j)) && (GET_LEVEL(ch) >= spell_info[i].min_level[j]) ) {
-  	    sprintf(buf, "%-20s %s\r\n", spells[i], how_good(GET_SKILL(ch, i)));
+  	    sprintf(buf, "%-24s %-10.10s LV: %-2d  CST: ", spells[i], how_good(GET_SKILL(ch, i)), spell_info[i].min_level[j]);
+	    strcat(buf2, buf);
+	    if ((temp = mag_manacost(ch, i)))
+		sprintf(buf,"%d\r\n", temp);
+	    else
+		sprintf(buf,"n/a\r\n");
 	    strcat(buf2, buf);
 	    break;
         }
@@ -178,11 +188,16 @@ int do_guild(struct char_data *ch, void *me, int cmd, char *argument, int skillt
   extern struct spell_info_type spell_info[];
   extern struct int_app_type int_app[36];
 
-  if (IS_NPC(ch) || !CMD_IS("practice"))
+  if (IS_NPC(ch) || !(CMD_IS("practice") || CMD_IS("advance")))
     return 0;
 
   skip_spaces(&argument);
 
+  if (CMD_IS("advance")) {
+    advance_char(ch);
+    return 1;
+  }
+
   if (!*argument) {
     list_skills(ch);
     return 1;
@@ -471,7 +486,7 @@ void npc_steal(struct char_data * ch, struct char_data * victim)
 
   if (AWAKE(victim) && (number(0, GET_LEVEL(ch)) == 0)) {
     act("You discover that $n has $s hands in your wallet.", FALSE, ch, 0, victim, TO_VICT);
-    act("$n tries to steal gold from $N.", TRUE, ch, 0, victim, TO_NOTVICT);
+    act("$n tries to steal some coins from $N.", TRUE, ch, 0, victim, TO_NOTVICT);
   } else {
     /* Steal some gold coins */
     gold = (int) ((GET_GOLD(victim) * number(1, 10)) / 100);
@@ -731,6 +746,20 @@ SPECIAL(guild_guard)
       }
     }
     /* Check Align */
+
+    /* Check Level */
+    if (guild_info[i][5] > GET_LEVEL(ch) &&
+    	   (world[ch->in_room].number == guild_info[i][3]) &&
+	   (cmd == guild_info[i][4]) ) {
+	act(denybuf2, FALSE, ch, 0, guard, TO_CHAR);
+	return TRUE;
+    }
+    if (guild_info[i][6] < GET_LEVEL(ch) &&
+    	   (world[ch->in_room].number == guild_info[i][3]) &&
+	   (cmd == guild_info[i][4]) ) {
+	act(denybuf2, FALSE, ch, 0, guard, TO_CHAR);
+	return TRUE;
+    }
   }
   return FALSE;
 }
@@ -890,7 +919,7 @@ SPECIAL(pet_shops)
       return (TRUE);
     }
     if (GET_GOLD(ch) < (GET_EXP(pet) * 100)) {
-      send_to_char("You don't have enough gold!\r\n", ch);
+      send_to_char("You don't have enough coins!\r\n", ch);
       return (TRUE);
     }
     if (GET_LEVEL(ch) < GET_LEVEL(pet)) {
@@ -899,7 +928,7 @@ SPECIAL(pet_shops)
     }
     GET_GOLD(ch) -= GET_EXP(pet) * 100;
 
-    pet = read_mobile(GET_MOB_RNUM(pet), REAL);
+    pet = read_mobile(GET_MOB_RNUM(pet), REAL, world[pet_room].zone);
     GET_EXP(pet) = 0;
     SET_BIT(AFF_FLAGS(pet), AFF_CHARM);
     SET_BIT(MOB_FLAGS(pet), MOB_PET);
@@ -957,7 +986,7 @@ SPECIAL(engraver)
       return (TRUE);
     }
     if (GET_GOLD(ch) < 100000) {
-      send_to_char("You don't have enough gold!\r\n", ch);
+      send_to_char("You don't have enough coins!\r\n", ch);
       return (TRUE);
     }
     GET_GOLD(ch) -= 100000;
@@ -988,7 +1017,7 @@ SPECIAL(engraver)
       return (TRUE);
     }
     if (GET_GOLD(ch) < 500000) {
-      send_to_char("You don't have enough gold!\r\n", ch);
+      send_to_char("You don't have enough coins!\r\n", ch);
       return (TRUE);
     }
     GET_GOLD(ch) -= 500000;
@@ -1015,11 +1044,11 @@ SPECIAL(war_reg)
 		send_to_char("Yes sir!  But, you are already registered!\r\n", ch);
 		return 1;
 	}
-	if (PRF_FLAGGED(ch, PRF_ASSASSIN)) {
+	if (PRF2_FLAGGED(ch, PRF2_ASSASSIN)) {
 		send_to_char("We do not want your kind here.  GO AWAY!\r\n", ch);
 		return 1;
     	}
-	if (PRF_FLAGGED(ch, PRF_BOUNTYHUNT)) {
+	if (PRF2_FLAGGED(ch, PRF2_BOUNTYHUNT)) {
 		send_to_char("We can not enlist bounty hunters, sorry.\r\n", ch);
 		return 1;
 	}
@@ -1032,11 +1061,11 @@ SPECIAL(war_reg)
 		send_to_char("Yes sir!  But, you are already registered!\r\n", ch);
 		return 1;
 	}
-	if (PRF_FLAGGED(ch, PRF_ASSASSIN)) {
+	if (PRF2_FLAGGED(ch, PRF2_ASSASSIN)) {
 		send_to_char("We do not want your kind here.  GO AWAY!\r\n", ch);
 		return 1;
     	}
-	if (PRF_FLAGGED(ch, PRF_BOUNTYHUNT)) {
+	if (PRF2_FLAGGED(ch, PRF2_BOUNTYHUNT)) {
 		send_to_char("We can not enlist bounty hunters, sorry.\r\n", ch);
 		return 1;
 	}
@@ -1049,11 +1078,11 @@ SPECIAL(war_reg)
 		send_to_char("Yes sir!  But, you are already registered!\r\n", ch);
 		return 1;
 	}
-	if (PRF_FLAGGED(ch, PRF_ASSASSIN)) {
+	if (PRF2_FLAGGED(ch, PRF2_ASSASSIN)) {
 		send_to_char("We do not want your kind here.  GO AWAY!\r\n", ch);
 		return 1;
     	}
-	if (PRF_FLAGGED(ch, PRF_BOUNTYHUNT)) {
+	if (PRF2_FLAGGED(ch, PRF2_BOUNTYHUNT)) {
 		send_to_char("We can not enlist bounty hunters, sorry.\r\n", ch);
 		return 1;
 	}
@@ -1103,7 +1132,7 @@ SPECIAL(recruiter)
 SPECIAL(bounty_reg)
 {
   if (CMD_IS("register")) {
-	if (PRF_FLAGGED(ch, PRF_BOUNTYHUNT)) {
+	if (PRF2_FLAGGED(ch, PRF2_BOUNTYHUNT)) {
 		send_to_char("But, you are already a registered bounty hunter!\r\n", ch);
 		return 1;
  	}
@@ -1111,12 +1140,12 @@ SPECIAL(bounty_reg)
 		send_to_char("Sorry, you can not register to be a bounty hunter.\r\n", ch);
 		return 1;
 	}
-	if (PRF_FLAGGED(ch, PRF_ASSASSIN)) {
+	if (PRF2_FLAGGED(ch, PRF2_ASSASSIN)) {
 		send_to_char("HA! You are more a fool than I once thought.\r\n", ch);
 		return 1;
 	}
 	send_to_char("Welcome, Bounty Hunter.  Now go get some Assassins!\r\n", ch);
-	SET_BIT(PRF_FLAGS(ch), PRF_BOUNTYHUNT);	
+	SET_BIT(PRF2_FLAGS(ch), PRF2_BOUNTYHUNT);	
 	return 1;
   }
   return 0;
@@ -1125,7 +1154,7 @@ SPECIAL(bounty_reg)
 SPECIAL(assass_reg)
 {
   if (CMD_IS("register")) {
-	if (PRF_FLAGGED(ch, PRF_ASSASSIN)) {
+	if (PRF2_FLAGGED(ch, PRF2_ASSASSIN)) {
 		send_to_char("You are already an assassin!\r\n", ch);
 		return 1;
  	}
@@ -1138,7 +1167,7 @@ SPECIAL(assass_reg)
 		return 1;
 	}
 	send_to_char("Welcome to the Assassin's Circle!\r\n", ch);
-	SET_BIT(PRF_FLAGS(ch), PRF_ASSASSIN);
+	SET_BIT(PRF2_FLAGS(ch), PRF2_ASSASSIN);
 	return 1;
   }
   return 0;
