@@ -13,6 +13,9 @@
 #include <string.h>
 
 #include "structs.h"
+#include "class.h"
+#include "objs.h"
+#include "rooms.h"
 #include "utils.h"
 #include "interpreter.h"
 #include "spells.h"
@@ -260,7 +263,6 @@ int mag_manacost(struct char_data * ch, int spellnum)
   mana = MAX(SINFO.mana_max - (SINFO.mana_change *
 		    (GET_LEVEL(ch) - min_level)),
 	     SINFO.mana_min);
-
   return mana;
 }
 
@@ -643,9 +645,6 @@ ACMD(do_cast)
   char *s, *t;
   int mana, spellnum, i, min_lev, target = 0;
 
-  if (IS_NPC(ch))
-    return;
-
   /* get: blank, spell name, target name */
   s = strtok(argument, "'");
 
@@ -758,7 +757,7 @@ ACMD(do_cast)
   /* You throws the dice and you takes your chances.. 101% is total failure */
   if (number(0, 101) > GET_SKILL(ch, spellnum)) {
     WAIT_STATE(ch, PULSE_VIOLENCE);
-    if (!tch || !skill_message(0, ch, tch, spellnum))
+    if (!tch || !skill_message(0, ch, tch, spellnum, 0))
       send_to_char("You lost your concentration!\r\n", ch);
     if (mana > 0)
       GET_MANA(ch) = MAX(0, MIN(GET_MAX_MANA(ch), GET_MANA(ch) - (mana >> 1)));
@@ -1214,7 +1213,7 @@ void mag_assign_spells(void)
 	 POS_STANDING, TAR_OBJ_ROOM, FALSE, MAG_SUMMONS);
 
   spello(SPELL_CLONE, 29, X, X, X, X, X, X, X, X, X, 27, X, X,
-	 SKILL_TYPE_MAGIC, 200, 150, 5,
+	 SKILL_TYPE_MAGIC, 100, 100, 0,
 	 POS_STANDING, TAR_CHAR_ROOM | TAR_SELF_ONLY, FALSE, MAG_SUMMONS);
 
   spello(SPELL_DEATHDANCE, 21, X, X, 21, 22, 22, X, 23, 14, 14, 20, X, X,
@@ -1226,15 +1225,15 @@ void mag_assign_spells(void)
 	 POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS);
 
   spello(SPELL_ENH_HEAL, X, 25, X, X, X, X, 25, X, 20, 15, X, X, X,
-	 SKILL_TYPE_E_MAGIC, 0, 0, 0,
+	 SKILL_TYPE_E_MAGIC, 60, 30, 3,
          0, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS);
 
   spello(SPELL_ENH_MANA, 25, X, X, X, X, X, X, 25, X, 15, X, X, X,
-	 SKILL_TYPE_E_MAGIC, 0, 0, 0,
+	 SKILL_TYPE_E_MAGIC, 60, 30, 3,
          0, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS);
 
   spello(SPELL_ENH_MOVE, 25, 25, X, X, X, X, 25, 25, 20, 15, X, X, X,
-	 SKILL_TYPE_E_MAGIC, 0, 0, 0,
+	 SKILL_TYPE_E_MAGIC, 60, 30, 3,
          0, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS);
 
   /*
@@ -1302,7 +1301,7 @@ void mag_assign_spells(void)
 	 SKILL_TYPE_FIGHTER,
 	 0, 0, 0, 0, 0, 0, 0);
 
-  spello(SKILL_DUAL_WIELD, X, X, X, X, 1, 10, X, 20, X, X, X, X, X,
+  spello(SKILL_DUAL_WIELD, X, X, X, 8, 10, 3, X, 20, X, X, X, X, X,
 	 SKILL_TYPE_MUNDANE,
 	 0, 0, 0, 0, 0, 0, 0);
 
