@@ -25,9 +25,16 @@ int	number(int from, int to);
 int	dice(int number, int size);
 void	sprintbit(long vektor, char *names[], char *result);
 void	sprinttype(int type, char *names[], char *result);
+char *  sprintbitascii(long vektor, char *result);
 int	get_line(FILE *fl, char *buf);
+int 	get_line2(FILE * fl, char *buf);
 int	get_filename(char *orig_name, char *filename, int mode);
 struct time_info_data age(struct char_data *ch);
+int	GET_REINCARN(struct char_data *ch);
+void	ADD_REINCARN(struct char_data *ch);
+bool	CAN_DAMAGE(struct char_data *ch, struct char_data *vic);
+bool	CAN_MURDER(struct char_data *ch, struct char_data *victim);
+int	get_alignment_type(int temp);
 
 /* undefine MAX and MIN so that our functions are used instead */
 #ifdef MAX
@@ -258,7 +265,6 @@ void	update_pos(struct char_data *victim);
 #define GET_WEIGHT(ch)	((ch)->player.weight)
 #define GET_SEX(ch)	((ch)->player.sex)
 
-
 #define GET_STR(ch)     ((ch)->aff_abils.str)
 #define GET_ADD(ch)     ((ch)->aff_abils.str_add)
 #define GET_DEX(ch)     ((ch)->aff_abils.dex)
@@ -288,6 +294,9 @@ void	update_pos(struct char_data *victim);
 #define HUNTING(ch)	  ((ch)->char_specials.hunting)
 #define GET_SAVE(ch, i)	  ((ch)->char_specials.saved.apply_saving_throw[i])
 #define GET_ALIGNMENT(ch) ((ch)->char_specials.saved.alignment)
+#define GET_HONOR(ch) 	  ((ch)->player_specials->saved.honor)
+#define GET_CHANNEL(ch)	  ((ch)->player_specials->saved.channel)
+#define GET_CLAN(ch)	  ((ch)->player_specials->saved.clan)
 
 #define GET_COND(ch, i)		((ch)->player_specials->saved.conditions[(i)])
 #define GET_LOADROOM(ch)	((ch)->player_specials->saved.load_room)
@@ -476,6 +485,12 @@ void	update_pos(struct char_data *victim);
 #define IS_PIXIE(ch)		(!IS_NPC(ch) && \
 				(GET_RACE(ch) == RACE_PIXIE))
 
+#define IS_WEREFORM(ch)		(!IS_NPC(ch) && \
+				(GET_RACE(ch) == RACE_WEREFORM))
+
+#define IS_DRAGON(ch)		(!IS_NPC(ch) && \
+				(GET_RACE(ch) == RACE_DRAGON))
+
 #define IS_DARKGNOME(ch)	(IS_GNOME(ch) && PRF2_FLAGGED(ch, PRF2_WAR_DRUHARI))
 
 #define IS_GOODGNOME(ch)	(IS_GNOME(ch) && !PRF2_FLAGGED(ch, PRF2_WAR_DRUHARI))
@@ -541,7 +556,8 @@ void	update_pos(struct char_data *victim);
 
 #define EXIT(ch, door)  (world[(ch)->in_room].dir_option[door])
 #define EXITN(room, door)               (world[room].dir_option[door])
-
+#define _2ND_EXIT(ch, door) (world[EXIT(ch, door)->to_room].dir_option[door])
+#define _3RD_EXIT(ch, door) (world[_2ND_EXIT(ch, door)->to_room].dir_option[door])
 
 #define CAN_GO(ch, door) (EXIT(ch,door) && \
 			 (EXIT(ch,door)->to_room != NOWHERE) && \
